@@ -22,7 +22,9 @@
 #endif
 
 #include <math.h>
-#include <libswfdec-gtk/swfdec_gtk_widget.h>
+#include <gdk/gdkkeysyms.h>
+#include "swfdec_gtk_widget.h"
+#include "swfdec_gtk_keys.h"
 
 struct _SwfdecGtkWidgetPrivate
 {
@@ -78,7 +80,7 @@ swfdec_gtk_widget_motion_notify (GtkWidget *gtkwidget, GdkEventMotion *event)
 
   gdk_window_get_pointer (gtkwidget->window, &x, &y, NULL);
 
-  if (priv->interactive)
+  if (priv->interactive && priv->player)
     swfdec_player_handle_mouse (priv->player, x, y, priv->button);
   
   return FALSE;
@@ -90,7 +92,7 @@ swfdec_gtk_widget_leave_notify (GtkWidget *gtkwidget, GdkEventCrossing *event)
   SwfdecGtkWidget *widget = SWFDEC_GTK_WIDGET (gtkwidget);
   SwfdecGtkWidgetPrivate *priv = widget->priv;
 
-  if (priv->interactive) {
+  if (priv->interactive && priv->player) {
     priv->button = 0;
     swfdec_player_handle_mouse (priv->player, event->x, event->y, 0);
   }
@@ -105,7 +107,7 @@ swfdec_gtk_widget_button_press (GtkWidget *gtkwidget, GdkEventButton *event)
 
   if (event->button == 1) {
     priv->button = 1;
-    if (priv->interactive)
+    if (priv->interactive && priv->player)
       swfdec_player_handle_mouse (priv->player, event->x, event->y, 1);
   }
   return FALSE;
@@ -119,23 +121,83 @@ swfdec_gtk_widget_button_release (GtkWidget *gtkwidget, GdkEventButton *event)
 
   if (event->button == 1) {
     priv->button = 0;
-    if (priv->interactive)
+    if (priv->interactive && priv->player)
       swfdec_player_handle_mouse (priv->player, event->x, event->y, 0);
   }
   return FALSE;
 }
 
+static guint
+swfdec_gtk_event_to_keycode (GdkEventKey *event)
+{
+  guint ret;
+
+  /* we try to match as well as possible to Flash _Windows_ key codes.
+   * Since a lot of Flash files won't special case weird Linux key codes and we
+   * want the best compatibility possible, we have to do that.
+   */
+  /* FIXME: I have no clue about non-western keyboards, so if you happen to use
+   * such a keyboard, please help out here if it doesn't match.
+   */
+
+  /* try to match latin keys directly */
+  if (event->keyval >= GDK_A && event->keyval <= GDK_Z)
+    return event->keyval - GDK_A + SWFDEC_KEY_A;
+  if (event->keyval >= GDK_a && event->keyval <= GDK_z)
+    return event->keyval - GDK_a + SWFDEC_KEY_A;
+
+  /* last resort: try to translate the hardware keycode directly */
+  ret = swfdec_gtk_keycode_from_hardware_keycode (event->hardware_keycode);
+  if (ret == 0)
+    g_printerr ("could not translate key to Flash keycode. HW keycode %u, keyval %u\n",
+	event->hardware_keycode, event->keyval);
+  return ret;
+}
+
+static gboolean
+swfdec_gtk_widget_key_press (GtkWidget *gtkwidget, GdkEventKey *event)
+{
+  SwfdecGtkWidget *widget = SWFDEC_GTK_WIDGET (gtkwidget);
+  SwfdecGtkWidgetPrivate *priv = widget->priv;
+
+  if (priv->interactive && priv->player) {
+    guint keycode = swfdec_gtk_event_to_keycode (event);
+    if (keycode != 0) {
+      swfdec_player_key_press (priv->player, keycode, 
+	  gdk_keyval_to_unicode (event->keyval));
+    }
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+static gboolean
+swfdec_gtk_widget_key_release (GtkWidget *gtkwidget, GdkEventKey *event)
+{
+  SwfdecGtkWidget *widget = SWFDEC_GTK_WIDGET (gtkwidget);
+  SwfdecGtkWidgetPrivate *priv = widget->priv;
+
+  if (priv->interactive && priv->player) {
+    guint keycode = swfdec_gtk_event_to_keycode (event);
+    if (keycode != 0) {
+      swfdec_player_key_release (priv->player, keycode, 
+	  gdk_keyval_to_unicode (event->keyval));
+    }
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 static cairo_surface_t *
 swfdec_gtk_widget_create_renderer (cairo_surface_type_t type, int width, int height)
 {
-  switch (type) {
-    case CAIRO_SURFACE_TYPE_IMAGE:
-      return cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
-    default:
-      break;
+  if (type == CAIRO_SURFACE_TYPE_IMAGE) {
+    return cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
+  } else {
+    return NULL;
   }
-
-  return NULL;
 }
 
 static gboolean
@@ -148,6 +210,8 @@ swfdec_gtk_widget_expose (GtkWidget *gtkwidget, GdkEventExpose *event)
 
   if (event->window != gtkwidget->window)
     return FALSE;
+  if (priv->player == NULL)
+    return FALSE;
 
   if (!priv->renderer_set ||
       (surface = swfdec_gtk_widget_create_renderer (priv->renderer, 
@@ -246,7 +310,8 @@ swfdec_gtk_widget_size_allocate (GtkWidget *gtkwidget, GtkAllocation *allocation
 
   gtkwidget->allocation = *allocation;
 
-  swfdec_player_set_size (priv->player, allocation->width, allocation->height);
+  if (priv->player && swfdec_player_is_initialized (priv->player))
+    swfdec_player_set_size (priv->player, allocation->width, allocation->height);
   if (GTK_WIDGET_REALIZED (gtkwidget)) {
     gdk_window_move_resize (gtkwidget->window, 
 	allocation->x, allocation->y, allocation->width, allocation->height);
@@ -262,8 +327,11 @@ swfdec_gtk_widget_size_request (GtkWidget *gtkwidget, GtkRequisition *req)
   if (priv->player == NULL) {
     req->width = req->height = 0;
   } else {
-    swfdec_player_get_image_size (priv->player, 
-	  &req->width, &req->height);
+    guint w, h;
+    swfdec_player_get_default_size (priv->player, &w, &h);
+    /* FIXME: set some sane upper limit here? */
+    req->width = MIN (w, G_MAXINT);
+    req->height = MIN (h, G_MAXINT);
   } 
 }
 
@@ -341,7 +409,9 @@ swfdec_gtk_widget_realize (GtkWidget *widget)
 			   GDK_BUTTON_RELEASE_MASK |
 			   GDK_LEAVE_NOTIFY_MASK | 
 			   GDK_POINTER_MOTION_MASK | 
-			   GDK_POINTER_MOTION_HINT_MASK;
+			   GDK_POINTER_MOTION_HINT_MASK |
+			   GDK_KEY_PRESS_MASK |
+			   GDK_KEY_RELEASE_MASK;
 
   attributes_mask = GDK_WA_X | GDK_WA_Y;
 
@@ -388,6 +458,8 @@ swfdec_gtk_widget_class_init (SwfdecGtkWidgetClass * g_class)
   widget_class->button_release_event = swfdec_gtk_widget_button_release;
   widget_class->motion_notify_event = swfdec_gtk_widget_motion_notify;
   widget_class->leave_notify_event = swfdec_gtk_widget_leave_notify;
+  widget_class->key_press_event = swfdec_gtk_widget_key_press;
+  widget_class->key_release_event = swfdec_gtk_widget_key_release;
 
   g_type_class_add_private (object_class, sizeof (SwfdecGtkWidgetPrivate));
 }
@@ -401,21 +473,26 @@ swfdec_gtk_widget_init (SwfdecGtkWidget * widget)
 
   priv->interactive = TRUE;
   priv->renderer = CAIRO_SURFACE_TYPE_IMAGE;
+
+  GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
 }
 
 static void
-swfdec_gtk_widget_invalidate_cb (SwfdecPlayer *player, double x, double y, 
-    double width, double height, SwfdecGtkWidget *widget)
+swfdec_gtk_widget_invalidate_cb (SwfdecPlayer *player, const SwfdecRectangle *extents,
+    const SwfdecRectangle *rect, guint n_rects, SwfdecGtkWidget *widget)
 {
-  GdkRectangle rect;
+  GdkRegion *region;
+  guint i;
 
   if (!GTK_WIDGET_REALIZED (widget))
     return;
-  rect.x = floor (x);
-  rect.y = floor (y);
-  rect.width = ceil (x + width) - rect.x;
-  rect.height = ceil (y + height) - rect.y;
-  gdk_window_invalidate_rect (GTK_WIDGET (widget)->window, &rect, FALSE);
+
+  region = gdk_region_new ();
+  for (i = 0; i < n_rects; i++) {
+    gdk_region_union_with_rect (region, (GdkRectangle *) &rect[i]);
+  }
+  gdk_window_invalidate_region (GTK_WIDGET (widget)->window, region, FALSE);
+  gdk_region_destroy (region);
 }
 
 static void
