@@ -23,22 +23,21 @@
 
 #include <math.h>
 #include <string.h>
+#include <stdlib.h>
 #include "swfdec_as_context.h"
 #include "swfdec_as_array.h"
-#include "swfdec_as_boolean.h"
 #include "swfdec_as_frame_internal.h"
 #include "swfdec_as_function.h"
+#include "swfdec_as_initialize.h"
 #include "swfdec_as_internal.h"
 #include "swfdec_as_interpret.h"
-#include "swfdec_as_math.h"
 #include "swfdec_as_native_function.h"
-#include "swfdec_as_number.h"
 #include "swfdec_as_object.h"
 #include "swfdec_as_stack.h"
-#include "swfdec_as_string.h"
 #include "swfdec_as_strings.h"
 #include "swfdec_as_types.h"
 #include "swfdec_debug.h"
+#include "swfdec_internal.h" /* for swfdec_player_preinit_global() */
 #include "swfdec_script.h"
 
 /*** GARBAGE COLLECTION DOCS ***/
@@ -169,8 +168,11 @@ swfdec_as_context_abort (SwfdecAsContext *context, const char *reason)
 {
   g_return_if_fail (context);
 
-  SWFDEC_ERROR (reason);
-  context->state = SWFDEC_AS_CONTEXT_ABORTED;
+  SWFDEC_ERROR ("%s", reason);
+  if (context->state != SWFDEC_AS_CONTEXT_ABORTED) {
+    context->state = SWFDEC_AS_CONTEXT_ABORTED;
+    g_object_notify (G_OBJECT (context), "aborted");
+  }
 }
 
 /*** MEMORY MANAGEMENT ***/
@@ -198,6 +200,8 @@ swfdec_as_context_use_mem (SwfdecAsContext *context, gsize bytes)
   
   context->memory += bytes;
   context->memory_since_gc += bytes;
+  SWFDEC_LOG ("+%4"G_GSIZE_FORMAT" bytes, total %7"G_GSIZE_FORMAT" (%7"G_GSIZE_FORMAT" since GC)",
+      bytes, context->memory, context->memory_since_gc);
   /* FIXME: Don't foget to abort on OOM */
   return TRUE;
 }
@@ -218,6 +222,8 @@ swfdec_as_context_unuse_mem (SwfdecAsContext *context, gsize bytes)
   g_return_if_fail (context->memory >= bytes);
 
   context->memory -= bytes;
+  SWFDEC_LOG ("-%4"G_GSIZE_FORMAT" bytes, total %7"G_GSIZE_FORMAT" (%7"G_GSIZE_FORMAT" since GC)",
+      bytes, context->memory, context->memory_since_gc);
 }
 
 /*** GC ***/
@@ -248,7 +254,7 @@ swfdec_as_context_remove_strings (gpointer key, gpointer value, gpointer data)
 }
 
 static gboolean
-swfdec_as_context_remove_objects (gpointer key, gpointer value, gpointer data)
+swfdec_as_context_remove_objects (gpointer key, gpointer value, gpointer debugger)
 {
   SwfdecAsObject *object;
 
@@ -261,6 +267,11 @@ swfdec_as_context_remove_objects (gpointer key, gpointer value, gpointer data)
     return FALSE;
   } else {
     SWFDEC_LOG ("deleted: %s %p", G_OBJECT_TYPE_NAME (object), object);
+    if (debugger) {
+      SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (debugger);
+      if (klass->remove)
+	klass->remove (debugger, object->context, object);
+    }
     swfdec_as_object_collect (object);
     return TRUE;
   }
@@ -274,7 +285,7 @@ swfdec_as_context_collect (SwfdecAsContext *context)
   g_hash_table_foreach_remove (context->strings, 
     swfdec_as_context_remove_strings, context);
   g_hash_table_foreach_remove (context->objects, 
-    swfdec_as_context_remove_objects, context);
+    swfdec_as_context_remove_objects, context->debugger);
   SWFDEC_INFO (">> done collecting garbage");
 }
 
@@ -290,6 +301,8 @@ swfdec_as_object_mark (SwfdecAsObject *object)
 {
   SwfdecAsObjectClass *klass;
 
+  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
+
   if (object->flags & SWFDEC_AS_GC_MARK)
     return;
   object->flags |= SWFDEC_AS_GC_MARK;
@@ -308,7 +321,11 @@ swfdec_as_object_mark (SwfdecAsObject *object)
 void
 swfdec_as_string_mark (const char *string)
 {
-  char *str = (char *) string - 1;
+  char *str;
+
+  g_return_if_fail (string != NULL);
+
+  str = (char *) string - 1;
   if (*str == 0)
     *str = SWFDEC_AS_GC_MARK;
 }
@@ -346,12 +363,11 @@ static void
 swfdec_as_context_do_mark (SwfdecAsContext *context)
 {
   swfdec_as_object_mark (context->global);
+  swfdec_as_value_mark (&context->exception_value);
   swfdec_as_object_mark (context->Function);
-  if (context->Function_prototype)
-    swfdec_as_object_mark (context->Function_prototype);
+  swfdec_as_object_mark (context->Function_prototype);
   swfdec_as_object_mark (context->Object);
   swfdec_as_object_mark (context->Object_prototype);
-  swfdec_as_object_mark (context->Array);
   g_hash_table_foreach (context->objects, swfdec_as_context_mark_roots, NULL);
 }
 
@@ -371,7 +387,7 @@ swfdec_as_context_gc (SwfdecAsContext *context)
 
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
   g_return_if_fail (context->frame == NULL);
-  g_return_if_fail (context->state != SWFDEC_AS_CONTEXT_NEW);
+  g_return_if_fail (context->state == SWFDEC_AS_CONTEXT_RUNNING);
 
   if (context->state == SWFDEC_AS_CONTEXT_ABORTED)
     return;
@@ -402,8 +418,7 @@ void
 swfdec_as_context_maybe_gc (SwfdecAsContext *context)
 {
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
-  if (context->state == SWFDEC_AS_CONTEXT_ABORTED)
-    return;
+  g_return_if_fail (context->state == SWFDEC_AS_CONTEXT_RUNNING);
   g_return_if_fail (context->frame == NULL);
 
   if (swfdec_as_context_needs_gc (context))
@@ -417,9 +432,61 @@ enum {
   LAST_SIGNAL
 };
 
+enum {
+  PROP_0,
+  PROP_DEBUGGER,
+  PROP_RANDOM_SEED,
+  PROP_ABORTED,
+  PROP_UNTIL_GC
+};
+
 G_DEFINE_TYPE (SwfdecAsContext, swfdec_as_context, G_TYPE_OBJECT)
 static guint signals[LAST_SIGNAL] = { 0, };
 
+static void
+swfdec_as_context_get_property (GObject *object, guint param_id, GValue *value, 
+    GParamSpec * pspec)
+{
+  SwfdecAsContext *context = SWFDEC_AS_CONTEXT (object);
+
+  switch (param_id) {
+    case PROP_DEBUGGER:
+      g_value_set_object (value, context->debugger);
+      break;
+    case PROP_ABORTED:
+      g_value_set_boolean (value, context->state == SWFDEC_AS_CONTEXT_ABORTED);
+      break;
+    case PROP_UNTIL_GC:
+      g_value_set_ulong (value, (gulong) context->memory_until_gc);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+      break;
+  }
+}
+
+static void
+swfdec_as_context_set_property (GObject *object, guint param_id, const GValue *value, 
+    GParamSpec * pspec)
+{
+  SwfdecAsContext *context = SWFDEC_AS_CONTEXT (object);
+
+  switch (param_id) {
+    case PROP_DEBUGGER:
+      context->debugger = SWFDEC_AS_DEBUGGER (g_value_dup_object (value));
+      break;
+    case PROP_RANDOM_SEED:
+      g_rand_set_seed (context->rand, g_value_get_uint (value));
+      break;
+    case PROP_UNTIL_GC:
+      context->memory_until_gc = g_value_get_ulong (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+      break;
+  }
+}
+
 static void
 swfdec_as_context_dispose (GObject *object)
 {
@@ -435,6 +502,10 @@ swfdec_as_context_dispose (GObject *object)
   g_hash_table_destroy (context->objects);
   g_hash_table_destroy (context->strings);
   g_rand_free (context->rand);
+  if (context->debugger) {
+    g_object_unref (context->debugger);
+    context->debugger = NULL;
+  }
 
   G_OBJECT_CLASS (swfdec_as_context_parent_class)->dispose (object);
 }
@@ -445,6 +516,23 @@ swfdec_as_context_class_init (SwfdecAsContextClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->dispose = swfdec_as_context_dispose;
+  object_class->get_property = swfdec_as_context_get_property;
+  object_class->set_property = swfdec_as_context_set_property;
+
+  g_object_class_install_property (object_class, PROP_DEBUGGER,
+      g_param_spec_object ("debugger", "debugger", "debugger used in this player",
+	  SWFDEC_TYPE_AS_DEBUGGER, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  g_object_class_install_property (object_class, PROP_RANDOM_SEED,
+      g_param_spec_uint ("random-seed", "random seed", 
+	  "seed used for calculating random numbers",
+	  0, G_MAXUINT32, 0, G_PARAM_WRITABLE)); /* FIXME: make this readwrite for replaying? */
+  g_object_class_install_property (object_class, PROP_ABORTED,
+      g_param_spec_boolean ("aborted", "aborted", "set when the script engine aborts due to an error",
+	FALSE, G_PARAM_READABLE));
+  g_object_class_install_property (object_class, PROP_UNTIL_GC,
+      g_param_spec_ulong ("memory-until-gc", "memory until gc", 
+	  "amount of bytes that need to be allocated before garbage collection triggers",
+	  0, G_MAXULONG, 8 * 1024 * 1024, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
   /**
    * SwfdecAsContext::trace:
@@ -466,7 +554,6 @@ swfdec_as_context_init (SwfdecAsContext *context)
 {
   const char *s;
 
-  context->memory_until_gc = 8 * 1024 * 1024; /* 8 MB before we run the GC */
   context->strings = g_hash_table_new (g_str_hash, g_str_equal);
   context->objects = g_hash_table_new (g_direct_hash, g_direct_equal);
 
@@ -593,6 +680,52 @@ swfdec_as_context_get_frame (SwfdecAsContext *context)
   return context->frame;
 }
 
+/**
+ * swfdec_as_context_throw:
+ * @context: a #SwfdecAsContext
+ * @value: a #SwfdecAsValue to be thrown
+ *
+ * Throws a new exception in the @context using the given @value. This function
+ * can only be called if the @context is not already throwing an exception.
+ **/
+void
+swfdec_as_context_throw (SwfdecAsContext *context, const SwfdecAsValue *value)
+{
+  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
+  g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
+  g_return_if_fail (!context->exception);
+
+  context->exception = TRUE;
+  context->exception_value = *value;
+}
+
+/**
+ * swfdec_as_context_catch:
+ * @context: a #SwfdecAsContext
+ * @value: a #SwfdecAsValue to be thrown
+ *
+ * Removes the currently thrown exception from @context and sets @value to the
+ * thrown value
+ *
+ * Returns: %TRUE if an exception was catched, %FALSE otherwise
+ **/
+gboolean
+swfdec_as_context_catch (SwfdecAsContext *context, SwfdecAsValue *value)
+{
+  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), FALSE);
+
+  if (!context->exception)
+    return FALSE;
+
+  if (value != NULL)
+    *value = context->exception_value;
+
+  context->exception = FALSE;
+  SWFDEC_AS_VALUE_SET_UNDEFINED (&context->exception_value);
+
+  return TRUE;
+}
+
 /**
  * swfdec_as_context_get_time:
  * @context: a #SwfdecAsContext
@@ -635,27 +768,33 @@ swfdec_as_context_run (SwfdecAsContext *context)
   SwfdecAsFrame *frame, *last_frame;
   SwfdecScript *script;
   const SwfdecActionSpec *spec;
-  guint8 *startpc, *pc, *endpc, *nextpc;
+  const guint8 *startpc, *pc, *endpc, *nextpc, *exitpc;
 #ifndef G_DISABLE_ASSERT
   SwfdecAsValue *check;
 #endif
   guint action, len;
-  guint8 *data;
-  int version;
-  SwfdecAsContextClass *klass;
-  void (* step) (SwfdecAsContext *context);
-  gboolean check_scope; /* some opcodes avoid a scope check */
+  const guint8 *data;
+  guint original_version;
+  void (* step) (SwfdecAsDebugger *debugger, SwfdecAsContext *context);
+  gboolean check_block; /* some opcodes avoid a scope check */
 
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
   if (context->frame == NULL || context->state == SWFDEC_AS_CONTEXT_ABORTED)
     return;
 
-  klass = SWFDEC_AS_CONTEXT_GET_CLASS (context);
-  step = klass->step;
+  if (context->debugger) {
+    SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
+    step = klass->step;
+  } else {
+    step = NULL;
+  }
 
   last_frame = context->last_frame;
   context->last_frame = context->frame->next;
+  original_version = context->version;
 start:
+  if (!swfdec_as_context_check_continue (context))
+    goto error;
   /* setup data */
   frame = context->frame;
   if (frame == context->last_frame)
@@ -665,6 +804,12 @@ start:
     swfdec_as_context_abort (context, "Stack overflow");
     return;
   }
+  /* if security is NULL, the function may not be called */
+  if (frame->security == NULL) {
+    SWFDEC_WARNING ("insufficient right to call %s", frame->function_name);
+    swfdec_as_frame_return (frame, NULL);
+    goto start;
+  }
   if (SWFDEC_IS_AS_NATIVE_FUNCTION (frame->function)) {
     SwfdecAsNativeFunction *native = SWFDEC_AS_NATIVE_FUNCTION (frame->function);
     SwfdecAsValue rval = { 0, };
@@ -679,17 +824,16 @@ start:
       } else {
 	SwfdecAsStack *stack;
 	SwfdecAsValue *cur;
-	guint i, n;
+	guint i;
 	if (frame->argc > 128) {
-	  SWFDEC_FIXME ("allow calling native functions with more than 128 args");
-	  n = 128;
-	} else {
-	  n = frame->argc;
+	  SWFDEC_FIXME ("allow calling native functions with more than 128 args (this one has %u)",
+	      frame->argc);
+	  frame->argc = 128;
 	}
-	argv = g_new (SwfdecAsValue, n);
+	argv = g_new (SwfdecAsValue, frame->argc);
 	stack = context->stack;
 	cur = context->cur;
-	for (i = 0; i < n; i++) {
+	for (i = 0; i < frame->argc; i++) {
 	  if (cur <= &stack->elements[0]) {
 	    stack = stack->next;
 	    cur = &stack->elements[stack->used_elements];
@@ -709,15 +853,22 @@ start:
   g_assert (frame->script);
   g_assert (frame->target);
   script = frame->script;
-  version = SWFDEC_AS_EXTRACT_SCRIPT_VERSION (script->version);
   context->version = script->version;
   startpc = script->buffer->data;
   endpc = startpc + script->buffer->length;
+  exitpc = script->exit;
   pc = frame->pc;
-  check_scope = TRUE;
+  check_block = TRUE;
 
   while (context->state < SWFDEC_AS_CONTEXT_ABORTED) {
-    if (pc == endpc) {
+    if (context->exception) {
+      swfdec_as_frame_handle_exception (frame);
+      if (frame != context->frame)
+	goto start;
+      pc = frame->pc;
+      continue;
+    }
+    if (pc == exitpc) {
       swfdec_as_frame_return (frame, NULL);
       goto start;
     }
@@ -725,8 +876,13 @@ start:
       SWFDEC_ERROR ("pc %p not in valid range [%p, %p) anymore", pc, startpc, endpc);
       goto error;
     }
-    if (check_scope)
-      swfdec_as_frame_check_scope (frame);
+    if (check_block && (pc < frame->block_start || pc >= frame->block_end)) {
+      SWFDEC_LOG ("code exited block");
+      swfdec_as_frame_pop_block (frame);
+      pc = frame->pc;
+      if (frame != context->frame)
+	goto start;
+    }
 
     /* decode next action */
     action = *pc;
@@ -737,7 +893,7 @@ start:
     /* invoke debugger if there is one */
     if (step) {
       frame->pc = pc;
-      (* step) (context);
+      (* step) (context->debugger, context);
       if (frame != context->frame || 
 	  frame->pc != pc) {
 	goto start;
@@ -763,13 +919,17 @@ start:
       nextpc = pc + 1;
     }
     /* check action is valid */
-    if (spec->exec[version] == NULL) {
+    if (!spec->exec) {
       SWFDEC_WARNING ("cannot interpret action %3u 0x%02X %s for version %u, skipping it", action,
 	  action, spec->name ? spec->name : "Unknown", script->version);
       frame->pc = pc = nextpc;
-      check_scope = TRUE;
+      check_block = TRUE;
       continue;
     }
+    if (script->version < spec->version) {
+      SWFDEC_WARNING ("cannot interpret action %3u 0x%02X %s for version %u, using version %u instead",
+	  action, action, spec->name ? spec->name : "Unknown", script->version, spec->version);
+    }
     if (spec->remove > 0) {
       if (spec->add > spec->remove)
 	swfdec_as_stack_ensure_free (context, spec->add - spec->remove);
@@ -786,15 +946,19 @@ start:
     check = (spec->add >= 0 && spec->remove >= 0) ? context->cur + spec->add - spec->remove : NULL;
 #endif
     /* execute action */
-    spec->exec[version] (context, action, data, len);
+    spec->exec (context, action, data, len);
     /* adapt the pc if the action did not, otherwise, leave it alone */
     /* FIXME: do this via flag? */
     if (frame->pc == pc) {
       frame->pc = pc = nextpc;
-      check_scope = TRUE;
+      check_block = TRUE;
     } else {
+      if (frame->pc < pc &&
+	  !swfdec_as_context_check_continue (context)) {
+	goto error;
+      }
       pc = frame->pc;
-      check_scope = FALSE;
+      check_block = FALSE;
     }
     if (frame == context->frame) {
 #ifndef G_DISABLE_ASSERT
@@ -815,6 +979,7 @@ error:
     swfdec_as_frame_return (context->frame, NULL);
 out:
   context->last_frame = last_frame;
+  context->version = original_version;
   return;
 }
 
@@ -873,15 +1038,11 @@ swfdec_as_context_eval_get_property (SwfdecAsContext *cx,
     swfdec_as_object_get_variable (obj, name, ret);
   } else {
     if (cx->frame) {
-      obj = swfdec_as_frame_find_variable (cx->frame, name);
-      if (obj) {
-	swfdec_as_object_get_variable (obj, name, ret);
-	return;
-      }
+      swfdec_as_frame_get_variable (cx->frame, name, ret);
     } else {
-      SWFDEC_ERROR ("no frame in eval?");
+      SWFDEC_WARNING ("eval called without a frame");
+      swfdec_as_object_get_variable (cx->global, name, ret);
     }
-    SWFDEC_AS_VALUE_SET_UNDEFINED (ret);
   }
 }
 
@@ -894,11 +1055,10 @@ swfdec_as_context_eval_set_property (SwfdecAsContext *cx,
       SWFDEC_ERROR ("no frame in eval_set?");
       return;
     }
-    obj = swfdec_as_frame_find_variable (cx->frame, name);
-    if (obj == NULL || obj == cx->global)
-      obj = cx->frame->target;
+    swfdec_as_frame_set_variable (cx->frame, name, ret);
+  } else {
+    swfdec_as_object_set_variable (obj, name, ret);
   }
-  swfdec_as_object_set_variable (obj, name, ret);
 }
 
 static void
@@ -939,11 +1099,14 @@ swfdec_as_context_eval_internal (SwfdecAsContext *cx, SwfdecAsObject *obj, const
       goto finish;
     }
   }
-  if (obj == NULL && cx->frame) {
-    swfdec_as_object_get_variable (SWFDEC_AS_OBJECT (cx->frame), SWFDEC_AS_STR_this, &cur);
-  } else {
-    SWFDEC_AS_VALUE_SET_OBJECT (&cur, obj);
+  if (obj == NULL) {
+    if (cx->frame)
+      obj = cx->frame->target;
+    else
+      obj = cx->global;
   }
+  g_assert (obj != NULL);
+  SWFDEC_AS_VALUE_SET_OBJECT (&cur, obj);
 
 finish:
   g_strfreev (varlist);
@@ -1000,26 +1163,21 @@ swfdec_as_context_eval_set (SwfdecAsContext *context, SwfdecAsObject *obj, const
 /*** AS CODE ***/
 
 static void
-swfdec_as_context_ASSetPropFlags_set_one_flag (SwfdecAsObject *object, const char *s, guint *flags)
+swfdec_as_context_ASSetPropFlags_set_one_flag (SwfdecAsObject *object,
+    const char *s, guint *flags)
 {
-  guint real;
-
-  /* first set all relevant flags */
-  real = flags[0] & flags[1];
-  swfdec_as_object_set_variable_flags (object, s, real);
-  /* then unset all relevant flags */
-  real = ~flags[0] & flags[1];
-  swfdec_as_object_unset_variable_flags (object, s, real);
+  swfdec_as_object_unset_variable_flags (object, s, flags[1]);
+  swfdec_as_object_set_variable_flags (object, s, flags[0]);
 }
 
 static gboolean
-swfdec_as_context_ASSetPropFlags_foreach (SwfdecAsObject *object, const char *s, 
-    SwfdecAsValue *val, guint cur_flags, gpointer data)
+swfdec_as_context_ASSetPropFlags_foreach (SwfdecAsObject *object,
+    const char *s, SwfdecAsValue *val, guint cur_flags, gpointer data)
 {
   guint *flags = data;
 
   /* shortcut if the flags already match */
-  if ((cur_flags & flags[1]) == flags[0])
+  if (cur_flags == ((cur_flags &~ flags[1]) | flags[0]))
     return TRUE;
 
   swfdec_as_context_ASSetPropFlags_set_one_flag (object, s, flags);
@@ -1033,53 +1191,148 @@ swfdec_as_context_ASSetPropFlags (SwfdecAsContext *cx, SwfdecAsObject *object,
   guint flags[2]; /* flags and mask - array so we can pass it as data pointer */
   SwfdecAsObject *obj;
 
-  if (cx->version < 6) {
-    SWFDEC_FIXME ("ASSetPropFlags needs some limitations for Flash 5");
-  }
   if (!SWFDEC_AS_VALUE_IS_OBJECT (&argv[0]))
     return;
   obj = SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]);
   flags[0] = swfdec_as_value_to_integer (cx, &argv[2]);
-  flags[1] = (argc > 3) ? swfdec_as_value_to_integer (cx, &argv[3]) : -1;
+  flags[1] = (argc > 3) ? swfdec_as_value_to_integer (cx, &argv[3]) : 0;
+
+  if (flags[0] == 0 && flags[1] == 0) {
+    // we should add autosizing length attribute here
+    SWFDEC_FIXME ("ASSetPropFlags to set special length attribute not implemented");
+    return;
+  }
+
   if (SWFDEC_AS_VALUE_IS_NULL (&argv[1])) {
     swfdec_as_object_foreach (obj, swfdec_as_context_ASSetPropFlags_foreach, flags);
-  } else if (SWFDEC_AS_VALUE_IS_STRING (&argv[1])) {
-    char **split = g_strsplit (SWFDEC_AS_VALUE_GET_STRING (&argv[1]), ",", -1);
+  } else {
+    char **split =
+      g_strsplit (swfdec_as_value_to_string (cx, &argv[1]), ",", -1);
     guint i;
     for (i = 0; split[i]; i++) {
       swfdec_as_context_ASSetPropFlags_set_one_flag (obj, 
 	  swfdec_as_context_get_string (cx, split[i]), flags);
     }
     g_strfreev (split); 
-  } else {
-    SWFDEC_FIXME ("ASSetPropFlags for non-null properties not implemented yet");
   }
 }
 
-static void
+SWFDEC_AS_NATIVE (200, 19, swfdec_as_context_isFinite)
+void
 swfdec_as_context_isFinite (SwfdecAsContext *cx, SwfdecAsObject *object, 
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *retval)
 {
-  double d = swfdec_as_value_to_number (cx, &argv[0]);
+  double d;
+
+  if (argc < 1)
+    return;
+
+  d = swfdec_as_value_to_number (cx, &argv[0]);
   SWFDEC_AS_VALUE_SET_BOOLEAN (retval, isfinite (d) ? TRUE : FALSE);
 }
 
-static void
+SWFDEC_AS_NATIVE (200, 18, swfdec_as_context_isNaN)
+void
 swfdec_as_context_isNaN (SwfdecAsContext *cx, SwfdecAsObject *object, 
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *retval)
 {
-  double d = swfdec_as_value_to_number (cx, &argv[0]);
+  double d;
+
+  if (argc < 1)
+    return;
+
+  d = swfdec_as_value_to_number (cx, &argv[0]);
   SWFDEC_AS_VALUE_SET_BOOLEAN (retval, isnan (d) ? TRUE : FALSE);
 }
 
-static void
+SWFDEC_AS_NATIVE (100, 2, swfdec_as_context_parseInt)
+void
 swfdec_as_context_parseInt (SwfdecAsContext *cx, SwfdecAsObject *object, 
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *retval)
 {
-  int i = swfdec_as_value_to_integer (cx, &argv[0]);
-  SWFDEC_AS_VALUE_SET_INT (retval, i);
+  const char *s;
+  char *tail;
+  int radix;
+  gint64 i;
+
+  if (argc < 1)
+    return;
+
+  s = swfdec_as_value_to_string (cx, &argv[0]);
+
+  if (argc >= 2) {
+    radix = swfdec_as_value_to_integer (cx, &argv[1]);
+
+    if (radix < 2 || radix > 36) {
+      SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
+      return;
+    }
+
+    // special case, strtol parses things that we shouldn't parse
+    if (radix == 16) {
+      const char *end = s + strspn (s, " \t\r\n");
+      if (end != s && end[0] == '0' && end[1] == 'x') {
+	SWFDEC_AS_VALUE_SET_NUMBER (retval, 0);
+	return;
+      }
+    }
+  } else {
+    radix = 0;
+  }
+
+  // special case
+  if ((s[0] == '-' || s[0] == '+') && s[1] == '0' && s[2] == 'x') {
+    SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
+    return;
+  }
+
+  if (s[0] == '0' && s[1] == 'x') {
+    s = s + 2;
+    i = g_ascii_strtoll (s, &tail, (radix != 0 ? radix : 16));
+  } else {
+    i = g_ascii_strtoll (s, &tail, (radix != 0 ? radix : 10));
+  }
+
+  if (tail == s) {
+    SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
+    return;
+  }
+
+  if (i > G_MAXINT32 || i < G_MININT32) {
+    SWFDEC_AS_VALUE_SET_NUMBER (retval, i);
+  } else {
+    SWFDEC_AS_VALUE_SET_INT (retval, i);
+  }
 }
 
+SWFDEC_AS_NATIVE (100, 3, swfdec_as_context_parseFloat)
+void
+swfdec_as_context_parseFloat (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *retval)
+{
+  char *s, *p, *tail;
+  double d;
+
+  if (argc < 1)
+    return;
+
+  // we need to remove everything after x or I, since strtod parses hexadecimal
+  // numbers and Infinity
+  s = g_strdup (swfdec_as_value_to_string (cx, &argv[0]));
+  if ((p = strpbrk (s, "xI")) != NULL) {
+    *p = '\0';
+  }
+
+  d = g_ascii_strtod (s, &tail);
+
+  if (tail == s) {
+    SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
+  } else {
+    SWFDEC_AS_VALUE_SET_NUMBER (retval, d);
+  }
+
+  g_free (s);
+}
 static void
 swfdec_as_context_init_global (SwfdecAsContext *context, guint version)
 {
@@ -1091,12 +1344,30 @@ swfdec_as_context_init_global (SwfdecAsContext *context, guint version)
   swfdec_as_object_set_variable (context->global, SWFDEC_AS_STR_NaN, &val);
   SWFDEC_AS_VALUE_SET_NUMBER (&val, HUGE_VAL);
   swfdec_as_object_set_variable (context->global, SWFDEC_AS_STR_Infinity, &val);
-  swfdec_as_object_add_function (context->global, SWFDEC_AS_STR_isFinite, 0,
-      swfdec_as_context_isFinite, 1);
-  swfdec_as_object_add_function (context->global, SWFDEC_AS_STR_isNaN, 0,
-      swfdec_as_context_isNaN, 1);
-  swfdec_as_object_add_function (context->global, SWFDEC_AS_STR_parseInt, 0,
-      swfdec_as_context_parseInt, 1);
+}
+
+void
+swfdec_as_context_run_init_script (SwfdecAsContext *context, const guint8 *data, 
+    gsize length, guint version)
+{
+  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
+  g_return_if_fail (data != NULL);
+  g_return_if_fail (length > 0);
+
+  if (version > 4) {
+    SwfdecBits bits;
+    SwfdecScript *script;
+    swfdec_bits_init_data (&bits, data, length);
+    script = swfdec_script_new_from_bits (&bits, "init", version);
+    if (script == NULL) {
+      g_warning ("script passed to swfdec_as_context_run_init_script is invalid");
+      return;
+    }
+    swfdec_as_object_run (context->global, script);
+    swfdec_script_unref (script);
+  } else {
+    SWFDEC_LOG ("not running init script, since version is <= 4");
+  }
 }
 
 /**
@@ -1117,20 +1388,63 @@ swfdec_as_context_startup (SwfdecAsContext *context, guint version)
   if (!swfdec_as_stack_push_segment (context))
     return;
   context->version = version;
+  /* init the two internal functions */
+  /* FIXME: remove them for normal contexts? */
+  swfdec_player_preinit_global (context, version);
   /* get the necessary objects up to define objects and functions sanely */
   swfdec_as_function_init_context (context, version);
   swfdec_as_object_init_context (context, version);
   /* define the global object and other important ones */
   swfdec_as_context_init_global (context, version);
-  swfdec_as_array_init_context (context, version);
-  /* define the type objects */
-  swfdec_as_boolean_init_context (context, version);
-  swfdec_as_number_init_context (context, version);
-  swfdec_as_string_init_context (context, version);
-  /* define the rest */
-  swfdec_as_math_init_context (context, version);
+
+  /* run init script */
+  swfdec_as_context_run_init_script (context, swfdec_as_initialize, sizeof (swfdec_as_initialize), 8);
 
   if (context->state == SWFDEC_AS_CONTEXT_NEW)
     context->state = SWFDEC_AS_CONTEXT_RUNNING;
 }
 
+/**
+ * swfdec_as_context_check_continue:
+ * @context: the context that might be running too long
+ *
+ * Checks if the context has been running too long. If it has, it gets aborted.
+ *
+ * Returns: %TRUE if this player aborted.
+ **/
+gboolean
+swfdec_as_context_check_continue (SwfdecAsContext *context)
+{
+  SwfdecAsContextClass *klass;
+
+  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), TRUE);
+
+  klass = SWFDEC_AS_CONTEXT_GET_CLASS (context);
+  if (klass->check_continue == NULL)
+    return TRUE;
+  if (!klass->check_continue (context)) {
+    swfdec_as_context_abort (context, "Runtime exceeded");
+    return FALSE;
+  }
+  return TRUE;
+}
+
+/**
+ * swfdec_as_context_is_aborted:
+ * @context: a #SwfdecAsContext
+ *
+ * Determines if the given context is aborted. An aborted context is not able
+ * to execute any scripts. Aborting can happen if the script engine detects bad 
+ * scripts that cause excessive memory usage, infinite loops or other problems.
+ * In that case the script engine aborts for safety reasons.
+ *
+ * Returns: %TRUE if the player is aborted, %FALSE if it runs normally.
+ **/
+gboolean
+swfdec_as_context_is_aborted (SwfdecAsContext *context)
+{
+  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), TRUE);
+
+  return context->state == SWFDEC_AS_CONTEXT_ABORTED;
+}
+
