@@ -29,6 +29,7 @@
 #include "swfdec_as_strings.h"
 #include "swfdec_as_super.h"
 #include "swfdec_debug.h"
+#include "swfdec_security_allow.h"
 
 /**
  * SECTION:SwfdecAsFrame
@@ -79,21 +80,31 @@ swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, SwfdecAsFr
 
   g_return_val_if_fail (iter != NULL, NULL);
   g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
-  /* FIXME! */
-  context = SWFDEC_AS_OBJECT (frame)->context;
-  g_return_val_if_fail (context->frame == frame, NULL);
 
+  if (frame->argc == 0) {
+    iter->i = iter->n = 0;
+    iter->stack = NULL;
+    iter->current = NULL;
+    return NULL;
+  }
+  context = SWFDEC_AS_OBJECT (frame)->context;
   if (frame->argv) {
     iter->stack = NULL;
     iter->current = (SwfdecAsValue *) frame->argv;
   } else {
-    iter->stack = context->stack;
+    SwfdecAsStack *stack = context->stack;
+    SwfdecAsValue *end;
     iter->current = frame->stack_begin - 1;
+    end = context->cur;
+    while (iter->current < stack->elements ||
+	iter->current > end) {
+      stack = stack->next;
+      end = &stack->elements[stack->used_elements];
+    }
+    iter->stack = stack;
   }
   iter->i = 0;
   iter->n = frame->argc;
-  if (frame->argc == 0)
-    iter->current = NULL;
   return iter->current;
 }
 
@@ -187,18 +198,89 @@ swfdec_as_stack_iterator_next (SwfdecAsStackIterator *iter)
   return iter->current;
 }
 
+/*** BLOCK HANDLING ***/
 
-G_DEFINE_TYPE (SwfdecAsFrame, swfdec_as_frame, SWFDEC_TYPE_AS_SCOPE)
+typedef struct {
+  const guint8 *		start;	/* start of block */
+  const guint8 *		end;	/* end of block (hitting this address will exit the block) */
+  SwfdecAsFrameBlockFunc	func;	/* function to call when block is exited (or frame is destroyed) */
+  gpointer			data;	/* data to pass to function */
+} SwfdecAsFrameBlock;
+
+/**
+ * swfdec_as_frame_push_block:
+ * @frame: a #SwfdecAsFrame
+ * @start: start of block
+ * @end: byte after end of block
+ * @func: function to call when block gets exited
+ * @data: data to pass to @func
+ *
+ * Registers a function that guards a block of memory. When the function 
+ * exits this block of memory, it will call this function. This can happen
+ * either when the program counter leaves the guarded region, when the 
+ * function returns or when the context aborted due to an unrecoverable error.
+ **/
+void
+swfdec_as_frame_push_block (SwfdecAsFrame *frame, const guint8 *start, 
+    const guint8 *end, SwfdecAsFrameBlockFunc func, gpointer data)
+{
+  SwfdecAsFrameBlock block = { start, end, func, data };
+
+  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (start <= end);
+  g_return_if_fail (start >= frame->block_start);
+  g_return_if_fail (end <= frame->block_end);
+  g_return_if_fail (func != NULL);
+
+  frame->block_start = start;
+  frame->block_end = end;
+  g_array_append_val (frame->blocks, block);
+}
+
+void
+swfdec_as_frame_pop_block (SwfdecAsFrame *frame)
+{
+  SwfdecAsFrameBlockFunc func;
+  gpointer data;
+  SwfdecAsFrameBlock *block;
+
+  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (frame->blocks->len > 0);
+
+  block = &g_array_index (frame->blocks, SwfdecAsFrameBlock, frame->blocks->len - 1);
+  func = block->func;
+  data = block->data;
+  g_array_set_size (frame->blocks, frame->blocks->len - 1);
+  if (frame->blocks->len) {
+    block--;
+    frame->block_start = block->start;
+    frame->block_end = block->end;
+  } else {
+    frame->block_start = frame->script->buffer->data;
+    frame->block_end = frame->script->buffer->data + frame->script->buffer->length;
+  }
+  /* only call function after we popped the block, so the block can push a new one */
+  func (frame, data);
+}
+
+/*** FRAME ***/
+
+G_DEFINE_TYPE (SwfdecAsFrame, swfdec_as_frame, SWFDEC_TYPE_AS_OBJECT)
 
 static void
 swfdec_as_frame_dispose (GObject *object)
 {
   SwfdecAsFrame *frame = SWFDEC_AS_FRAME (object);
 
+  /* pop blocks while state is intact */
+  while (frame->blocks->len > 0)
+    swfdec_as_frame_pop_block (frame);
+
+  /* clean up */
   g_slice_free1 (sizeof (SwfdecAsValue) * frame->n_registers, frame->registers);
-  if (frame->script) {
-    swfdec_script_unref (frame->script);
-    frame->script = NULL;
+  if (frame->security) {
+    g_object_unref (frame->security);
+    frame->security = NULL;
   }
   if (frame->constant_pool) {
     swfdec_constant_pool_free (frame->constant_pool);
@@ -208,6 +290,12 @@ swfdec_as_frame_dispose (GObject *object)
     swfdec_buffer_unref (frame->constant_pool_buffer);
     frame->constant_pool_buffer = NULL;
   }
+  g_array_free (frame->blocks, TRUE);
+  g_slist_free (frame->scope_chain);
+  if (frame->script) {
+    swfdec_script_unref (frame->script);
+    frame->script = NULL;
+  }
 
   G_OBJECT_CLASS (swfdec_as_frame_parent_class)->dispose (object);
 }
@@ -220,8 +308,7 @@ swfdec_as_frame_mark (SwfdecAsObject *object)
 
   if (frame->next)
     swfdec_as_object_mark (SWFDEC_AS_OBJECT (frame->next));
-  if (frame->scope)
-    swfdec_as_object_mark (SWFDEC_AS_OBJECT (frame->scope));
+  g_slist_foreach (frame->scope_chain, (GFunc) swfdec_as_object_mark, NULL);
   if (frame->thisp)
     swfdec_as_object_mark (frame->thisp);
   if (frame->super)
@@ -289,6 +376,8 @@ static void
 swfdec_as_frame_init (SwfdecAsFrame *frame)
 {
   frame->function_name = "unnamed";
+  frame->blocks = g_array_new (FALSE, FALSE, sizeof (SwfdecAsFrameBlock));
+  frame->block_end = (gpointer) -1;
 }
 
 static void
@@ -296,6 +385,11 @@ swfdec_as_frame_load (SwfdecAsFrame *frame)
 {
   SwfdecAsContext *context = SWFDEC_AS_OBJECT (frame)->context;
 
+  if (context->frame) {
+    frame->security = g_object_ref (context->frame->security);
+  } else {
+    frame->security = swfdec_security_allow_new ();
+  }
   frame->stack_begin = context->cur;
   context->base = frame->stack_begin;
   frame->next = context->frame;
@@ -320,8 +414,8 @@ swfdec_as_frame_new (SwfdecAsContext *context, SwfdecScript *script)
   frame->script = swfdec_script_ref (script);
   frame->function_name = script->name;
   SWFDEC_DEBUG ("new frame for function %s", frame->function_name);
-  frame->pc = script->buffer->data;
-  frame->scope = SWFDEC_AS_SCOPE (frame);
+  frame->pc = script->main;
+  frame->scope_chain = g_slist_prepend (frame->scope_chain, frame);
   frame->n_registers = script->n_registers;
   frame->registers = g_slice_alloc0 (sizeof (SwfdecAsValue) * frame->n_registers);
   if (script->constant_pool) {
@@ -419,6 +513,12 @@ swfdec_as_frame_return (SwfdecAsFrame *frame, SwfdecAsValue *return_value)
       swfdec_as_stack_pop_segment (context);
     }
   }
+  if (context->debugger) {
+    SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
+
+    if (klass->leave_frame)
+      klass->leave_frame (context->debugger, context, frame, &retval);
+  }
   /* set return value */
   if (frame->return_value) {
     *frame->return_value = retval;
@@ -454,82 +554,91 @@ swfdec_as_frame_set_this (SwfdecAsFrame *frame, SwfdecAsObject *thisp)
 }
 
 /**
- * swfdec_as_frame_find_variable:
+ * swfdec_as_frame_get_variable_and_flags:
  * @frame: a #SwfdecAsFrame
- * @variable: name of the variable to find
+ * @variable: name of the variable
+ * @value: pointer to take value of the variable or %NULL
+ * @flags: pointer to take flags or %NULL
+ * @pobject: pointer to take the actual object that held the variable or %NULL
  *
- * Finds the given variable in the current scope chain. Returns the first 
- * object in the scope chain that contains this variable in its prototype 
- * chain. If you want to know the explicit object that contains the variable,
- * you have to call swfdec_as_object_get_variable_and_flags() on the result.
- * If no such variable exist in the scope chain, %NULL is returned.
- * <note>The returned object might be an internal object. You probably do not
- * want to expose it to scripts. Call swfdec_as_object_resolve () on the
- * returned value to be sure of not having an internal object.</note>
+ * Walks the scope chain of @frame trying to resolve the given @variable and if
+ * found, returns its value and flags. Note that there might be a difference 
+ * between @pobject and the returned object, since the returned object will be
+ * part of the scope chain while @pobject will contain the actual property. It
+ * will be a prototype of the returned object though.
  *
- * Returns: the object that contains @variable or %NULL if none.
+ * Returns: Object in scope chain that contained the variable.
  **/
 SwfdecAsObject *
-swfdec_as_frame_find_variable (SwfdecAsFrame *frame, const char *variable)
+swfdec_as_frame_get_variable_and_flags (SwfdecAsFrame *frame, const char *variable,
+    SwfdecAsValue *value, guint *flags, SwfdecAsObject **pobject)
 {
-  SwfdecAsScope *cur;
-  guint i;
-  SwfdecAsValue val;
+  GSList *walk;
 
   g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
   g_return_val_if_fail (variable != NULL, NULL);
 
-  cur = frame->scope;
-  for (i = 0; i < 256; i++) {
-    if (swfdec_as_object_get_variable (SWFDEC_AS_OBJECT (cur), variable, &val))
-      return SWFDEC_AS_OBJECT (cur);
-    if (cur->next == NULL)
-      break;
-    cur = cur->next;
+  for (walk = frame->scope_chain; walk; walk = walk->next) {
+    if (swfdec_as_object_get_variable_and_flags (walk->data, variable, value, 
+	  flags, pobject))
+      return walk->data;
   }
-  if (i == 256) {
-    swfdec_as_context_abort (SWFDEC_AS_OBJECT (frame)->context, "Scope recursion limit exceeded");
-    return NULL;
-  }
-  g_assert (SWFDEC_IS_AS_FRAME (cur));
   /* we've walked the scope chain down. Now look in the special objects. */
   /* 1) the target */
-  if (swfdec_as_object_get_variable (frame->target, variable, &val))
+  if (swfdec_as_object_get_variable_and_flags (frame->target, variable, value, 
+	flags, pobject))
     return frame->target;
   /* 2) the global object */
-  if (swfdec_as_object_get_variable (SWFDEC_AS_OBJECT (frame)->context->global, variable, &val))
+  if (swfdec_as_object_get_variable_and_flags (
+	SWFDEC_AS_OBJECT (frame)->context->global, variable, value, flags, pobject))
     return SWFDEC_AS_OBJECT (frame)->context->global;
 
   return NULL;
 }
 
-/* FIXME: merge with find_variable somehow */
-gboolean
+void
+swfdec_as_frame_set_variable_and_flags (SwfdecAsFrame *frame, const char *variable,
+    const SwfdecAsValue *value, guint default_flags)
+{
+  SwfdecAsObject *pobject, *set;
+  GSList *walk;
+
+  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (variable != NULL);
+
+  set = NULL;
+  for (walk = frame->scope_chain; walk; walk = walk->next) {
+    if (swfdec_as_object_get_variable_and_flags (walk->data, variable, NULL, NULL, &pobject) &&
+	pobject == walk->data) {
+      set = walk->data;
+      break;
+    }
+  }
+  if (set == NULL)
+    set = frame->target;
+
+  swfdec_as_object_set_variable_and_flags (set, variable, value, default_flags);
+}
+
+SwfdecAsDeleteReturn
 swfdec_as_frame_delete_variable (SwfdecAsFrame *frame, const char *variable)
 {
-  SwfdecAsScope *cur;
-  guint i;
+  GSList *walk;
+  SwfdecAsDeleteReturn ret;
 
   g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), FALSE);
   g_return_val_if_fail (variable != NULL, FALSE);
 
-  cur = frame->scope;
-  for (i = 0; i < 256; i++) {
-    if (swfdec_as_object_delete_variable (SWFDEC_AS_OBJECT (cur), variable))
-      return TRUE;
-    if (cur->next == NULL)
-      break;
-    cur = cur->next;
+  for (walk = frame->scope_chain; walk; walk = walk->next) {
+    ret = swfdec_as_object_delete_variable (walk->data, variable);
+    if (ret)
+      return ret;
   }
-  if (i == 256) {
-    swfdec_as_context_abort (SWFDEC_AS_OBJECT (frame)->context, "Scope recursion limit exceeded");
-    return FALSE;
-  }
-  g_assert (SWFDEC_IS_AS_FRAME (cur));
   /* we've walked the scope chain down. Now look in the special objects. */
   /* 1) the target set via SetTarget */
-  if (swfdec_as_object_delete_variable (frame->target, variable))
-    return TRUE;
+  ret = swfdec_as_object_delete_variable (frame->target, variable);
+  if (ret)
+    return ret;
   /* 2) the global object */
   return swfdec_as_object_delete_variable (SWFDEC_AS_OBJECT (frame)->context->global, variable);
 }
@@ -569,18 +678,19 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
 
   g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
 
-  if (frame->script == NULL)
-    return;
+
   /* setup */
   object = SWFDEC_AS_OBJECT (frame);
   context = object->context;
   script = frame->script;
+  if (frame->script == NULL)
+    goto out;
 
   /* create arguments and super object if necessary */
   if ((script->flags & (SWFDEC_SCRIPT_PRELOAD_ARGS | SWFDEC_SCRIPT_SUPPRESS_ARGS)) != SWFDEC_SCRIPT_SUPPRESS_ARGS) {
     args = swfdec_as_array_new (context);
     if (!args)
-      return;
+      goto out;
     for (cur = swfdec_as_stack_iterator_init_arguments (&iter, frame); cur != NULL;
 	cur = swfdec_as_stack_iterator_next (&iter)) {
       swfdec_as_array_push (SWFDEC_AS_ARRAY (args), cur);
@@ -589,9 +699,6 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
     /* silence gcc */
     args = NULL;
   }
-  if ((script->flags & (SWFDEC_SCRIPT_PRELOAD_SUPER | SWFDEC_SCRIPT_SUPPRESS_SUPER)) != SWFDEC_SCRIPT_SUPPRESS_SUPER) {
-    frame->super = swfdec_as_super_new (frame);
-  }
 
   /* set the default variables (unless suppressed */
   if (!(script->flags & SWFDEC_SCRIPT_SUPPRESS_THIS)) {
@@ -657,57 +764,53 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
     }
   }
   if (script->flags & SWFDEC_SCRIPT_PRELOAD_ROOT && current_reg < script->n_registers) {
-    SwfdecAsObject *obj;
-    
-    obj = swfdec_as_frame_find_variable (frame, SWFDEC_AS_STR__root);
-    if (obj) {
-      swfdec_as_object_get_variable (obj, SWFDEC_AS_STR__root, &frame->registers[current_reg]);
-    } else {
+    if (!swfdec_as_frame_get_variable (frame, SWFDEC_AS_STR__root, &frame->registers[current_reg])) {
       SWFDEC_WARNING ("no root to preload");
-      SWFDEC_AS_VALUE_SET_UNDEFINED (&frame->registers[current_reg++]);
     }
+    current_reg++;
   }
   if (script->flags & SWFDEC_SCRIPT_PRELOAD_PARENT && current_reg < script->n_registers) {
-    SwfdecAsObject *obj;
-    
-    obj = swfdec_as_frame_find_variable (frame, SWFDEC_AS_STR__parent);
-    if (obj) {
-      swfdec_as_object_get_variable (obj, SWFDEC_AS_STR__parent, &frame->registers[current_reg++]);
-    } else {
-      SWFDEC_WARNING ("no parent to preload");
-      SWFDEC_AS_VALUE_SET_UNDEFINED (&frame->registers[current_reg++]);
+    if (!swfdec_as_frame_get_variable (frame, SWFDEC_AS_STR__parent, &frame->registers[current_reg])) {
+      SWFDEC_WARNING ("no root to preload");
     }
     current_reg++;
   }
   if (script->flags & SWFDEC_SCRIPT_PRELOAD_GLOBAL && current_reg < script->n_registers) {
     SWFDEC_AS_VALUE_SET_OBJECT (&frame->registers[current_reg++], context->global);
   }
+  /* set block boundaries */
+  frame->block_start = frame->script->buffer->data;
+  frame->block_end = frame->script->buffer->data + frame->script->buffer->length;
+
+out:
+  if (context->state == SWFDEC_AS_CONTEXT_ABORTED) {
+    swfdec_as_frame_return (frame, NULL);
+    return;
+  }
+  if (context->debugger) {
+    SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
+
+    if (klass->enter_frame)
+      klass->enter_frame (context->debugger, context, frame);
+  }
 }
 
-/**
- * swfdec_as_frame_check_scope:
- * @frame: a #SwfdecAsFrame
- *
- * Checks that the current scope of the given @frame is still correct.
- * If it is not, the current scope is popped and the next one is used.
- * If the
- **/
 void
-swfdec_as_frame_check_scope (SwfdecAsFrame *frame)
+swfdec_as_frame_handle_exception (SwfdecAsFrame *frame)
 {
-  SwfdecAsScope *frame_scope;
+  SwfdecAsContext *cx;
 
   g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  cx = SWFDEC_AS_OBJECT (frame)->context;
+  g_return_if_fail (cx->exception);
 
-  frame_scope = SWFDEC_AS_SCOPE (frame);
-  while (frame->scope != frame_scope) {
-    SwfdecAsScope *cur = frame->scope;
-
-    if (frame->pc >= cur->startpc && 
-	frame->pc < cur->endpc)
-      break;
-    
-    frame->scope = cur->next;
+  /* pop blocks in the hope that we are inside a Try block */
+  while (cx->exception && frame->blocks->len) {
+    swfdec_as_frame_pop_block (frame);
+  }
+  /* no Try blocks caught it, exit frame */
+  if (cx->exception) {
+    swfdec_as_frame_return (frame, NULL);
   }
 }
 
@@ -728,3 +831,74 @@ swfdec_as_frame_get_next (SwfdecAsFrame *frame)
   return frame->next;
 }
 
+/**
+ * swfdec_as_frame_get_function_name:
+ * @frame: a #SwfdecAsFrame
+ *
+ * Gets the name of the function that is currently executing. This function is
+ * intended for debugging purposes.
+ *
+ * Returns: a string. Do not free.
+ **/
+const char *
+swfdec_as_frame_get_function_name (SwfdecAsFrame *frame)
+{
+  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+
+  g_assert (frame->function_name);
+  return frame->function_name;
+}
+
+/**
+ * swfdec_as_frame_get_script:
+ * @frame: a #SwfdecAsFrame
+ *
+ * Gets the script associated with the given @frame. If the frame references
+ * a native function, there will be no script and this function returns %NULL.
+ *
+ * Returns: The script executed by this frame or %NULL
+ **/
+SwfdecScript *
+swfdec_as_frame_get_script (SwfdecAsFrame *frame)
+{
+  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+
+  return frame->script;
+}
+
+/**
+ * swfdec_as_frame_get_this:
+ * @frame: a #SwfdecAsFrame
+ *
+ * Gets the this object of the given @frame. If the frame has no this object,
+ * %NULL is returned.
+ *
+ * Returns: The this object of the frame or %NULL if none.
+ **/
+SwfdecAsObject *
+swfdec_as_frame_get_this (SwfdecAsFrame *frame)
+{
+  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+
+  return frame->thisp;
+}
+
+/**
+ * swfdec_as_frame_set_security:
+ * @frame: the frame to be executed
+ * @guard: the security to be used in this frame
+ *
+ * Sets the security to be used in this frame to @guard.
+ **/
+void
+swfdec_as_frame_set_security (SwfdecAsFrame *frame, SwfdecSecurity *guard)
+{
+  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (SWFDEC_IS_SECURITY (guard));
+
+  if (frame->security)
+    g_object_unref (frame->security);
+
+  frame->security = g_object_ref (guard);
+}
+
