@@ -27,6 +27,7 @@
 #include "swfdec_as_string.h"
 #include "swfdec_as_array.h"
 #include "swfdec_as_context.h"
+#include "swfdec_as_internal.h"
 #include "swfdec_as_native_function.h"
 #include "swfdec_as_strings.h"
 #include "swfdec_debug.h"
@@ -43,12 +44,21 @@ swfdec_as_string_do_mark (SwfdecAsObject *object)
   SWFDEC_AS_OBJECT_CLASS (swfdec_as_string_parent_class)->mark (object);
 }
 
+static char *
+swfdec_as_string_debug (SwfdecAsObject *object)
+{
+  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+
+  return g_strdup (string->string);
+}
+
 static void
 swfdec_as_string_class_init (SwfdecAsStringClass *klass)
 {
   SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
 
   asobject_class->mark = swfdec_as_string_do_mark;
+  asobject_class->debug = swfdec_as_string_debug;
 }
 
 static void
@@ -59,6 +69,24 @@ swfdec_as_string_init (SwfdecAsString *string)
 
 /*** AS CODE ***/
 
+static const char *
+swfdec_as_string_object_to_string (SwfdecAsContext *context,
+    SwfdecAsObject *object)
+{
+  SwfdecAsValue val;
+
+  g_return_val_if_fail (object == NULL || SWFDEC_IS_AS_OBJECT (object),
+      SWFDEC_AS_STR_EMPTY);
+
+  if (object == NULL) {
+    SWFDEC_AS_VALUE_SET_UNDEFINED (&val);
+  } else {
+    SWFDEC_AS_VALUE_SET_OBJECT (&val, object);
+  }
+
+  return swfdec_as_value_to_string (context, &val);
+}
+
 static inline const char *
 swfdec_as_str_nth_char (const char *s, guint n)
 {
@@ -67,44 +95,83 @@ swfdec_as_str_nth_char (const char *s, guint n)
   return s;
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 9, swfdec_as_string_lastIndexOf)
+void
+swfdec_as_string_lastIndexOf (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+{
+  const char *string = swfdec_as_string_object_to_string (cx, object);
+  gsize len;
+  const char *s;
+
+  if (argc < 1)
+    return;
+
+  s = swfdec_as_value_to_string (cx, &argv[0]);
+  if (argc == 2) {
+    int offset = swfdec_as_value_to_integer (cx, &argv[1]);
+    if (offset < 0) {
+      SWFDEC_AS_VALUE_SET_INT (ret, -1);
+      return;
+    }
+    len = g_utf8_offset_to_pointer (string, offset + 1) - string;
+  } else {
+    len = G_MAXSIZE;
+  }
+  s = g_strrstr_len (string, len, s);
+  if (s) {
+    SWFDEC_AS_VALUE_SET_INT (ret, g_utf8_pointer_to_offset (string, s));
+  } else {
+    SWFDEC_AS_VALUE_SET_INT (ret, -1);
+  }
+}
+
+SWFDEC_AS_NATIVE (251, 8, swfdec_as_string_indexOf)
+void
 swfdec_as_string_indexOf (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  const char *string = swfdec_as_string_object_to_string (cx, object);
   int offset=0, len, i=-1;
   const char *s, *t = NULL;
 
-  s = swfdec_as_value_to_string (object->context, &argv[0]);
+  if (argc < 1)
+    return;
+
+  s = swfdec_as_value_to_string (cx, &argv[0]);
   if (argc == 2)
-    offset = swfdec_as_value_to_integer (object->context, &argv[1]);
+    offset = swfdec_as_value_to_integer (cx, &argv[1]);
   if (offset < 0)
     offset = 0;
-  len = g_utf8_strlen (string->string, -1);
+  len = g_utf8_strlen (string, -1);
   if (offset < len) {
-    t = strstr (g_utf8_offset_to_pointer (string->string, offset), s);
+    t = strstr (g_utf8_offset_to_pointer (string, offset), s);
   }
   if (t != NULL) {
-    i = g_utf8_pointer_to_offset (string->string, t);
+    i = g_utf8_pointer_to_offset (string, t);
   }
 
   SWFDEC_AS_VALUE_SET_INT (ret, i);
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 5, swfdec_as_string_charAt)
+void
 swfdec_as_string_charAt (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  const char *string = swfdec_as_string_object_to_string (cx, object);
   int i;
   const char *s, *t;
 
-  i = swfdec_as_value_to_integer (object->context, &argv[0]);
+  if (argc < 1)
+    return;
+
+  i = swfdec_as_value_to_integer (cx, &argv[0]);
   if (i < 0) {
     SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STR_EMPTY);
     return;
   }
-  s = swfdec_as_str_nth_char (string->string, i);
+  s = swfdec_as_str_nth_char (string, i);
   if (*s == 0) {
     SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STR_EMPTY);
     return;
@@ -114,21 +181,25 @@ swfdec_as_string_charAt (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_AS_VALUE_SET_STRING (ret, s);
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 6, swfdec_as_string_charCodeAt)
+void
 swfdec_as_string_charCodeAt (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  const char *string = swfdec_as_string_object_to_string (cx, object);
   int i;
   const char *s;
   gunichar c;
 
+  if (argc < 1)
+    return;
+
   i = swfdec_as_value_to_integer (cx, &argv[0]);
   if (i < 0) {
     SWFDEC_AS_VALUE_SET_NUMBER (ret, NAN);
     return;
   }
-  s = swfdec_as_str_nth_char (string->string, i);
+  s = swfdec_as_str_nth_char (string, i);
   if (*s == 0) {
     if (cx->version > 5) {
       SWFDEC_AS_VALUE_SET_NUMBER (ret, NAN);
@@ -151,18 +222,23 @@ swfdec_as_string_fromCharCode_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
   char *s;
   GByteArray *array = g_byte_array_new ();
 
-  for (i = 0; i < argc; i++) {
-    c = ((guint) swfdec_as_value_to_integer (cx, &argv[i])) % 65536;
-    if (c > 255) {
-      append = c / 256;
+  if (argc > 0) {
+    for (i = 0; i < argc; i++) {
+      c = ((guint) swfdec_as_value_to_integer (cx, &argv[i])) % 65536;
+      if (c > 255) {
+	append = c / 256;
+	g_byte_array_append (array, &append, 1);
+      }
+      append = c;
       g_byte_array_append (array, &append, 1);
     }
-    append = c;
-    g_byte_array_append (array, &append, 1);
+
+    /* FIXME: are these the correct charset names? */
+    s = g_convert ((char *) array->data, array->len, "UTF-8", "LATIN1", NULL, NULL, &error);
+  } else{
+    s = g_strdup ("");
   }
 
-  /* FIXME: are these the correct charset names? */
-  s = g_convert ((char *) array->data, array->len, "UTF-8", "LATIN1", NULL, NULL, &error);
   if (s) {
     SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_context_get_string (cx, s));
     g_free (s);
@@ -174,7 +250,7 @@ swfdec_as_string_fromCharCode_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
 }
 
 static void
-swfdec_as_string_fromCharCode (SwfdecAsContext *cx, SwfdecAsObject *object,
+swfdec_as_string_fromCharCode_6 (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   gunichar tmp[8];
@@ -205,7 +281,20 @@ swfdec_as_string_fromCharCode (SwfdecAsContext *cx, SwfdecAsObject *object,
     g_free (chars);
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 14, swfdec_as_string_fromCharCode)
+void
+swfdec_as_string_fromCharCode (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+{
+  if (cx->version <= 5) {
+    swfdec_as_string_fromCharCode_5 (cx, object, argc, argv, ret);
+  } else {
+    swfdec_as_string_fromCharCode_6 (cx, object, argc, argv, ret);
+  }
+}
+
+SWFDEC_AS_CONSTRUCTOR (251, 0, swfdec_as_string_construct, swfdec_as_string_get_type)
+void
 swfdec_as_string_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
@@ -223,29 +312,34 @@ swfdec_as_string_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 
     string->string = s;
     SWFDEC_AS_VALUE_SET_INT (&val, g_utf8_strlen (string->string, -1));
-    swfdec_as_object_set_variable (object, SWFDEC_AS_STR_length, &val);
+    swfdec_as_object_set_variable_and_flags (object, SWFDEC_AS_STR_length,
+	&val, SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
     SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
   } else {
     SWFDEC_AS_VALUE_SET_STRING (ret, s);
   }
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 2, swfdec_as_string_toString)
+void
 swfdec_as_string_toString (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  if (!SWFDEC_IS_AS_STRING (object))
+    return;
 
-  SWFDEC_AS_VALUE_SET_STRING (ret, string->string);
+  SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STRING (object)->string);
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 1, swfdec_as_string_valueOf)
+void
 swfdec_as_string_valueOf (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  if (!SWFDEC_IS_AS_STRING (object))
+    return;
 
-  SWFDEC_AS_VALUE_SET_STRING (ret, string->string);
+  SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STRING (object)->string);
 }
 
 #if 0
@@ -253,7 +347,6 @@ charAt(index:Number) : String
 charCodeAt(index:Number) : Number
 concat(value:Object) : String
 indexOf(value:String, [startIndex:Number]) : Number
-lastIndexOf(value:String, [startIndex:Number]) : Number
 slice(start:Number, end:Number) : String
 split(delimiter:String, [limit:Number]) : Array
 #endif
@@ -267,7 +360,10 @@ swfdec_as_string_split_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
   const char *str, *end, *delim;
   int count;
 
-  str = SWFDEC_AS_STRING (object)->string;
+  if (argc < 1)
+    return;
+
+  str = swfdec_as_string_object_to_string (cx, object);
   arr = SWFDEC_AS_ARRAY (swfdec_as_array_new (cx));
   if (arr == NULL)
     return;
@@ -296,7 +392,7 @@ swfdec_as_string_split_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
     swfdec_as_array_push (arr, &val);
     return;
   }
-  while (*str && count > 0) {
+  while (count > 0) {
     end = strchr (str, delim[0]);
     if (end == NULL) {
       SWFDEC_AS_VALUE_SET_STRING (&val, swfdec_as_context_get_string (cx, str));
@@ -312,7 +408,7 @@ swfdec_as_string_split_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
 }
 
 static void
-swfdec_as_string_split (SwfdecAsContext *cx, SwfdecAsObject *object,
+swfdec_as_string_split_6 (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   SwfdecAsArray *arr;
@@ -321,7 +417,10 @@ swfdec_as_string_split (SwfdecAsContext *cx, SwfdecAsObject *object,
   int count;
   guint len;
 
-  str = SWFDEC_AS_STRING (object)->string;
+  if (argc < 1)
+    return;
+
+  str = swfdec_as_string_object_to_string (cx, object);
   arr = SWFDEC_AS_ARRAY (swfdec_as_array_new (cx));
   if (arr == NULL)
     return;
@@ -367,7 +466,69 @@ swfdec_as_string_split (SwfdecAsContext *cx, SwfdecAsObject *object,
   }
 }
 
-static const char *
+SWFDEC_AS_NATIVE (251, 12, swfdec_as_string_split)
+void
+swfdec_as_string_split (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+{
+  if (cx->version <= 5) {
+    swfdec_as_string_split_5 (cx, object, argc, argv, ret);
+  } else {
+    swfdec_as_string_split_6 (cx, object, argc, argv, ret);
+  }
+}
+
+SWFDEC_AS_NATIVE (251, 10, swfdec_as_string_slice)
+void
+swfdec_as_string_slice (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+{
+  int start, end, length;
+  const char *str;
+
+  if (argc == 0)
+    return;
+
+  str = swfdec_as_string_object_to_string (cx, object);
+  length = strlen (str);
+
+  start = swfdec_as_value_to_integer (cx, &argv[0]);
+  if (start < 0)
+    start += length;
+  start = CLAMP (start, 0, length);
+
+  if (argc > 1) {
+    end = swfdec_as_value_to_integer (cx, &argv[1]);
+    if (end < 0)
+      end += length;
+    end = CLAMP (end, start, length);
+  } else {
+    end = length;
+  }
+
+  SWFDEC_AS_VALUE_SET_STRING (ret,
+      swfdec_as_context_give_string (cx, g_strndup (str + start, end - start)));
+}
+
+SWFDEC_AS_NATIVE (251, 7, swfdec_as_string_concat)
+void
+swfdec_as_string_concat (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+{
+  guint i;
+  GString *string;
+
+  string = g_string_new (swfdec_as_string_object_to_string (cx, object));
+
+  for (i = 0; i < argc; i++) {
+    string = g_string_append (string, swfdec_as_value_to_string (cx, &argv[i]));
+  }
+
+  SWFDEC_AS_VALUE_SET_STRING (ret,
+      swfdec_as_context_give_string (cx, g_string_free (string, FALSE)));
+}
+
+const char *
 swfdec_as_str_sub (SwfdecAsContext *cx, const char *str, guint offset, guint len)
 {
   const char *end;
@@ -378,17 +539,21 @@ swfdec_as_str_sub (SwfdecAsContext *cx, const char *str, guint offset, guint len
   return str;
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 13, swfdec_as_string_substr)
+void
 swfdec_as_string_substr (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  const char *string = swfdec_as_string_object_to_string (cx, object);
   int from, to, len;
 
+  if (argc < 1)
+    return;
+
   from = swfdec_as_value_to_integer (cx, &argv[0]);
-  len = g_utf8_strlen (string->string, -1);
+  len = g_utf8_strlen (string, -1);
   
-  if (argc > 1) {
+  if (argc > 1 && !SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[1])) {
     to = swfdec_as_value_to_integer (cx, &argv[1]);
     /* FIXME: wtf? */
     if (to < 0) {
@@ -408,19 +573,23 @@ swfdec_as_string_substr (SwfdecAsContext *cx, SwfdecAsObject *object,
     from += len;
   from = CLAMP (from, 0, len);
   to = CLAMP (to, 0, len - from);
-  SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_str_sub (cx, string->string, from, to));
+  SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_str_sub (cx, string, from, to));
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 11, swfdec_as_string_substring)
+void
 swfdec_as_string_substring (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  const char *string = swfdec_as_string_object_to_string (cx, object);
   int from, to, len;
 
-  len = g_utf8_strlen (string->string, -1);
+  if (argc < 1)
+    return;
+
+  len = g_utf8_strlen (string, -1);
   from = swfdec_as_value_to_integer (cx, &argv[0]);
-  if (argc > 1) {
+  if (argc > 1 && !SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[1])) {
     to = swfdec_as_value_to_integer (cx, &argv[1]);
   } else {
     to = len;
@@ -436,41 +605,41 @@ swfdec_as_string_substring (SwfdecAsContext *cx, SwfdecAsObject *object,
     to = from;
     from = tmp;
   }
-  SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_str_sub (cx, string->string, from, to - from));
+  SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_str_sub (cx, string, from, to - from));
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 4, swfdec_as_string_toLowerCase)
+void
 swfdec_as_string_toLowerCase (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  const char *string = swfdec_as_string_object_to_string (cx, object);
   char *s;
 
-  s = g_utf8_strdown (string->string, -1);
+  s = g_utf8_strdown (string, -1);
   SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_context_get_string (cx, s));
   g_free (s);
 }
 
-static void
+SWFDEC_AS_NATIVE (251, 3, swfdec_as_string_toUpperCase)
+void
 swfdec_as_string_toUpperCase (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsString *string = SWFDEC_AS_STRING (object);
+  const char *string = swfdec_as_string_object_to_string (cx, object);
   char *s;
 
-  s = g_utf8_strup (string->string, -1);
+  s = g_utf8_strup (string, -1);
   SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_context_get_string (cx, s));
   g_free (s);
 }
 
 /* escape and unescape are implemented here so the mad string functions share the same place */
 
-static void
-swfdec_as_string_unescape_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
-    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+static char *
+swfdec_as_string_unescape_5 (SwfdecAsContext *cx, const char *msg)
 {
   GByteArray *array;
-  const char *msg;
   char cur = 0; /* currently decoded character */
   char *out, *in, *s;
   guint decoding = 0; /* set if we're decoding a %XY string */
@@ -479,13 +648,12 @@ swfdec_as_string_unescape_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
 #define APPEND(chr) G_STMT_START{ \
   g_byte_array_append (array, (guchar *) chr, 1); \
 }G_STMT_END
-  array = g_byte_array_new ();
-  msg = swfdec_as_value_to_string (cx, &argv[0]);
   in = s = g_convert (msg, -1, "LATIN1", "UTF-8", NULL, NULL, NULL);
   if (s == NULL) {
     SWFDEC_FIXME ("%s can not be converted to utf8 - is this Flash 5 or what?", msg);
-    return;
+    return NULL;
   }
+  array = g_byte_array_new ();
   while (*s != 0) {
     if (decoding) {
       decoding++;
@@ -516,38 +684,38 @@ swfdec_as_string_unescape_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
   }
   g_free (in);
   if (array->len == 0) {
-    SWFDEC_AS_VALUE_SET_UNDEFINED (ret);
-    return;
+    g_byte_array_free (array, TRUE);
+    return NULL;
   }
   cur = 0;
   g_byte_array_append (array, (guchar *) &cur, 1);
   out = g_convert ((char *) array->data, -1, "UTF-8", "LATIN1", NULL, NULL, NULL);
+  g_byte_array_free (array, TRUE);
   if (out) {
-    SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_context_get_string (cx, out));
-    g_free (out);
+    return out;
   } else {
     g_warning ("can't convert %s to UTF-8", msg);
-    SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STR_EMPTY);
+    g_free (out);
+    return g_strdup ("");
   }
-  g_byte_array_free (array, TRUE);
 #undef APPEND
 }
 
-static void
-swfdec_as_string_escape (SwfdecAsContext *cx, SwfdecAsObject *object,
-    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+char *
+swfdec_as_string_escape (SwfdecAsContext *cx, const char *s)
 {
   GByteArray *array;
-  const char *s;
   char *in = NULL;
 
+  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (cx), NULL);
+  g_return_val_if_fail (s != NULL, NULL);
+
   array = g_byte_array_new ();
-  s = swfdec_as_value_to_string (cx, &argv[0]);
   if (cx->version <= 5) {
     in = g_convert (s, -1, "LATIN1", "UTF-8", NULL, NULL, NULL);
     if (s == NULL) {
       SWFDEC_FIXME ("%s can not be converted to utf8 - is this Flash 5 or what?", s);
-      return;
+      return NULL;
     } else {
       s = in;
     }
@@ -568,17 +736,34 @@ swfdec_as_string_escape (SwfdecAsContext *cx, SwfdecAsObject *object,
     s++;
   }
   g_byte_array_append (array, (guchar *) s, 1);
-  SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_context_get_string (cx, (char *) array->data));
-  g_byte_array_free (array, TRUE);
   g_free (in);
+  return (char *) g_byte_array_free (array, FALSE);
 }
 
-static void
-swfdec_as_string_unescape (SwfdecAsContext *cx, SwfdecAsObject *object,
+SWFDEC_AS_NATIVE (100, 0, swfdec_as_string_escape_internal)
+void
+swfdec_as_string_escape_internal (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+{
+  const char *s;
+  char *result;
+
+  SWFDEC_AS_CHECK (0, NULL, "s", &s);
+
+  result = swfdec_as_string_escape (cx, s);
+  if (result != NULL) {
+    SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_context_get_string (cx, result));
+    g_free (result);
+  } else {
+    SWFDEC_AS_VALUE_SET_UNDEFINED (ret);
+  }
+}
+
+static char *
+swfdec_as_string_unescape_6 (SwfdecAsContext *cx, const char *s)
 {
   GByteArray *array;
-  const char *s, *msg;
+  const char *msg;
   char cur = 0; /* currently decoded character */
   guint decoding = 0; /* set if we're decoding a %XY string */
   guint utf8left = 0; /* how many valid utf8 chars are still required */
@@ -615,7 +800,7 @@ swfdec_as_string_unescape (SwfdecAsContext *cx, SwfdecAsObject *object,
   } \
 }G_STMT_END
   array = g_byte_array_new ();
-  msg = s = swfdec_as_value_to_string (cx, &argv[0]);
+  msg = s;
   while (*s != 0) {
     if (decoding) {
       decoding++;
@@ -657,65 +842,40 @@ swfdec_as_string_unescape (SwfdecAsContext *cx, SwfdecAsObject *object,
   }
   g_byte_array_append (array, (guchar *) &cur, 1);
   if (g_utf8_validate ((char *) array->data, -1, NULL)) {
-    SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_context_get_string (cx, (char *) array->data));
+    return (char *) g_byte_array_free (array, FALSE);
   } else {
     g_warning ("%s unescaped is invalid UTF-8", msg);
-    SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STR_EMPTY);
+    g_byte_array_free (array, TRUE);
+    return g_strdup ("");
   }
-  g_byte_array_free (array, TRUE);
 #undef APPEND
 }
 
-void
-swfdec_as_string_init_context (SwfdecAsContext *context, guint version)
+char *
+swfdec_as_string_unescape (SwfdecAsContext *context, const char *string)
 {
-  SwfdecAsObject *string, *proto;
-  SwfdecAsValue val;
-
-  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
-
-  string = SWFDEC_AS_OBJECT (swfdec_as_object_add_function (context->global,
-      SWFDEC_AS_STR_String, 0, swfdec_as_string_construct, 0));
-  swfdec_as_native_function_set_construct_type (SWFDEC_AS_NATIVE_FUNCTION (string), SWFDEC_TYPE_AS_STRING);
-  if (!string)
-    return;
-  proto = swfdec_as_object_new (context);
-  /* set the right properties on the String object */
-  SWFDEC_AS_VALUE_SET_OBJECT (&val, proto);
-  swfdec_as_object_set_variable (string, SWFDEC_AS_STR_prototype, &val);
-  if (version <= 5) {
-    swfdec_as_object_add_function (string, SWFDEC_AS_STR_fromCharCode, 0, swfdec_as_string_fromCharCode_5, 0);
-  } else {
-    swfdec_as_object_add_function (string, SWFDEC_AS_STR_fromCharCode, 0, swfdec_as_string_fromCharCode, 0);
-  }
-
-  /* set the right properties on the String.prototype object */
-  SWFDEC_AS_VALUE_SET_OBJECT (&val, context->Object_prototype);
-  swfdec_as_object_set_variable (proto, SWFDEC_AS_STR___proto__, &val);
-  SWFDEC_AS_VALUE_SET_OBJECT (&val, string);
-  swfdec_as_object_set_variable (proto, SWFDEC_AS_STR_constructor, &val);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_charAt, SWFDEC_TYPE_AS_STRING, swfdec_as_string_charAt, 1);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_indexOf, SWFDEC_TYPE_AS_STRING, swfdec_as_string_indexOf, 1);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_charCodeAt, SWFDEC_TYPE_AS_STRING, swfdec_as_string_charCodeAt, 1);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_substr, SWFDEC_TYPE_AS_STRING, swfdec_as_string_substr, 1);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_substring, SWFDEC_TYPE_AS_STRING, swfdec_as_string_substring, 1);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_toLowerCase, SWFDEC_TYPE_AS_STRING, swfdec_as_string_toLowerCase, 0);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_toString, SWFDEC_TYPE_AS_STRING, swfdec_as_string_toString, 0);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_toUpperCase, SWFDEC_TYPE_AS_STRING, swfdec_as_string_toUpperCase, 0);
-  swfdec_as_object_add_function (proto, SWFDEC_AS_STR_valueOf, SWFDEC_TYPE_AS_STRING, swfdec_as_string_valueOf, 0);
   if (context->version < 6) {
-    swfdec_as_object_add_function (proto, SWFDEC_AS_STR_split, SWFDEC_TYPE_AS_STRING, swfdec_as_string_split_5, 1);
+    return swfdec_as_string_unescape_5 (context, string);
   } else {
-    swfdec_as_object_add_function (proto, SWFDEC_AS_STR_split, SWFDEC_TYPE_AS_STRING, swfdec_as_string_split, 1);
-  }
-
-  /* add properties to global object */
-  if (version <= 5) {
-    swfdec_as_object_add_function (context->global, SWFDEC_AS_STR_escape, 0, swfdec_as_string_escape, 1);
-    swfdec_as_object_add_function (context->global, SWFDEC_AS_STR_unescape, 0, swfdec_as_string_unescape_5, 1);
-  } else {
-    swfdec_as_object_add_function (context->global, SWFDEC_AS_STR_escape, 0, swfdec_as_string_escape, 1);
-    swfdec_as_object_add_function (context->global, SWFDEC_AS_STR_unescape, 0, swfdec_as_string_unescape, 1);
+    return swfdec_as_string_unescape_6 (context, string);
   }
 }
 
+SWFDEC_AS_NATIVE (100, 1, swfdec_as_string_unescape_internal)
+void
+swfdec_as_string_unescape_internal (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
+{
+  const char *s;
+  char *result;
+
+  SWFDEC_AS_CHECK (0, NULL, "s", &s);
+
+  result = swfdec_as_string_unescape (cx, s);
+  if (result != NULL) {
+    SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_context_get_string (cx, result));
+    g_free (result);
+  } else {
+    SWFDEC_AS_VALUE_SET_UNDEFINED (ret);
+  }
+}
