@@ -31,6 +31,7 @@
 #include "swfdec_as_number.h"
 #include "swfdec_as_stack.h"
 #include "swfdec_as_strings.h"
+#include "swfdec_as_super.h"
 #include "swfdec_debug.h"
 #include "swfdec_movie.h"
 
@@ -369,21 +370,27 @@ swfdec_as_value_to_string (SwfdecAsContext *context, const SwfdecAsValue *value)
       {
 	SwfdecAsObject *object = SWFDEC_AS_VALUE_GET_OBJECT (value);
 	if (SWFDEC_IS_MOVIE (object)) {
-	  char *str = swfdec_movie_get_path (SWFDEC_MOVIE (object));
-	  const char *ret = swfdec_as_context_get_string (context, str);
-	  g_free (str);
-	  return ret;
+	  SwfdecMovie *movie = swfdec_movie_resolve (SWFDEC_MOVIE (object));
+	  if (movie == NULL) {
+	    return SWFDEC_AS_STR_EMPTY;
+	  } else {
+	    char *str = swfdec_movie_get_path (SWFDEC_MOVIE (object), TRUE);
+	    return swfdec_as_context_give_string (context, str);
+	  }
 	} else {
 	  SwfdecAsValue ret;
 	  swfdec_as_object_call (object, SWFDEC_AS_STR_toString, 0, NULL, &ret);
 	  if (SWFDEC_AS_VALUE_IS_STRING (&ret))
 	    return SWFDEC_AS_VALUE_GET_STRING (&ret);
-	  else if (context->version <= 5 && SWFDEC_IS_AS_FUNCTION (SWFDEC_AS_VALUE_GET_OBJECT (value)))
+	  else if (SWFDEC_IS_AS_SUPER (SWFDEC_AS_VALUE_GET_OBJECT (value)))
+	    return SWFDEC_AS_STR__type_Object_;
+	  else if (SWFDEC_IS_AS_FUNCTION (SWFDEC_AS_VALUE_GET_OBJECT (value)))
 	    return SWFDEC_AS_STR__type_Function_;
 	  else
 	    return SWFDEC_AS_STR__type_Object_;
 	}
       }
+    case SWFDEC_AS_TYPE_INT:
     default:
       g_assert_not_reached ();
       return SWFDEC_AS_STR_EMPTY;
@@ -417,6 +424,7 @@ swfdec_as_value_to_debug (const SwfdecAsValue *value)
       return g_strdup_printf ("%g", SWFDEC_AS_VALUE_GET_NUMBER (value));
     case SWFDEC_AS_TYPE_OBJECT:
       return swfdec_as_object_get_debug (SWFDEC_AS_VALUE_GET_OBJECT (value));
+    case SWFDEC_AS_TYPE_INT:
     default:
       g_assert_not_reached ();
       return NULL;
@@ -471,6 +479,7 @@ swfdec_as_value_to_number (SwfdecAsContext *context, const SwfdecAsValue *value)
       }
     case SWFDEC_AS_TYPE_OBJECT:
       return NAN;
+    case SWFDEC_AS_TYPE_INT:
     default:
       g_assert_not_reached ();
       return NAN;
@@ -496,10 +505,10 @@ swfdec_as_value_to_integer (SwfdecAsContext *context, const SwfdecAsValue *value
   if (!isfinite (d))
     return 0;
   if (d < 0) {
-    d = fmod (-d, 4294967296);
+    d = fmod (-d, 4294967296.0);
     return - (guint) d;
   } else {
-    d = fmod (d, 4294967296);
+    d = fmod (d, 4294967296.0);
     return (guint) d;
   }
 }
@@ -542,6 +551,7 @@ swfdec_as_value_to_object (SwfdecAsContext *context, const SwfdecAsValue *value)
       break;
     case SWFDEC_AS_TYPE_OBJECT:
       return SWFDEC_AS_VALUE_GET_OBJECT (value);
+    case SWFDEC_AS_TYPE_INT:
     default:
       g_assert_not_reached ();
       return NULL;
@@ -551,11 +561,10 @@ swfdec_as_value_to_object (SwfdecAsContext *context, const SwfdecAsValue *value)
   if (!SWFDEC_AS_VALUE_IS_OBJECT (&val) ||
       !SWFDEC_IS_AS_FUNCTION (fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (&val)))
     return NULL;
-  swfdec_as_object_create (fun, 1, value);
+  swfdec_as_object_create (fun, 1, value, &val);
   swfdec_as_context_run (context);
-  value = swfdec_as_stack_pop (context);
-  if (SWFDEC_AS_VALUE_IS_OBJECT (value)) {
-    return SWFDEC_AS_VALUE_GET_OBJECT (value);
+  if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
+    return SWFDEC_AS_VALUE_GET_OBJECT (&val);
   } else {
     SWFDEC_ERROR ("did not construct an object");
     return NULL;
@@ -575,34 +584,35 @@ swfdec_as_value_to_object (SwfdecAsContext *context, const SwfdecAsValue *value)
 gboolean
 swfdec_as_value_to_boolean (SwfdecAsContext *context, const SwfdecAsValue *value)
 {
-g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), FALSE);
-g_return_val_if_fail (SWFDEC_IS_AS_VALUE (value), FALSE);
+  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), FALSE);
+  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (value), FALSE);
 
-/* FIXME: what do we do when called in flash 4? */
-switch (value->type) {
-case SWFDEC_AS_TYPE_UNDEFINED:
-case SWFDEC_AS_TYPE_NULL:
-return FALSE;
-case SWFDEC_AS_TYPE_BOOLEAN:
-return SWFDEC_AS_VALUE_GET_BOOLEAN (value);
-case SWFDEC_AS_TYPE_NUMBER:
-{
-  double d = SWFDEC_AS_VALUE_GET_NUMBER (value);
-  return d != 0.0 && !isnan (d);
-}
-case SWFDEC_AS_TYPE_STRING:
-if (context->version <= 6) {
-  double d = swfdec_as_value_to_number (context, value);
-  return d != 0.0 && !isnan (d);
-} else {
-  return SWFDEC_AS_VALUE_GET_STRING (value) != SWFDEC_AS_STR_EMPTY;
-}
-case SWFDEC_AS_TYPE_OBJECT:
-return TRUE;
-default:
-g_assert_not_reached ();
-return FALSE;
-}
+  /* FIXME: what do we do when called in flash 4? */
+  switch (value->type) {
+    case SWFDEC_AS_TYPE_UNDEFINED:
+    case SWFDEC_AS_TYPE_NULL:
+      return FALSE;
+    case SWFDEC_AS_TYPE_BOOLEAN:
+      return SWFDEC_AS_VALUE_GET_BOOLEAN (value);
+    case SWFDEC_AS_TYPE_NUMBER:
+      {
+	double d = SWFDEC_AS_VALUE_GET_NUMBER (value);
+	return d != 0.0 && !isnan (d);
+      }
+    case SWFDEC_AS_TYPE_STRING:
+      if (context->version <= 6) {
+	double d = swfdec_as_value_to_number (context, value);
+	return d != 0.0 && !isnan (d);
+      } else {
+	return SWFDEC_AS_VALUE_GET_STRING (value) != SWFDEC_AS_STR_EMPTY;
+      }
+    case SWFDEC_AS_TYPE_OBJECT:
+      return TRUE;
+    case SWFDEC_AS_TYPE_INT:
+    default:
+      g_assert_not_reached ();
+      return FALSE;
+  }
 }
 
 /**
@@ -617,11 +627,11 @@ return FALSE;
 void
 swfdec_as_value_to_primitive (SwfdecAsValue *value)
 {
-g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
+  g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
 
-if (SWFDEC_AS_VALUE_IS_OBJECT (value) && !SWFDEC_IS_MOVIE (SWFDEC_AS_VALUE_GET_OBJECT (value))) {
-swfdec_as_object_call (SWFDEC_AS_VALUE_GET_OBJECT (value), SWFDEC_AS_STR_valueOf,
-  0, NULL, value);
-}
+  if (SWFDEC_AS_VALUE_IS_OBJECT (value) && !SWFDEC_IS_MOVIE (SWFDEC_AS_VALUE_GET_OBJECT (value))) {
+    swfdec_as_object_call (SWFDEC_AS_VALUE_GET_OBJECT (value), SWFDEC_AS_STR_valueOf,
+	0, NULL, value);
+  }
 }
 
