@@ -59,6 +59,15 @@
  * swfdec_buffer_new_from_file() without the need for a different API.
  */
 
+/**
+ * SwfdecBufferFreeFunc:
+ * @data: The data to free
+ * @priv: The private data registered for passing to this function
+ *
+ * This is the function prototype for the function that is called for freeing
+ * the memory pointed to by a buffer. See swfdec_buffer_new() for an example.
+ */
+
 GType
 swfdec_buffer_get_type (void)
 {
@@ -80,7 +89,8 @@ swfdec_buffer_get_type (void)
  * <informalexample><programlisting>SwfdecBuffer *buffer = swfdec_buffer_new ();
  * buffer->data = mydata;
  * buffer->length = mydata_length;
- * buffer->free = mydata_freefunc;</programlisting></informalexample>
+ * buffer->free = mydata_freefunc;
+ * buffer->priv = mydata_private;</programlisting></informalexample>
  *
  * Returns: a new #SwfdecBuffer referencing nothing.
  **/
@@ -94,12 +104,6 @@ swfdec_buffer_new (void)
   return buffer;
 }
 
-static void
-swfdec_buffer_free_mem (SwfdecBuffer * buffer, void *priv)
-{
-  g_free (buffer->data);
-}
-
 /**
  * swfdec_buffer_new_and_alloc:
  * @size: amount of bytes to allocate
@@ -116,7 +120,7 @@ swfdec_buffer_new_and_alloc (guint size)
 
   buffer->data = g_malloc (size);
   buffer->length = size;
-  buffer->free = swfdec_buffer_free_mem;
+  buffer->free = (SwfdecBufferFreeFunc) g_free;
 
   return buffer;
 }
@@ -137,7 +141,7 @@ swfdec_buffer_new_and_alloc0 (guint size)
 
   buffer->data = g_malloc0 (size);
   buffer->length = size;
-  buffer->free = swfdec_buffer_free_mem;
+  buffer->free = (SwfdecBufferFreeFunc) g_free;
 
   return buffer;
 }
@@ -162,15 +166,15 @@ swfdec_buffer_new_for_data (unsigned char *data, guint size)
   buffer = swfdec_buffer_new ();
   buffer->data = data;
   buffer->length = size;
-  buffer->free = swfdec_buffer_free_mem;
+  buffer->free = (SwfdecBufferFreeFunc) g_free;
 
   return buffer;
 }
 
 static void
-swfdec_buffer_free_subbuffer (SwfdecBuffer * buffer, void *priv)
+swfdec_buffer_free_subbuffer (unsigned char *data, gpointer priv)
 {
-  swfdec_buffer_unref (buffer->parent);
+  swfdec_buffer_unref (priv);
 }
 
 /**
@@ -194,14 +198,7 @@ swfdec_buffer_new_subbuffer (SwfdecBuffer * buffer, guint offset, guint length)
 
   subbuffer = swfdec_buffer_new ();
 
-  if (buffer->parent) {
-    swfdec_buffer_ref (buffer->parent);
-    subbuffer->parent = buffer->parent;
-  } else {
-    swfdec_buffer_ref (buffer);
-    subbuffer->parent = buffer;
-  }
-  g_assert (subbuffer->parent->parent == NULL);
+  subbuffer->priv = swfdec_buffer_ref (swfdec_buffer_get_super (buffer));
   subbuffer->data = buffer->data + offset;
   subbuffer->length = length;
   subbuffer->free = swfdec_buffer_free_subbuffer;
@@ -209,8 +206,31 @@ swfdec_buffer_new_subbuffer (SwfdecBuffer * buffer, guint offset, guint length)
   return subbuffer;
 }
 
+/**
+ * swfdec_buffer_get_super:
+ * @buffer: a #SwfdecBuffer
+ *
+ * Returns the largest buffer that contains the memory pointed to by @buffer.
+ * This will either be the passed @buffer itself, or if the buffer was created
+ * via swfdec_buffer_new_subbuffer(), the buffer used for that.
+ *
+ * Returns: The largest @buffer available that contains the memory pointed to 
+ *          by @buffer.
+ **/
+SwfdecBuffer *
+swfdec_buffer_get_super (SwfdecBuffer *buffer)
+{
+  g_return_val_if_fail (buffer != NULL, NULL);
+
+  if (buffer->free == swfdec_buffer_free_subbuffer)
+    buffer = buffer->priv;
+
+  g_assert (buffer->free != swfdec_buffer_free_subbuffer);
+  return buffer;
+}
+
 static void
-swfdec_buffer_free_mapped (SwfdecBuffer * buffer, void *priv)
+swfdec_buffer_free_mapped (unsigned char *data, gpointer priv)
 {
   g_mapped_file_free (priv);
 }
@@ -220,9 +240,9 @@ swfdec_buffer_free_mapped (SwfdecBuffer * buffer, void *priv)
  * @filename: file to read
  * @error: return location for a #GError or %NULL
  *
- * Tries to create a buffer for the given @filename using a #GMappedFile. If
- * the creation fails, %NULL is returned and @error is set. The error can be
- * any of the errors that are valid from g_mapped_file_new().
+ * Creates a buffer containing the contents of the given file. If loading the
+ * file fails, %NULL is returned and @error is set. The error can be
+ * any of the errors that are valid for g_file_get_contents().
  *
  * Returns: a new #SwfdecBuffer or %NULL on failure
  **/
@@ -230,22 +250,25 @@ SwfdecBuffer *
 swfdec_buffer_new_from_file (const char *filename, GError **error)
 {
   GMappedFile *file;
-  SwfdecBuffer *buffer;
+  char *data;
+  gsize length;
 
   g_return_val_if_fail (filename != NULL, NULL);
 
-  file = g_mapped_file_new (filename, FALSE, error);
-  if (file == NULL) {
-    return NULL;
+  file = g_mapped_file_new (filename, FALSE, NULL);
+  if (file != NULL) {
+    SwfdecBuffer *buffer = swfdec_buffer_new ();
+    buffer->data = (unsigned char *) g_mapped_file_get_contents (file), 
+    buffer->length = g_mapped_file_get_length (file);
+    buffer->free = swfdec_buffer_free_mapped;
+    buffer->priv = file;
+    return buffer;
   }
 
-  buffer = swfdec_buffer_new ();
-  buffer->data = (unsigned char *) g_mapped_file_get_contents (file), 
-  buffer->length = g_mapped_file_get_length (file);
-  buffer->free = swfdec_buffer_free_mapped;
-  buffer->priv = file;
+  if (!g_file_get_contents (filename, &data, &length, error))
+    return NULL;
 
-  return buffer;
+  return swfdec_buffer_new_for_data ((guint8 *) data, length);
 }
 
 /**
@@ -282,7 +305,7 @@ swfdec_buffer_unref (SwfdecBuffer * buffer)
   buffer->ref_count--;
   if (buffer->ref_count == 0) {
     if (buffer->free)
-      buffer->free (buffer, buffer->priv);
+      buffer->free (buffer->data, buffer->priv);
     g_free (buffer);
   }
 }
@@ -501,6 +524,31 @@ swfdec_buffer_queue_pull (SwfdecBufferQueue * queue, guint length)
   return newbuffer;
 }
 
+/**
+ * swfdec_buffer_queue_peek_buffer:
+ * @queue: a #SwfdecBufferQueue
+ *
+ * Gets the first buffer out of @queue and returns it. This function is 
+ * equivalent to calling swfdec_buffer_queue_peek() with the size of the
+ * first buffer in it.
+ *
+ * Returns: The first buffer in @queue or %NULL if @queue is empty. Use 
+ *          swfdec_buffer_unref() after use.
+ **/
+SwfdecBuffer *
+swfdec_buffer_queue_peek_buffer (SwfdecBufferQueue * queue)
+{
+  SwfdecBuffer *buffer;
+
+  g_return_val_if_fail (queue != NULL, NULL);
+  if (queue->buffers == NULL)
+    return NULL;
+
+  buffer = queue->buffers->data;
+
+  return swfdec_buffer_queue_peek (queue, buffer->length);
+}
+
 /**
  * swfdec_buffer_queue_peek:
  * @queue: a #SwfdecBufferQueue to read from
