@@ -48,7 +48,7 @@ static const SwfdecButtonCondition event_table[2][4][4] = {
     { SWFDEC_BUTTON_OVER_UP_TO_IDLE, -1, -1, SWFDEC_BUTTON_OVER_UP_TO_OVER_DOWN },
     { -1, SWFDEC_BUTTON_OVER_DOWN_TO_OUT_DOWN, SWFDEC_BUTTON_OVER_DOWN_TO_OVER_UP, -1 } },
   { { -1, -1, SWFDEC_BUTTON_IDLE_TO_OVER_UP, -1 },
-    { 0, -1, -1, SWFDEC_BUTTON_IDLE_TO_OVER_DOWN },
+    { -2, -1, -1, SWFDEC_BUTTON_IDLE_TO_OVER_DOWN },
     { SWFDEC_BUTTON_OVER_UP_TO_IDLE, -1, -1, SWFDEC_BUTTON_OVER_UP_TO_OVER_DOWN },
     { -1, SWFDEC_BUTTON_OVER_DOWN_TO_IDLE, SWFDEC_BUTTON_OVER_DOWN_TO_OVER_UP, -1 } }
 };
@@ -99,16 +99,17 @@ swfdec_button_movie_execute (SwfdecButtonMovie *movie,
   const char *name;
 
   if (movie->button->menubutton) {
-    g_assert ((condition & (SWFDEC_BUTTON_OVER_DOWN_TO_OUT_DOWN \
-                         | SWFDEC_BUTTON_OUT_DOWN_TO_OVER_DOWN \
-                         | SWFDEC_BUTTON_OUT_DOWN_TO_IDLE)) == 0);
+    g_assert ((condition & ((1 << SWFDEC_BUTTON_OVER_DOWN_TO_OUT_DOWN) \
+                         | (1 << SWFDEC_BUTTON_OUT_DOWN_TO_OVER_DOWN) \
+                         | (1 << SWFDEC_BUTTON_OUT_DOWN_TO_IDLE))) == 0);
   } else {
-    g_assert ((condition & (SWFDEC_BUTTON_IDLE_TO_OVER_DOWN \
-                         | SWFDEC_BUTTON_OVER_DOWN_TO_IDLE)) == 0);
+    g_assert ((condition & ((1 << SWFDEC_BUTTON_IDLE_TO_OVER_DOWN) \
+                         | (1 << SWFDEC_BUTTON_OVER_DOWN_TO_IDLE))) == 0);
   }
   if (movie->button->events)
     swfdec_event_list_execute (movie->button->events, 
-	SWFDEC_AS_OBJECT (SWFDEC_MOVIE (movie)->parent), condition, 0);
+	SWFDEC_AS_OBJECT (SWFDEC_MOVIE (movie)->parent), 
+	SWFDEC_SECURITY (SWFDEC_MOVIE (movie)->resource), condition, 0);
   name = swfdec_button_condition_get_name (condition);
   swfdec_as_object_call (SWFDEC_AS_OBJECT (movie), name, 0, NULL, NULL);
 }
@@ -155,7 +156,7 @@ swfdec_button_movie_change_state (SwfdecButtonMovie *movie, SwfdecButtonState st
 static void
 swfdec_button_movie_change_mouse (SwfdecButtonMovie *movie, gboolean mouse_in, int button)
 {
-  SwfdecButtonCondition event;
+  int event;
   int sound;
 
   if (movie->mouse_in == mouse_in &&
@@ -171,8 +172,15 @@ swfdec_button_movie_change_mouse (SwfdecButtonMovie *movie, gboolean mouse_in, i
 		     [(movie->mouse_in ? 2 : 0) + movie->mouse_button]
 		     [(mouse_in ? 2 : 0) + button];
 
-  g_assert (event != (guint) -1);
-  if (event != 0) {
+#ifndef G_DISABLE_ASSERT
+  if (event == -1) {
+    g_error ("Unhandled event for %s: %u => %u",
+	movie->button->menubutton ? "menu" : "button",
+	(movie->mouse_in ? 2 : 0) + movie->mouse_button,
+	(mouse_in ? 2 : 0) + button);
+  }
+#endif
+  if (event >= 0) {
     SWFDEC_LOG ("emitting event for condition %u", event);
     swfdec_button_movie_execute (movie, event);
   }
@@ -182,7 +190,7 @@ swfdec_button_movie_change_mouse (SwfdecButtonMovie *movie, gboolean mouse_in, i
   if (sound >= 0 && movie->button->sounds[sound]) {
     SwfdecAudio *audio;
     SWFDEC_LOG ("playing button sound %d", sound);
-    audio = swfdec_audio_event_new (
+    audio = swfdec_audio_event_new_from_chunk (
 	SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context),
 	movie->button->sounds[sound]);
     if (audio)
