@@ -25,7 +25,7 @@
 #include "swfdec_as_strings.h"
 #include "swfdec_debug.h"
 #include "swfdec_player_internal.h"
-#include "swfdec_script.h"
+#include "swfdec_script_internal.h"
 
 typedef struct _SwfdecEvent SwfdecEvent;
 
@@ -118,6 +118,7 @@ SwfdecEventList *
 swfdec_event_list_copy (SwfdecEventList *list)
 {
   g_return_val_if_fail (list != NULL, NULL);
+  g_return_val_if_fail (list->refcount > 0, NULL);
 
   list->refcount++;
 
@@ -130,6 +131,7 @@ swfdec_event_list_free (SwfdecEventList *list)
   guint i;
 
   g_return_if_fail (list != NULL);
+  g_return_if_fail (list->refcount > 0);
 
   list->refcount--;
   if (list->refcount > 0)
@@ -143,56 +145,14 @@ swfdec_event_list_free (SwfdecEventList *list)
   g_free (list);
 }
 
-static const char *
-swfdec_event_list_condition_name (guint conditions)
-{
-  if (conditions & SWFDEC_EVENT_LOAD)
-    return "Load";
-  if (conditions & SWFDEC_EVENT_ENTER)
-    return "Enter";
-  if (conditions & SWFDEC_EVENT_UNLOAD)
-    return "Unload";
-  if (conditions & SWFDEC_EVENT_MOUSE_MOVE)
-    return "MouseMove";
-  if (conditions & SWFDEC_EVENT_MOUSE_DOWN)
-    return "MouseDown";
-  if (conditions & SWFDEC_EVENT_MOUSE_UP)
-    return "MouseUp";
-  if (conditions & SWFDEC_EVENT_KEY_UP)
-    return "KeyUp";
-  if (conditions & SWFDEC_EVENT_KEY_DOWN)
-    return "KeyDown";
-  if (conditions & SWFDEC_EVENT_DATA)
-    return "Data";
-  if (conditions & SWFDEC_EVENT_INITIALIZE)
-    return "Initialize";
-  if (conditions & SWFDEC_EVENT_PRESS)
-    return "Press";
-  if (conditions & SWFDEC_EVENT_RELEASE)
-    return "Release";
-  if (conditions & SWFDEC_EVENT_RELEASE_OUTSIDE)
-    return "ReleaseOutside";
-  if (conditions & SWFDEC_EVENT_ROLL_OVER)
-    return "RollOver";
-  if (conditions & SWFDEC_EVENT_ROLL_OUT)
-    return "RollOut";
-  if (conditions & SWFDEC_EVENT_DRAG_OVER)
-    return "DragOver";
-  if (conditions & SWFDEC_EVENT_DRAG_OUT)
-    return "DragOut";
-  if (conditions & SWFDEC_EVENT_KEY_PRESS)
-    return "KeyPress";
-  if (conditions & SWFDEC_EVENT_CONSTRUCT)
-    return "Construct";
-  return "No Event";
-}
-
+#define N_CONDITIONS 19
 void
 swfdec_event_list_parse (SwfdecEventList *list, SwfdecBits *bits, int version,
     guint conditions, guint8 key, const char *description)
 {
   SwfdecEvent event;
   char *name;
+  guint i;
 
   g_return_if_fail (list != NULL);
   g_return_if_fail (list->refcount == 1);
@@ -200,9 +160,10 @@ swfdec_event_list_parse (SwfdecEventList *list, SwfdecBits *bits, int version,
 
   event.conditions = conditions;
   event.key = key;
-  name = g_strconcat (description, ".", 
-      swfdec_event_list_condition_name (conditions), NULL);
-  event.script = swfdec_script_new_for_context (SWFDEC_AS_CONTEXT (list->player), bits, name, version);
+  i = g_bit_nth_lsf (conditions, -1);
+  name = g_strconcat (description, ".", i < N_CONDITIONS ? 
+      swfdec_event_type_get_name (i) : "???", NULL);
+  event.script = swfdec_script_new_from_bits (bits, name, version);
   g_free (name);
   if (event.script) 
     g_array_append_val (list->events, event);
@@ -210,21 +171,28 @@ swfdec_event_list_parse (SwfdecEventList *list, SwfdecBits *bits, int version,
 
 void
 swfdec_event_list_execute (SwfdecEventList *list, SwfdecAsObject *object, 
-    guint condition, guint8 key)
+    SwfdecSecurity *sec, guint condition, guint8 key)
 {
   guint i;
 
   g_return_if_fail (list != NULL);
   g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
+  g_return_if_fail (SWFDEC_IS_SECURITY (sec));
+  g_return_if_fail (condition < N_CONDITIONS);
 
+  condition = (1 << condition);
+  /* FIXME: Do we execute all events if the event list is gone already? */
+  /* need to ref here because followup code could free all references to the list */
+  list = swfdec_event_list_copy (list);
   for (i = 0; i < list->events->len; i++) {
     SwfdecEvent *event = &g_array_index (list->events, SwfdecEvent, i);
     if ((event->conditions & condition) &&
 	event->key == key) {
       SWFDEC_LOG ("executing script for event %u on scriptable %p", condition, object);
-      swfdec_as_object_run (object, event->script);
+      swfdec_as_object_run_with_security (object, event->script, sec);
     }
   }
+  swfdec_event_list_free (list);
 }
 
 gboolean
@@ -232,20 +200,18 @@ swfdec_event_list_has_conditions (SwfdecEventList *list, SwfdecAsObject *object,
     guint condition, guint8 key)
 {
   guint i;
-  const char *name;
 
   g_return_val_if_fail (list != NULL, FALSE);
   g_return_val_if_fail (SWFDEC_IS_AS_OBJECT (object), FALSE);
+  g_return_val_if_fail (condition < N_CONDITIONS, FALSE);
 
+  condition = 1 << condition;
   for (i = 0; i < list->events->len; i++) {
     SwfdecEvent *event = &g_array_index (list->events, SwfdecEvent, i);
     if ((event->conditions & condition) &&
 	event->key == key)
       return TRUE;
   }
-  name = swfdec_event_type_get_name (condition);
-  if (name)
-    return swfdec_as_object_has_function (object, name);
   return FALSE;
 }
 
