@@ -20,6 +20,7 @@
 #include <libswfdec/swfdec_bits.h>
 #include <libswfdec/swfdec_as_types.h>
 #include <libswfdec/swfdec_player.h>
+#include <libswfdec/swfdec_security.h>
 #include <libswfdec/swfdec_types.h>
 
 #ifndef _SWFDEC_EVENT_H_
@@ -28,25 +29,25 @@
 G_BEGIN_DECLS
 
 typedef enum _SwfdecEventType {
-  SWFDEC_EVENT_LOAD		= (1 << 0),
-  SWFDEC_EVENT_ENTER		= (1 << 1),
-  SWFDEC_EVENT_UNLOAD		= (1 << 2),
-  SWFDEC_EVENT_MOUSE_MOVE	= (1 << 3),
-  SWFDEC_EVENT_MOUSE_DOWN	= (1 << 4),
-  SWFDEC_EVENT_MOUSE_UP		= (1 << 5),
-  SWFDEC_EVENT_KEY_UP		= (1 << 6),
-  SWFDEC_EVENT_KEY_DOWN		= (1 << 7),
-  SWFDEC_EVENT_DATA		= (1 << 8),
-  SWFDEC_EVENT_INITIALIZE	= (1 << 9),
-  SWFDEC_EVENT_PRESS		= (1 << 10),
-  SWFDEC_EVENT_RELEASE		= (1 << 11),
-  SWFDEC_EVENT_RELEASE_OUTSIDE	= (1 << 12),
-  SWFDEC_EVENT_ROLL_OVER	= (1 << 13),
-  SWFDEC_EVENT_ROLL_OUT		= (1 << 14),
-  SWFDEC_EVENT_DRAG_OVER	= (1 << 15),
-  SWFDEC_EVENT_DRAG_OUT		= (1 << 16),
-  SWFDEC_EVENT_KEY_PRESS	= (1 << 17),
-  SWFDEC_EVENT_CONSTRUCT	= (1 << 18)
+  SWFDEC_EVENT_LOAD = 0,
+  SWFDEC_EVENT_ENTER = 1,
+  SWFDEC_EVENT_UNLOAD = 2,
+  SWFDEC_EVENT_MOUSE_MOVE = 3,
+  SWFDEC_EVENT_MOUSE_DOWN = 4,
+  SWFDEC_EVENT_MOUSE_UP = 5,
+  SWFDEC_EVENT_KEY_UP = 6,
+  SWFDEC_EVENT_KEY_DOWN = 7,
+  SWFDEC_EVENT_DATA = 8,
+  SWFDEC_EVENT_INITIALIZE = 9,
+  SWFDEC_EVENT_PRESS = 10,
+  SWFDEC_EVENT_RELEASE = 11,
+  SWFDEC_EVENT_RELEASE_OUTSIDE = 12,
+  SWFDEC_EVENT_ROLL_OVER = 13,
+  SWFDEC_EVENT_ROLL_OUT = 14,
+  SWFDEC_EVENT_DRAG_OVER = 15,
+  SWFDEC_EVENT_DRAG_OUT = 16,
+  SWFDEC_EVENT_KEY_PRESS = 17,
+  SWFDEC_EVENT_CONSTRUCT = 18
 } SwfdecEventType;
 
 const char *		swfdec_event_type_get_name	(SwfdecEventType      type);
@@ -63,6 +64,7 @@ void			swfdec_event_list_parse		(SwfdecEventList *    list,
 							 const char *	      description);
 void			swfdec_event_list_execute	(SwfdecEventList *    list,
 							 SwfdecAsObject *     object,
+							 SwfdecSecurity *     sec,
 							 guint		      condition,
 							 guint8		      key);
 gboolean		swfdec_event_list_has_conditions(SwfdecEventList *    list,
