@@ -39,16 +39,15 @@ typedef struct _SwfdecFlvDataTag SwfdecFlvDataTag;
 
 struct _SwfdecFlvVideoTag {
   guint			timestamp;		/* milliseconds */
-  SwfdecVideoFormat	format;			/* format in use */
+  guint			format;			/* format in use */
   int			frame_type;		/* 0: undefined, 1: keyframe, 2: iframe, 3: H263 disposable iframe */
   SwfdecBuffer *	buffer;			/* buffer for this data */
 };
 
 struct _SwfdecFlvAudioTag {
   guint			timestamp;		/* milliseconds */
-  SwfdecAudioFormat	format;			/* format in use */
-  gboolean		width;			/* TRUE for 16bit, FALSE for 8bit */
-  SwfdecAudioOut	original_format;      	/* channel/rate information */
+  guint			format;			/* format in use */
+  SwfdecAudioFormat	original_format;      	/* channel/rate information */
   SwfdecBuffer *	buffer;			/* buffer for this data */
 };
 
@@ -89,7 +88,8 @@ swfdec_flv_decoder_dispose (GObject *object)
     g_array_free (flv->data, TRUE);
     flv->data = NULL;
   }
-
+  swfdec_buffer_queue_unref (flv->queue);
+  flv->queue = NULL;
 
   G_OBJECT_CLASS (swfdec_flv_decoder_parent_class)->dispose (object);
 }
@@ -97,14 +97,12 @@ swfdec_flv_decoder_dispose (GObject *object)
 static SwfdecStatus
 swfdec_flv_decoder_parse_header (SwfdecFlvDecoder *flv)
 {
-  SwfdecDecoder *dec = SWFDEC_DECODER (flv);
   SwfdecBuffer *buffer;
   SwfdecBits bits;
   guint version, header_length;
   gboolean has_audio, has_video;
 
-  /* NB: we're still reading from the original queue, since deflating is not initialized yet */
-  buffer = swfdec_buffer_queue_peek (dec->queue, 9);
+  buffer = swfdec_buffer_queue_peek (flv->queue, 9);
   if (buffer == NULL)
     return SWFDEC_STATUS_NEEDBITS;
 
@@ -129,7 +127,7 @@ swfdec_flv_decoder_parse_header (SwfdecFlvDecoder *flv)
     /* FIXME: treat as error or ignore? */
     return SWFDEC_STATUS_ERROR;
   }
-  buffer = swfdec_buffer_queue_pull (dec->queue, header_length);
+  buffer = swfdec_buffer_queue_pull (flv->queue, header_length);
   if (buffer == NULL)
     return SWFDEC_STATUS_NEEDBITS;
   swfdec_buffer_unref (buffer);
@@ -152,12 +150,11 @@ swfdec_flv_decoder_parse_header (SwfdecFlvDecoder *flv)
 static SwfdecStatus
 swfdec_flv_decoder_parse_last_tag (SwfdecFlvDecoder *flv)
 {
-  SwfdecDecoder *dec = SWFDEC_DECODER (flv);
   SwfdecBuffer *buffer;
   SwfdecBits bits;
   guint last_tag;
 
-  buffer = swfdec_buffer_queue_pull (dec->queue, 4);
+  buffer = swfdec_buffer_queue_pull (flv->queue, 4);
   if (buffer == NULL)
     return SWFDEC_STATUS_NEEDBITS;
 
@@ -239,7 +236,8 @@ swfdec_flv_decoder_parse_video_tag (SwfdecFlvDecoder *flv, SwfdecBits *bits, gui
   SwfdecFlvVideoTag tag;
 
   if (flv->video == NULL) {
-    SWFDEC_ERROR ("wow, video tags in an FLV without video!");
+    SWFDEC_INFO ("video tags even though header didn't decalre them. Initializing...");
+    flv->video = g_array_new (FALSE, FALSE, sizeof (SwfdecFlvVideoTag));
     return SWFDEC_STATUS_OK;
   }
 
@@ -281,6 +279,7 @@ swfdec_flv_decoder_parse_video_tag (SwfdecFlvDecoder *flv, SwfdecBits *bits, gui
     dec->width = cairo_image_surface_get_width (surface);
     dec->height = cairo_image_surface_get_height (surface);
     swfdec_video_decoder_free (decoder);
+    cairo_surface_destroy (surface);
     return SWFDEC_STATUS_INIT;
   } else {
     return SWFDEC_STATUS_IMAGE;
@@ -291,24 +290,19 @@ static void
 swfdec_flv_decoder_parse_audio_tag (SwfdecFlvDecoder *flv, SwfdecBits *bits, guint timestamp)
 {
   SwfdecFlvAudioTag tag;
-  int rate;
-  gboolean stereo;
 
   if (flv->audio == NULL) {
-    SWFDEC_ERROR ("wow, audio tags in an FLV without audio!");
+    SWFDEC_INFO ("audio tags even though header didn't decalre them. Initializing...");
+    flv->audio = g_array_new (FALSE, FALSE, sizeof (SwfdecFlvAudioTag));
     return;
   }
 
   tag.timestamp = timestamp;
   tag.format = swfdec_bits_getbits (bits, 4);
-  rate = 44100 / (1 << (3 - swfdec_bits_getbits (bits, 2)));
-  tag.width = swfdec_bits_getbit (bits);
-  stereo = swfdec_bits_getbit (bits);
-  tag.original_format = SWFDEC_AUDIO_OUT_GET (stereo ? 2 : 1, rate);
+  tag.original_format = swfdec_audio_format_parse (bits);
   tag.buffer = swfdec_bits_get_buffer (bits, -1);
-  SWFDEC_LOG ("  format: %u", (guint) tag.format);
-  SWFDEC_LOG ("  rate: %d", rate);
-  SWFDEC_LOG ("  channels: %d", stereo ? 2 : 1);
+  SWFDEC_LOG ("  codec: %u", (guint) tag.format);
+  SWFDEC_LOG ("  format: %s", swfdec_audio_format_to_string (tag.original_format));
   if (tag.buffer == NULL) {
     SWFDEC_WARNING ("no buffer, ignoring");
     return;
@@ -361,20 +355,19 @@ swfdec_flv_decoder_parse_data_tag (SwfdecFlvDecoder *flv, SwfdecBits *bits, guin
 static SwfdecStatus
 swfdec_flv_decoder_parse_tag (SwfdecFlvDecoder *flv)
 {
-  SwfdecDecoder *dec = SWFDEC_DECODER (flv);
   SwfdecBuffer *buffer;
   SwfdecBits bits;
   guint size, type, timestamp;
   SwfdecStatus ret = SWFDEC_STATUS_OK;
 
-  buffer = swfdec_buffer_queue_peek (dec->queue, 4);
+  buffer = swfdec_buffer_queue_peek (flv->queue, 4);
   if (buffer == NULL)
     return SWFDEC_STATUS_NEEDBITS;
   swfdec_bits_init (&bits, buffer);
   swfdec_bits_get_u8 (&bits);
   size = swfdec_bits_get_bu24 (&bits);
   swfdec_buffer_unref (buffer);
-  buffer = swfdec_buffer_queue_pull (dec->queue, 11 + size);
+  buffer = swfdec_buffer_queue_pull (flv->queue, 11 + size);
   if (buffer == NULL)
     return SWFDEC_STATUS_NEEDBITS;
   swfdec_bits_init (&bits, buffer);
@@ -409,31 +402,45 @@ swfdec_flv_decoder_parse_tag (SwfdecFlvDecoder *flv)
 }
 
 static SwfdecStatus
-swfdec_flv_decoder_parse (SwfdecDecoder *dec)
+swfdec_flv_decoder_parse (SwfdecDecoder *dec, SwfdecBuffer *buffer)
 {
   SwfdecFlvDecoder *flv = SWFDEC_FLV_DECODER (dec);
-  SwfdecStatus ret;
+  SwfdecStatus status = 0;
 
-  switch (flv->state) {
-    case SWFDEC_STATE_HEADER:
-      ret = swfdec_flv_decoder_parse_header (flv);
-      break;
-    case SWFDEC_STATE_LAST_TAG:
-      ret = swfdec_flv_decoder_parse_last_tag (flv);
-      break;
-    case SWFDEC_STATE_TAG:
-      ret = swfdec_flv_decoder_parse_tag (flv);
-      break;
-    case SWFDEC_STATE_EOF:
-      ret = SWFDEC_STATUS_EOF;
-      break;
-    default:
-      g_assert_not_reached ();
-      ret = SWFDEC_STATUS_ERROR;
-      break;
-  }
+  swfdec_buffer_queue_push (flv->queue, buffer);
 
-  return ret;
+  do {
+    switch (flv->state) {
+      case SWFDEC_STATE_HEADER:
+	status |= swfdec_flv_decoder_parse_header (flv);
+	break;
+      case SWFDEC_STATE_LAST_TAG:
+	status |= swfdec_flv_decoder_parse_last_tag (flv);
+	break;
+      case SWFDEC_STATE_TAG:
+	status |= swfdec_flv_decoder_parse_tag (flv);
+	break;
+      case SWFDEC_STATE_EOF:
+	status |= SWFDEC_STATUS_EOF;
+	break;
+      default:
+	g_assert_not_reached ();
+	status |= SWFDEC_STATUS_ERROR;
+	break;
+    }
+  } while ((status & (SWFDEC_STATUS_EOF | SWFDEC_STATUS_NEEDBITS | SWFDEC_STATUS_ERROR)) == 0);
+
+  return status;
+}
+
+static SwfdecStatus
+swfdec_flv_decoder_eof (SwfdecDecoder *dec)
+{
+  SwfdecFlvDecoder *flv = SWFDEC_FLV_DECODER (dec);
+
+  flv->state = SWFDEC_STATE_EOF;
+
+  return 0;
 }
 
 static void
@@ -445,17 +452,19 @@ swfdec_flv_decoder_class_init (SwfdecFlvDecoderClass *class)
   object_class->dispose = swfdec_flv_decoder_dispose;
 
   decoder_class->parse = swfdec_flv_decoder_parse;
+  decoder_class->eof = swfdec_flv_decoder_eof;
 }
 
 static void
 swfdec_flv_decoder_init (SwfdecFlvDecoder *flv)
 {
   flv->state = SWFDEC_STATE_HEADER;
+  flv->queue = swfdec_buffer_queue_new ();
 }
 
 SwfdecBuffer *
 swfdec_flv_decoder_get_video (SwfdecFlvDecoder *flv, guint timestamp,
-    gboolean keyframe, SwfdecVideoFormat *format, guint *real_timestamp, guint *next_timestamp)
+    gboolean keyframe, guint *format, guint *real_timestamp, guint *next_timestamp)
 {
   guint id, offset;
   SwfdecFlvVideoTag *tag;
@@ -469,7 +478,7 @@ swfdec_flv_decoder_get_video (SwfdecFlvDecoder *flv, guint timestamp,
     if (real_timestamp)
       *real_timestamp = 0;
     if (format)
-      *format = SWFDEC_VIDEO_FORMAT_UNDEFINED;
+      *format = SWFDEC_VIDEO_CODEC_UNDEFINED;
     return NULL;
   }
   offset = g_array_index (flv->video, SwfdecFlvVideoTag, 0).timestamp;
@@ -520,7 +529,7 @@ swfdec_flv_decoder_get_video_info (SwfdecFlvDecoder *flv,
 
 SwfdecBuffer *
 swfdec_flv_decoder_get_audio (SwfdecFlvDecoder *flv, guint timestamp,
-    SwfdecAudioFormat *codec_format, gboolean *width, SwfdecAudioOut *format,
+    guint *codec, SwfdecAudioFormat *format,
     guint *real_timestamp, guint *next_timestamp)
 {
   guint id, offset;
@@ -534,12 +543,10 @@ swfdec_flv_decoder_get_audio (SwfdecFlvDecoder *flv, guint timestamp,
       *next_timestamp = 0;
     if (real_timestamp)
       *real_timestamp = 0;
-    if (codec_format)
-      *codec_format = SWFDEC_AUDIO_FORMAT_UNDEFINED;
-    if (width)
-      *width = TRUE;
+    if (codec)
+      *codec = SWFDEC_AUDIO_CODEC_UNDEFINED;
     if (format)
-      *format = SWFDEC_AUDIO_OUT_STEREO_44100;
+      *format = swfdec_audio_format_new (44100, 2, TRUE);
     return NULL;
   }
   offset = g_array_index (flv->audio, SwfdecFlvAudioTag, 0).timestamp;
@@ -554,10 +561,8 @@ swfdec_flv_decoder_get_audio (SwfdecFlvDecoder *flv, guint timestamp,
   tag = &g_array_index (flv->audio, SwfdecFlvAudioTag, id);
   if (real_timestamp)
     *real_timestamp = tag->timestamp - offset;
-  if (codec_format)
-    *codec_format = tag->format;
-  if (width)
-    *width = tag->width;
+  if (codec)
+    *codec = tag->format;
   if (format)
     *format = tag->original_format;
   return tag->buffer;
@@ -600,27 +605,6 @@ swfdec_flv_decoder_get_data (SwfdecFlvDecoder *flv, guint timestamp, guint *real
   return tag->buffer;
 }
 
-/*** HACK ***/
-
-/* This is a hack to allow native FLV playback IN SwfdecPlayer */
-
-#include "swfdec_loadertarget.h"
-#include "swfdec_net_stream.h"
-#include "swfdec_sprite.h"
-#include "swfdec_video_movie.h"
-
-#if 0
-static void
-notify_initialized (SwfdecPlayer *player, GParamSpec *pspec, SwfdecVideoMovie *movie)
-{
-  movie->video->width = player->width;
-  movie->video->height = player->height;
-
-  swfdec_movie_queue_update (SWFDEC_MOVIE (movie), SWFDEC_MOVIE_INVALID_MATRIX);
-  swfdec_movie_invalidate (SWFDEC_MOVIE (movie));
-}
-#endif
-
 gboolean
 swfdec_flv_decoder_is_eof (SwfdecFlvDecoder *flv)
 {
@@ -629,47 +613,3 @@ swfdec_flv_decoder_is_eof (SwfdecFlvDecoder *flv)
   return flv->state == SWFDEC_STATE_EOF;
 }
 
-void
-swfdec_flv_decoder_eof (SwfdecFlvDecoder *flv)
-{
-  g_return_if_fail (SWFDEC_IS_FLV_DECODER (flv));
-
-  flv->state = SWFDEC_STATE_EOF;
-}
-
-SwfdecMovie *
-swfdec_flv_decoder_add_movie (SwfdecFlvDecoder *flv, SwfdecMovie *parent)
-{
-  //g_assert_not_reached ();
-  return NULL;
-#if 0
-  SwfdecContent *content = swfdec_content_new (0);
-  SwfdecMovie *movie;
-  SwfdecVideo *video;
-  SwfdecConnection *conn;
-  SwfdecNetStream *stream;
-
-  /* set up the video movie */
-  video = g_object_new (SWFDEC_TYPE_VIDEO, NULL);
-  video->width = G_MAXUINT;
-  video->height = G_MAXUINT;
-  content->graphic = SWFDEC_GRAPHIC (video);
-  content->free = TRUE;
-  movie = swfdec_movie_new (parent, content);
-  g_object_weak_ref (G_OBJECT (movie), (GWeakNotify) g_object_unref, video);
-  g_signal_connect (SWFDEC_ROOT_MOVIE (parent)->player, "notify::initialized",
-      G_CALLBACK (notify_initialized), movie);
-  /* set up the playback stream */
-  conn = swfdec_connection_new (SWFDEC_ROOT_MOVIE (parent)->player->jscx);
-  stream = swfdec_net_stream_new (SWFDEC_ROOT_MOVIE (parent)->player, conn);
-  swfdec_net_stream_set_loader (stream, SWFDEC_ROOT_MOVIE (parent)->loader);
-  stream->flvdecoder = flv;
-  swfdec_video_movie_set_input (SWFDEC_VIDEO_MOVIE (movie), &stream->input);
-  swfdec_net_stream_set_playing (stream, TRUE);
-  g_object_unref (conn);
-  g_object_unref (stream);
-
-  return movie;
-#endif
-}
-
