@@ -33,6 +33,7 @@
 #include "swfdec_debug.h"
 #include "swfdec_swf_decoder.h"
 
+static const cairo_user_data_key_t key;
 
 static void merge_alpha (SwfdecImage * image, unsigned char *image_data,
     unsigned char *alpha);
@@ -50,10 +51,7 @@ swfdec_image_unload (SwfdecCached *cached)
   if (image->surface) {
     cairo_surface_destroy (image->surface);
     image->surface = NULL;
-  } else if (image->data) {
-    g_free (image->data);
   }
-  image->data = NULL;
 }
 
 static void
@@ -71,6 +69,8 @@ swfdec_image_dispose (GObject *object)
   }
 
   G_OBJECT_CLASS (swfdec_image_parent_class)->dispose (object);
+
+  g_assert (image->surface == NULL);
 }
 
 static void
@@ -169,25 +169,28 @@ static void
 swfdec_image_jpeg_load (SwfdecImage *image)
 {
   gboolean ret;
+  guint8 *data;
 
   if (image->jpegtables) {
     ret = swfdec_jpeg_decode_argb (
         image->jpegtables->data, image->jpegtables->length,
         image->raw_data->data, image->raw_data->length,
-        (void *)&image->data, &image->width, &image->height);
+        (void *) &data, &image->width, &image->height);
   } else {
     ret = swfdec_jpeg_decode_argb (
         image->raw_data->data, image->raw_data->length,
         NULL, 0,
-        (void *)&image->data, &image->width, &image->height);
+        (void *)&data, &image->width, &image->height);
   }
 
-  if (!ret) {
+  if (!ret)
     return;
-  }
 
   swfdec_cached_load (SWFDEC_CACHED (image), 4 * image->width * image->height);
-  image->rowstride = image->width * 4;
+  image->surface = cairo_image_surface_create_for_data (data, CAIRO_FORMAT_RGB24,
+      image->width, image->height, image->width * 4);
+  cairo_surface_set_user_data (image->surface, &key, data, g_free);
+  cairo_surface_reference (image->surface);
 
   SWFDEC_LOG ("  width = %d", image->width);
   SWFDEC_LOG ("  height = %d", image->height);
@@ -217,16 +220,19 @@ static void
 swfdec_image_jpeg2_load (SwfdecImage *image)
 {
   gboolean ret;
+  guint8 *data;
 
   ret = swfdec_jpeg_decode_argb (image->raw_data->data, image->raw_data->length,
       NULL, 0,
-      (void *)&image->data, &image->width, &image->height);
-  if (!ret) {
+      (void *)&data, &image->width, &image->height);
+  if (!ret)
     return;
-  }
 
   swfdec_cached_load (SWFDEC_CACHED (image), 4 * image->width * image->height);
-  image->rowstride = image->width * 4;
+  image->surface = cairo_image_surface_create_for_data (data, CAIRO_FORMAT_RGB24,
+      image->width, image->height, image->width * 4);
+  cairo_surface_set_user_data (image->surface, &key, data, g_free);
+  cairo_surface_reference (image->surface);
 
   SWFDEC_LOG ("  width = %d", image->width);
   SWFDEC_LOG ("  height = %d", image->height);
@@ -259,6 +265,7 @@ swfdec_image_jpeg3_load (SwfdecImage *image)
   SwfdecBuffer *buffer;
   int jpeg_length;
   gboolean ret;
+  guint8 *data;
 
   swfdec_bits_init (&bits, image->raw_data);
 
@@ -269,19 +276,17 @@ swfdec_image_jpeg3_load (SwfdecImage *image)
 
   ret = swfdec_jpeg_decode_argb (buffer->data, buffer->length,
       NULL, 0,
-      (void *)&image->data, &image->width, &image->height);
+      (void *)&data, &image->width, &image->height);
   swfdec_buffer_unref (buffer);
 
-  if (!ret) {
+  if (!ret)
     return;
-  }
 
   swfdec_cached_load (SWFDEC_CACHED (image), 4 * image->width * image->height);
-  image->rowstride = image->width * 4;
 
   buffer = swfdec_bits_decompress (&bits, -1, image->width * image->height);
   if (buffer) {
-    merge_alpha (image, image->data, buffer->data);
+    merge_alpha (image, data, buffer->data);
     swfdec_buffer_unref (buffer);
   } else {
     SWFDEC_WARNING ("cannot set alpha channel information, decompression failed");
@@ -289,6 +294,11 @@ swfdec_image_jpeg3_load (SwfdecImage *image)
 
   SWFDEC_LOG ("  width = %d", image->width);
   SWFDEC_LOG ("  height = %d", image->height);
+
+  image->surface = cairo_image_surface_create_for_data (data,
+      CAIRO_FORMAT_ARGB32, image->width, image->height, image->width * 4);
+  cairo_surface_set_user_data (image->surface, &key, data, g_free);
+  cairo_surface_reference (image->surface);
 }
 
 static void
@@ -299,7 +309,7 @@ merge_alpha (SwfdecImage * image, unsigned char *image_data,
   unsigned char *p;
 
   for (y = 0; y < image->height; y++) {
-    p = image_data + y * image->rowstride;
+    p = image_data + y * image->width * 4;
     for (x = 0; x < image->width; x++) {
       p[SWFDEC_COLOR_INDEX_ALPHA] = *alpha;
       p += 4;
@@ -315,6 +325,7 @@ swfdec_image_lossless_load (SwfdecImage *image)
   guint color_table_size;
   unsigned char *ptr;
   SwfdecBits bits;
+  guint8 *data;
   int have_alpha = (image->type == SWFDEC_IMAGE_TYPE_LOSSLESS2);
 
   swfdec_bits_init (&bits, image->raw_data);
@@ -346,15 +357,14 @@ swfdec_image_lossless_load (SwfdecImage *image)
     guint i;
     guint rowstride = (image->width + 3) & ~3;
 
-    image->data = g_malloc (4 * image->width * image->height);
-    image->rowstride = image->width * 4;
+    data = g_malloc (4 * image->width * image->height);
 
     if (have_alpha) {
       buffer = swfdec_bits_decompress (&bits, -1, color_table_size * 4 + rowstride * image->height);
       if (buffer == NULL) {
 	SWFDEC_ERROR ("failed to decompress data");
-	memset (image->data, 0, 4 * image->width * image->height);
-	return;
+	memset (data, 0, 4 * image->width * image->height);
+	goto out;
       }
       ptr = buffer->data;
       for (i = 0; i < color_table_size; i++) {
@@ -375,8 +385,8 @@ swfdec_image_lossless_load (SwfdecImage *image)
       buffer = swfdec_bits_decompress (&bits, -1, color_table_size * 3 + rowstride * image->height);
       if (buffer == NULL) {
 	SWFDEC_ERROR ("failed to decompress data");
-	memset (image->data, 0, 4 * image->width * image->height);
-	return;
+	memset (data, 0, 4 * image->width * image->height);
+	goto out;
       }
       ptr = buffer->data;
       for (i = color_table_size - 1; i < color_table_size; i--) {
@@ -398,7 +408,7 @@ swfdec_image_lossless_load (SwfdecImage *image)
       }
       indexed_data = ptr + color_table_size * 3;
     }
-    swfdec_image_colormap_decode (image, image->data, indexed_data,
+    swfdec_image_colormap_decode (image, data, indexed_data,
 	ptr, color_table_size);
 
     swfdec_buffer_unref (buffer);
@@ -414,13 +424,12 @@ swfdec_image_lossless_load (SwfdecImage *image)
     }
 
     buffer = swfdec_bits_decompress (&bits, -1, 2 * ((image->width + 1) & ~1) * image->height);
-    image->data = g_malloc (4 * image->width * image->height);
-    idata = image->data;
-    image->rowstride = image->width * 4;
+    data = g_malloc (4 * image->width * image->height);
+    idata = data;
     if (buffer == NULL) {
       SWFDEC_ERROR ("failed to decompress data");
-      memset (image->data, 0, 4 * image->width * image->height);
-      return;
+      memset (data, 0, 4 * image->width * image->height);
+      goto out;
     }
     ptr = buffer->data;
 
@@ -439,18 +448,16 @@ swfdec_image_lossless_load (SwfdecImage *image)
 	ptr += 2;
     }
     swfdec_buffer_unref (buffer);
-  }
-  if (format == 5) {
+  } else if (format == 5) {
     SwfdecBuffer *buffer;
     int i, j;
     buffer = swfdec_bits_decompress (&bits, -1, 4 * image->width * image->height);
-    image->rowstride = 4 * image->width;
     if (buffer == NULL) {
       SWFDEC_ERROR ("failed to decompress data");
-      image->data = g_malloc0 (4 * image->width * image->height);
-      return;
+      data = g_malloc0 (4 * image->width * image->height);
+      goto out;
     }
-    ptr = image->data = buffer->data;
+    ptr = data = buffer->data;
     /* image is stored in 0RGB format.  We use ARGB/BGRA. */
     for (j = 0; j < image->height; j++) {
       for (i = 0; i < image->width; i++) {
@@ -463,7 +470,17 @@ swfdec_image_lossless_load (SwfdecImage *image)
     buffer->data = NULL;
     buffer->length = 0;
     swfdec_buffer_unref (buffer);
+  } else {
+    SWFDEC_ERROR ("unknown lossless image format %u", format);
+    return;
   }
+
+out:
+  image->surface = cairo_image_surface_create_for_data (data, 
+      have_alpha ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24, 
+      image->width, image->height, image->width * 4);
+  cairo_surface_set_user_data (image->surface, &key, data, g_free);
+  cairo_surface_reference (image->surface);
 }
 
 int
@@ -543,13 +560,37 @@ swfdec_image_colormap_decode (SwfdecImage * image,
   }
 }
 
-static gboolean
-swfdec_image_ensure_loaded (SwfdecImage *image)
+static cairo_status_t
+swfdec_image_png_read (void *bitsp, unsigned char *data, unsigned int length)
+{
+  SwfdecBits *bits = bitsp;
+  const guint8 *ptr;
+
+  ptr = bits->ptr;
+  if (swfdec_bits_skip_bytes (bits, length) != length)
+    return CAIRO_STATUS_READ_ERROR;
+
+  memcpy (data, ptr, length);
+  return CAIRO_STATUS_SUCCESS;
+}
+
+static void
+swfdec_image_png_load (SwfdecImage *image)
+{
+  SwfdecBits bits;
+
+  swfdec_bits_init (&bits, image->raw_data);
+  image->surface = cairo_image_surface_create_from_png_stream (
+      swfdec_image_png_read, &bits);
+}
+
+cairo_surface_t *
+swfdec_image_create_surface (SwfdecImage *image)
 {
   if (image->raw_data == NULL)
-    return FALSE;
+    return NULL;
 
-  if (image->data == NULL) {
+  if (image->surface == NULL) {
     switch (image->type) {
       case SWFDEC_IMAGE_TYPE_JPEG:
 	swfdec_image_jpeg_load (image);
@@ -566,65 +607,28 @@ swfdec_image_ensure_loaded (SwfdecImage *image)
       case SWFDEC_IMAGE_TYPE_LOSSLESS2:
 	swfdec_image_lossless_load (image);
 	break;
+      case SWFDEC_IMAGE_TYPE_PNG:
+	swfdec_image_png_load (image);
+	break;
       case SWFDEC_IMAGE_TYPE_UNKNOWN:
       default:
 	g_assert_not_reached ();
 	break;
     }
-    if (image->data == NULL) {
-      SWFDEC_WARNING ("failed to load image data");
-      return FALSE;
+    if (image->surface == NULL) {
+      SWFDEC_WARNING ("failed to decode image");
+      return NULL;
     }
   } else {
     swfdec_cached_use (SWFDEC_CACHED (image));
   }
-  return TRUE;
-}
-
-static gboolean
-swfdec_image_has_alpha (SwfdecImage *image)
-{
-  return image->type == SWFDEC_IMAGE_TYPE_LOSSLESS2 ||
-      image->type == SWFDEC_IMAGE_TYPE_JPEG3;
-}
-
-cairo_surface_t *
-swfdec_image_create_surface (SwfdecImage *image)
-{
-  static const cairo_user_data_key_t key;
-
-  g_return_val_if_fail (SWFDEC_IS_IMAGE (image), NULL);
-
-  if (!swfdec_image_ensure_loaded (image))
-    return NULL;
-  if (image->surface) {
-    cairo_surface_reference (image->surface);
-    return image->surface;
-  }
-
-  if (swfdec_image_has_alpha (image)) {
-    cairo_surface_t *surface;
-    guint8 *data;
-    surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 
-	image->width, image->height);
-    /* FIXME: only works if rowstride == image->width * 4 */
-    data = cairo_image_surface_get_data (surface);
-    memcpy (data, image->data, image->width * image->height * 4);
-    return surface;
-  } else {
-    image->surface = cairo_image_surface_create_for_data (image->data,
-	CAIRO_FORMAT_RGB24, image->width, image->height, image->rowstride);
-    cairo_surface_set_user_data (image->surface, &key, image->data, g_free);
-    cairo_surface_reference (image->surface);
-    return image->surface;
-  }
+  return cairo_surface_reference (image->surface);
 }
 
 cairo_surface_t *
 swfdec_image_create_surface_transformed (SwfdecImage *image, const SwfdecColorTransform *trans)
 {
-  static const cairo_user_data_key_t key;
-  cairo_surface_t *surface;
+  cairo_surface_t *surface, *source;
   guint8 *tdata;
   const guint8 *sdata;
   guint i, n;
@@ -637,24 +641,66 @@ swfdec_image_create_surface_transformed (SwfdecImage *image, const SwfdecColorTr
   if (swfdec_color_transform_is_identity (trans))
     return swfdec_image_create_surface (image);
 
-  if (!swfdec_image_ensure_loaded (image))
-    return NULL;
-
+  source = swfdec_image_create_surface (image);
   tdata = g_try_malloc (image->width * image->height * 4);
   if (!tdata) {
     SWFDEC_ERROR ("failed to allocate memory for transformed image");
+    cairo_surface_destroy (source);
     return NULL;
   }
-  sdata = image->data;
+  /* FIXME: This code assumes a rowstride of 4 * width */
+  sdata = cairo_image_surface_get_data (source);
   n = image->width * image->height;
   for (i = 0; i < n; i++) {
     ((guint32 *) tdata)[i] = swfdec_color_apply_transform_premultiplied (((guint32 *) sdata)[i], trans);
     /* optimization: check for alpha channel to speed up compositing */
     has_alpha = tdata[4 * i + SWFDEC_COLOR_INDEX_ALPHA] != 0xFF;
   }
+  cairo_surface_destroy (source);
   surface = cairo_image_surface_create_for_data (tdata,
       has_alpha ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24, 
       image->width, image->height, image->width * 4);
   cairo_surface_set_user_data (surface, &key, tdata, g_free);
   return surface;
 }
+
+/* NB: must be at least SWFDEC_DECODER_DETECT_LENGTH bytes */
+SwfdecImageType
+swfdec_image_detect (const guint8 *data)
+{
+  g_return_val_if_fail (data != NULL, SWFDEC_IMAGE_TYPE_UNKNOWN);
+
+  if (data[0] == 0xFF && data[1] == 0xD8)
+    return SWFDEC_IMAGE_TYPE_JPEG2;
+  else if (data[0] == 89 && data[1] == 'P' &&
+      data[2] == 'N' && data[3] == 'G')
+    return SWFDEC_IMAGE_TYPE_PNG;
+  else
+    return SWFDEC_IMAGE_TYPE_UNKNOWN;
+}
+
+SwfdecImage *
+swfdec_image_new (SwfdecBuffer *buffer)
+{
+  SwfdecImage *image;
+  SwfdecImageType type;
+
+  g_return_val_if_fail (buffer != NULL, NULL);
+
+  /* check type of the image */
+  if (buffer->length < 4)
+    goto fail;
+  type = swfdec_image_detect (buffer->data);
+  if (type == SWFDEC_IMAGE_TYPE_UNKNOWN)
+    goto fail;
+
+  image = g_object_new (SWFDEC_TYPE_IMAGE, NULL);
+  image->type = type;
+  image->raw_data = buffer;
+
+  return image;
+
+fail:
+  swfdec_buffer_unref (buffer);
+  return NULL;
+}
