@@ -62,6 +62,8 @@
  * @SWFDEC_LOADER_DATA_FLV: Data describing a Flash video stream.
  * @SWFDEC_LOADER_DATA_XML: Data in XML format.
  * @SWFDEC_LOADER_DATA_TEXT: Textual data.
+ * @SWFDEC_LOADER_DATA_JPEG: a JPEG image
+ * @SWFDEC_LOADER_DATA_PNG: a PNG image
  *
  * This type describes the different types of data that can be loaded inside 
  * Swfdec. Swfdec identifies its data streams and you can use the 
@@ -112,7 +114,7 @@ swfdec_loader_get_property (GObject *object, guint param_id, GValue *value,
       g_value_set_enum (value, loader->data_type);
       break;
     case PROP_SIZE:
-      g_value_set_ulong (value, loader->size);
+      g_value_set_long (value, loader->size);
       break;
     case PROP_LOADED:
       g_value_set_ulong (value, swfdec_loader_get_loaded (loader));
@@ -137,8 +139,8 @@ swfdec_loader_set_property (GObject *object, guint param_id, const GValue *value
       swfdec_loader_error (loader, g_value_get_string (value));
       break;
     case PROP_SIZE:
-      if (loader->size == 0 && g_value_get_ulong (value) > 0)
-	swfdec_loader_set_size (loader, g_value_get_ulong (value));
+      if (loader->size == -1 && g_value_get_long (value) >= 0)
+	swfdec_loader_set_size (loader, g_value_get_long (value));
       break;
     case PROP_URL:
       loader->url = g_value_dup_boxed (value);
@@ -186,8 +188,8 @@ swfdec_loader_class_init (SwfdecLoaderClass *klass)
       g_param_spec_enum ("data-type", "data type", "the data's type as identified by Swfdec",
 	  SWFDEC_TYPE_LOADER_DATA_TYPE, SWFDEC_LOADER_DATA_UNKNOWN, G_PARAM_READABLE));
   g_object_class_install_property (object_class, PROP_SIZE,
-      g_param_spec_ulong ("size", "size", "amount of bytes in loader",
-	  0, G_MAXULONG, 0, G_PARAM_READWRITE));
+      g_param_spec_long ("size", "size", "amount of bytes in loader",
+	  -1, G_MAXLONG, -1, G_PARAM_READWRITE));
   g_object_class_install_property (object_class, PROP_LOADED,
       g_param_spec_ulong ("loaded", "loaded", "bytes already loaded",
 	  0, G_MAXULONG, 0, G_PARAM_READWRITE));
@@ -201,63 +203,93 @@ swfdec_loader_init (SwfdecLoader *loader)
 {
   loader->queue = swfdec_buffer_queue_new ();
   loader->data_type = SWFDEC_LOADER_DATA_UNKNOWN;
+
+  loader->size = -1;
 }
 
 /*** INTERNAL API ***/
 
 static void
-swfdec_loader_perform_open (gpointer loaderp, gpointer unused)
+swfdec_loader_process (gpointer loaderp, gpointer unused)
 {
   SwfdecLoader *loader = loaderp;
 
-  swfdec_loader_target_open (loader->target, loader);
+  g_assert (loader->target != NULL);
+
+  loader->queued = FALSE;
+  if (loader->state == loader->processed_state)
+    return;
+  g_assert (loader->state != SWFDEC_LOADER_STATE_CLOSED);
+  if (loader->state == SWFDEC_LOADER_STATE_ERROR) {
+    swfdec_loader_target_error (loader->target, loader);
+    return;
+  }
+  g_object_ref (loader);
+  while (loader->state != loader->processed_state) {
+    if (loader->processed_state == SWFDEC_LOADER_STATE_NEW) {
+      loader->processed_state = SWFDEC_LOADER_STATE_OPEN;
+      swfdec_loader_target_open (loader->target, loader);
+    } else if (loader->processed_state == SWFDEC_LOADER_STATE_OPEN) {
+      loader->processed_state = SWFDEC_LOADER_STATE_READING;
+      swfdec_loader_target_parse (loader->target, loader);
+    } else if (loader->processed_state == SWFDEC_LOADER_STATE_READING) {
+      loader->processed_state = SWFDEC_LOADER_STATE_EOF;
+      swfdec_loader_target_eof (loader->target, loader);
+    }
+    /* stupid reentrancy */
+    if (loader->processed_state == SWFDEC_LOADER_STATE_NEW)
+      break;
+  }
+  g_object_unref (loader);
 }
 
 static void
-swfdec_loader_perform_eof (gpointer loaderp, gpointer unused)
+swfdec_loader_queue_processing (SwfdecLoader *loader)
 {
-  SwfdecLoader *loader = loaderp;
-
-  swfdec_loader_target_eof (loader->target, loader);
-}
-
-static void
-swfdec_loader_perform_error (gpointer loaderp, gpointer unused)
-{
-  SwfdecLoader *loader = loaderp;
-
-  swfdec_loader_target_error (loader->target, loader);
-}
-
-static void
-swfdec_loader_perform_push (gpointer loaderp, gpointer unused)
-{
-  SwfdecLoader *loader = loaderp;
-
-  swfdec_loader_target_parse (loader->target, loader);
+  if (loader->queued)
+    return;
+  loader->queued = TRUE;
+  if (loader->target) {
+    g_assert (loader->player);
+    swfdec_player_add_external_action (loader->player, loader,
+	swfdec_loader_process, NULL);
+  }
 }
 
 SwfdecLoader *
-swfdec_loader_load (SwfdecLoader *loader, const char *url_string,
+swfdec_loader_load (SwfdecLoader *loader, const SwfdecURL *url,
     SwfdecLoaderRequest request, const char *data, gsize data_len)
 {
   SwfdecLoader *ret;
   SwfdecLoaderClass *klass;
-  SwfdecURL *url;
 
   g_return_val_if_fail (SWFDEC_IS_LOADER (loader), NULL);
-  g_return_val_if_fail (url_string != NULL, NULL);
+  g_return_val_if_fail (url != NULL, NULL);
   g_return_val_if_fail (data != NULL || data_len == 0, NULL);
 
   klass = SWFDEC_LOADER_GET_CLASS (loader);
   g_return_val_if_fail (klass->load != NULL, NULL);
-  url = swfdec_url_new_relative (loader->url, url_string);
   ret = g_object_new (G_OBJECT_CLASS_TYPE (klass), "url", url, NULL);
-  swfdec_url_free (url);
   klass->load (ret, loader, request, data, data_len);
   return ret;
 }
 
+void
+swfdec_loader_close (SwfdecLoader *loader)
+{
+  SwfdecLoaderClass *klass;
+
+  g_return_if_fail (SWFDEC_IS_LOADER (loader));
+  klass = SWFDEC_LOADER_GET_CLASS (loader);
+  
+  if (klass->close)
+    klass->close (loader);
+  if (loader->state != SWFDEC_LOADER_STATE_ERROR) {
+    loader->state = SWFDEC_LOADER_STATE_CLOSED;
+    loader->processed_state = SWFDEC_LOADER_STATE_CLOSED;
+  }
+}
+
 void
 swfdec_loader_set_target (SwfdecLoader *loader, SwfdecLoaderTarget *target)
 {
@@ -267,38 +299,12 @@ swfdec_loader_set_target (SwfdecLoader *loader, SwfdecLoaderTarget *target)
   if (loader->target) {
     swfdec_player_remove_all_external_actions (loader->player, loader);
   }
+  loader->queued = FALSE;
   loader->target = target;
+  loader->processed_state = SWFDEC_LOADER_STATE_NEW;
   if (target) {
     loader->player = swfdec_loader_target_get_player (target);
-    switch (loader->state) {
-      case SWFDEC_LOADER_STATE_NEW:
-	break;
-      case SWFDEC_LOADER_STATE_OPEN:
-	swfdec_player_add_external_action (loader->player, loader,
-	    swfdec_loader_perform_open, NULL);
-	break;
-      case SWFDEC_LOADER_STATE_READING:
-	swfdec_player_add_external_action (loader->player, loader,
-	    swfdec_loader_perform_open, NULL);
-	swfdec_player_add_external_action (loader->player, loader,
-	    swfdec_loader_perform_push, NULL);
-	break;
-      case SWFDEC_LOADER_STATE_EOF:
-	swfdec_player_add_external_action (loader->player, loader,
-	    swfdec_loader_perform_open, NULL);
-	swfdec_player_add_external_action (loader->player, loader,
-	    swfdec_loader_perform_push, NULL);
-	swfdec_player_add_external_action (loader->player, loader,
-	    swfdec_loader_perform_eof, NULL);
-	break;
-      case SWFDEC_LOADER_STATE_ERROR:
-	swfdec_player_add_external_action (loader->player, loader,
-	    swfdec_loader_perform_error, NULL);
-	break;
-      default:
-	g_assert_not_reached ();
-	break;
-    }
+    swfdec_loader_queue_processing (loader);
   } else {
     loader->player = NULL;
   }
@@ -326,11 +332,10 @@ swfdec_loader_error (SwfdecLoader *loader, const char *error)
     return;
   }
 
+  SWFDEC_ERROR ("error in loader %p: %s", loader, error);
   loader->state = SWFDEC_LOADER_STATE_ERROR;
   loader->error = g_strdup (error);
-  if (loader->target)
-    swfdec_player_add_external_action (loader->player, loader,
-	swfdec_loader_perform_error, NULL);
+  swfdec_loader_queue_processing (loader);
 }
 
 /**
@@ -355,8 +360,7 @@ swfdec_loader_open (SwfdecLoader *loader, const char *url)
     loader->url = swfdec_url_new (url);
     g_object_notify (G_OBJECT (loader), "url");
   }
-  if (loader->player)
-    swfdec_player_add_external_action (loader->player, loader, swfdec_loader_perform_open, NULL);
+  swfdec_loader_queue_processing (loader);
 }
 
 /**
@@ -378,9 +382,9 @@ swfdec_loader_push (SwfdecLoader *loader, SwfdecBuffer *buffer)
   swfdec_buffer_queue_push (loader->queue, buffer);
   g_object_notify (G_OBJECT (loader), "loaded");
   loader->state = SWFDEC_LOADER_STATE_READING;
-  if (loader->player)
-    swfdec_player_add_external_action (loader->player, loader,
-	swfdec_loader_perform_push, NULL);
+  if (loader->processed_state == SWFDEC_LOADER_STATE_READING)
+    loader->processed_state = SWFDEC_LOADER_STATE_OPEN;
+  swfdec_loader_queue_processing (loader);
 }
 
 /**
@@ -402,9 +406,7 @@ swfdec_loader_eof (SwfdecLoader *loader)
   }
   g_object_notify (G_OBJECT (loader), "eof");
   loader->state = SWFDEC_LOADER_STATE_EOF;
-  if (loader->player)
-    swfdec_player_add_external_action (loader->player, loader,
-	swfdec_loader_perform_eof, NULL);
+  swfdec_loader_queue_processing (loader);
 }
 
 /**
@@ -429,7 +431,7 @@ swfdec_loader_get_filename (SwfdecLoader *loader)
   url = swfdec_loader_get_url (loader);
   path = swfdec_url_get_path (url);
   if (path) {
-    char *s = strchr (path, '/');
+    char *s = strrchr (path, '/');
     if (s)
       path = s + 1;
     if (path[0] == 0)
@@ -512,8 +514,8 @@ void
 swfdec_loader_set_size (SwfdecLoader *loader, gulong size)
 {
   g_return_if_fail (SWFDEC_IS_LOADER (loader));
-  g_return_if_fail (loader->size == 0);
-  g_return_if_fail (size > 0);
+  g_return_if_fail (loader->size == -1);
+  g_return_if_fail (size <= G_MAXLONG);
 
   loader->size = size;
   g_object_notify (G_OBJECT (loader), "size");
@@ -523,15 +525,15 @@ swfdec_loader_set_size (SwfdecLoader *loader, gulong size)
  * swfdec_loader_get_size:
  * @loader: a #SwfdecLoader
  *
- * Queries the amount of bytes inside @loader. If the size is unknown, 0 is 
- * returned.
+ * Queries the amount of bytes inside @loader. If the size is unknown, -1 is 
+ * returned. Otherwise the number is greater or equal to 0.
  *
- * Returns: the total number of bytes for this loader or 0 if unknown
+ * Returns: the total number of bytes for this loader or -1 if unknown
  **/
-gulong
+glong
 swfdec_loader_get_size (SwfdecLoader *loader)
 {
-  g_return_val_if_fail (SWFDEC_IS_LOADER (loader), 0);
+  g_return_val_if_fail (SWFDEC_IS_LOADER (loader), -1);
 
   return loader->size;
 }
@@ -577,6 +579,10 @@ swfdec_loader_data_type_get_extension (SwfdecLoaderDataType type)
       return "xml";
     case SWFDEC_LOADER_DATA_TEXT:
       return "txt";
+    case SWFDEC_LOADER_DATA_JPEG:
+      return "jpg";
+    case SWFDEC_LOADER_DATA_PNG:
+      return "png";
     default:
       g_warning ("unknown data type %u", type);
       return "";
@@ -602,54 +608,6 @@ swfdec_urlencode_append_string (GString *str, const char *s)
   }
 }
 
-static char *
-swfdec_urldecode_one_string (const char *s, const char **out)
-{
-  GString *ret = g_string_new ("");
-
-  while (*s) {
-    if (strchr (urlencode_unescaped, *s)) {
-      g_string_append_c (ret, *s);
-    } else if (*s == '+') {
-      g_string_append_c (ret, ' ');
-    } else if (*s == '%') {
-      guint byte;
-      s++;
-      if (*s >= '0' && *s <= '9') {
-	byte = *s - '0';
-      } else if (*s >= 'A' && *s <= 'F') {
-	byte = *s - 'A' + 10;
-      } else if (*s >= 'a' && *s <= 'f') {
-	byte = *s - 'a' + 10;
-      } else {
-	g_string_free (ret, TRUE);
-	*out = s;
-	return NULL;
-      }
-      byte *= 16;
-      s++;
-      if (*s >= '0' && *s <= '9') {
-	byte += *s - '0';
-      } else if (*s >= 'A' && *s <= 'F') {
-	byte += *s - 'A' + 10;
-      } else if (*s >= 'a' && *s <= 'f') {
-	byte += *s - 'a' + 10;
-      } else {
-	g_string_free (ret, TRUE);
-	*out = s;
-	return NULL;
-      }
-      g_assert (byte < 256);
-      g_string_append_c (ret, byte);
-    } else {
-      break;
-    }
-    s++;
-  }
-  *out = s;
-  return g_string_free (ret, FALSE);
-}
-
 /**
  * swfdec_string_append_urlencoded:
  * @str: a #GString
@@ -673,61 +631,3 @@ swfdec_string_append_urlencoded (GString *str, const char *name, const char *val
     swfdec_urlencode_append_string (str, value);
 }
 
-/**
- * swfdec_urldecode_one:
- * @string: string in 'application/x-www-form-urlencoded' form
- * @name: pointer that will hold a newly allocated string for the name of the 
- *        parsed property or NULL
- * @value: pointer that will hold a newly allocated string containing the 
- *         value of the parsed property or NULL
- * @end: If not %NULL, on success, pointer to the first byte in @s that was 
- *       not parsed. On failure it will point to the byte causing the problem
- *
- * Tries to parse the given @string into a name/value pair, assuming the string 
- * is in the application/x-www-form-urlencoded format. If the parsing succeeds,
- * @name and @value will contain the parsed values and %TRUE will be returned.
- *
- * Returns: %TRUE if parsing the property succeeded, %FALSE otherwise
- */
-gboolean
-swfdec_urldecode_one (const char *string, char **name, char **value, const char **end)
-{
-  char *name_str, *value_str;
-
-  g_return_val_if_fail (string != NULL, FALSE);
-
-  name_str = swfdec_urldecode_one_string (string, &string);
-  if (name_str == NULL)
-    goto fail;
-  if (*string != '=') {
-    g_free (name_str);
-    goto fail;
-  }
-  string++;
-  value_str = swfdec_urldecode_one_string (string, &string);
-  if (value_str == NULL) {
-    g_free (name_str);
-    goto fail;
-  }
-
-  if (name)
-    *name = name_str;
-  else
-    g_free (name_str);
-  if (value)
-    *value = value_str;
-  else
-    g_free (value_str);
-  if (end)
-    *end = string;
-  return TRUE;
-
-fail:
-  if (name)
-    *name = NULL;
-  if (value)
-    *value = NULL;
-  if (end)
-    *end = string;
-  return FALSE;
-}
