@@ -30,23 +30,21 @@ typedef enum {
   SWFDEC_LOADER_STATE_OPEN,		/* loader is opened and has got the HTTP headers */
   SWFDEC_LOADER_STATE_READING,		/* loader has read some bytes of data and is still reading */
   SWFDEC_LOADER_STATE_EOF,		/* swfdec_loader_eof() has been called */
+  SWFDEC_LOADER_STATE_CLOSED,		/* loader has been closed by Swfdec */
   SWFDEC_LOADER_STATE_ERROR		/* loader is in error state */
 } SwfdecLoaderState;
 
 SwfdecLoader *		swfdec_loader_load		(SwfdecLoader *		loader,
-							 const char *		url,
+							 const SwfdecURL *    	url,
 							 SwfdecLoaderRequest	request,
 							 const char *		data,
 							 gsize			data_len);
+void			swfdec_loader_close		(SwfdecLoader *		loader);
 void			swfdec_loader_set_target	(SwfdecLoader *		loader,
 							 SwfdecLoaderTarget *	target);
 void			swfdec_loader_set_data_type	(SwfdecLoader *		loader,
 							 SwfdecLoaderDataType	type);
 
-gboolean		swfdec_urldecode_one		(const char *		string,
-							 char **		name,
-							 char **		value,
-							 const char **		end);
 void			swfdec_string_append_urlencoded	(GString *		str,
 							 const char *		name,
 							 const char *		value);
