@@ -58,7 +58,7 @@ struct _SwfdecContent {
   gboolean		has_color_transform : 1;
   char *		name;
   SwfdecEventList *	events;
-  cairo_operator_t	operator;	/* operator to use when painting (aka blend mode) */   
+  guint			blend_mode;	/* operator to use when painting (aka blend mode) */   
 
   /* only used by buttons */
   SwfdecContent *	sequence;
@@ -75,13 +75,24 @@ struct _SwfdecContent {
 #define SWFDEC_MOVIE_CLASS(klass)            (G_TYPE_CHECK_CLASS_CAST ((klass), SWFDEC_TYPE_MOVIE, SwfdecMovieClass))
 #define SWFDEC_MOVIE_GET_CLASS(obj)          (G_TYPE_INSTANCE_GET_CLASS ((obj), SWFDEC_TYPE_MOVIE, SwfdecMovieClass))
 
+/* NB: each following state includes the previous */
 typedef enum {
-  SWFDEC_MOVIE_UP_TO_DATE = 0,
-  SWFDEC_MOVIE_INVALID_CHILDREN,
-  SWFDEC_MOVIE_INVALID_EXTENTS,
-  SWFDEC_MOVIE_INVALID_MATRIX
+  SWFDEC_MOVIE_UP_TO_DATE = 0,		/* everything OK */
+  SWFDEC_MOVIE_INVALID_CHILDREN,	/* call update on children */
+  SWFDEC_MOVIE_INVALID_EXTENTS,		/* recalculate extents */
+  SWFDEC_MOVIE_INVALID_CONTENTS,	/* trigger an invalidation */
+  SWFDEC_MOVIE_INVALID_MATRIX		/* matrix is invalid, recalculate */
 } SwfdecMovieCacheState;
 
+typedef void (*SwfdecMovieVariableListenerFunction) (SwfdecAsObject *object,
+    const char *name, const SwfdecAsValue *val);
+
+typedef struct {
+  SwfdecAsObject *			object;
+  const char *				name;
+  SwfdecMovieVariableListenerFunction	function;
+} SwfdecMovieVariableListener;
+
 struct _SwfdecMovie {
   SwfdecAsObject	object;
 
@@ -91,6 +102,7 @@ struct _SwfdecMovie {
   int			depth;			/* depth of movie (equals content->depth unless explicitly set) */
   SwfdecMovieCacheState	cache_state;		/* whether we are up to date */
   SwfdecMovieState	state;			/* state the movie is in */
+  GSList		*variable_listeners;	/* textfield's listening to changes in variables - SwfdecMovieVariableListener */
 
   /* static properties (set by PlaceObject tags) */
   const char *		original_name;		/* the original name - GC'd and static */
@@ -102,7 +114,7 @@ struct _SwfdecMovie {
 
   /* parenting information */
   SwfdecMovie *		parent;			/* movie that contains us or NULL for root movies */
-  SwfdecSwfInstance *	swf;			/* the instance that created us */
+  SwfdecResource *	resource;     		/* the resource that created us */
 
   /* positioning - the values are applied in this order */
   SwfdecRect		extents;		/* the extents occupied after transform is applied */
@@ -114,16 +126,25 @@ struct _SwfdecMovie {
   cairo_matrix_t	matrix;			/* cairo matrix computed from above and content->transform */
   cairo_matrix_t	inverse_matrix;		/* the inverse of the cairo matrix */
   SwfdecColorTransform	color_transform;	/* scripted color transformation */
+  guint			blend_mode;		/* blend mode to use - see to-cairo conversion code for what they mean */
 
   /* iteration state */
   gboolean		visible;		/* whether we currently can be seen or iterate */
   gboolean		will_be_removed;	/* it's known that this movie will not survive the next iteration */
 
+  /* drawing state */
+  /* FIXME: could it be that shape drawing (SwfdecGraphicMovie etc) uses these same objects? */
+  SwfdecImage *		image;			/* image loaded via loadMovie */
+  SwfdecRect		draw_extents;		/* extents of the items in the following list */
+  GSList *		draws;			/* all the items to draw */
+  SwfdecDraw *		draw_fill;	      	/* current fill style or NULL */
+  SwfdecDraw *		draw_line;	      	/* current line style or NULL */
+  int			draw_x;			/* current x position for drawing */
+  int			draw_y;			/* current y position for drawing */
+
   /* leftover unimplemented variables from the Actionscript spec */
 #if 0
   int droptarget;
-  char *target;
-  char *url;
 #endif
 };
 
@@ -135,13 +156,13 @@ struct _SwfdecMovieClass {
   void			(* finish_movie)	(SwfdecMovie *		movie);
   void			(* replace)		(SwfdecMovie *		movie,
 						 SwfdecGraphic *	graphic);
+  void			(* set_ratio)		(SwfdecMovie *		movie);
   void			(* update_extents)	(SwfdecMovie *		movie,
 						 SwfdecRect *   	extents);
   void			(* render)		(SwfdecMovie *		movie, 
 						 cairo_t *		cr,
 						 const SwfdecColorTransform *trans,
-						 const SwfdecRect *	inval,
-						 gboolean		fill);
+						 const SwfdecRect *	inval);
 
   /* mouse handling */
   gboolean		(* mouse_in)		(SwfdecMovie *		movie,
@@ -163,6 +184,7 @@ GType		swfdec_movie_get_type		(void);
 SwfdecMovie *	swfdec_movie_new		(SwfdecPlayer *		player,
 						 int			depth,
 						 SwfdecMovie *		parent,
+						 SwfdecResource *	resource,
 						 SwfdecGraphic *	graphic,
 						 const char *		name);
 SwfdecMovie *	swfdec_movie_new_for_content  	(SwfdecMovie *		parent,
@@ -173,6 +195,10 @@ SwfdecMovie *	swfdec_movie_duplicate		(SwfdecMovie *		movie,
 void		swfdec_movie_initialize		(SwfdecMovie *		movie);
 SwfdecMovie *	swfdec_movie_find		(SwfdecMovie *		movie,
 						 int			depth);
+SwfdecMovie *	swfdec_movie_get_by_name	(SwfdecMovie *		movie,
+						 const char *		name,
+						 gboolean		unnamed);
+SwfdecMovie *	swfdec_movie_get_root		(SwfdecMovie *		movie);
 void		swfdec_movie_remove		(SwfdecMovie *		movie);
 void		swfdec_movie_destroy		(SwfdecMovie *		movie);
 void		swfdec_movie_set_static_properties 
@@ -181,6 +207,7 @@ void		swfdec_movie_set_static_properties
 						 const SwfdecColorTransform *ctrans,
 						 int			ratio,
 						 int			clip_depth,
+						 guint			blend_mode,
 						 SwfdecEventList *	events);
 void		swfdec_movie_invalidate		(SwfdecMovie *		movie);
 void		swfdec_movie_queue_update	(SwfdecMovie *		movie,
@@ -192,40 +219,45 @@ void		swfdec_movie_local_to_global	(SwfdecMovie *		movie,
 void		swfdec_movie_global_to_local	(SwfdecMovie *		movie,
 						 double *		x,
 						 double *		y);
+void		swfdec_movie_rect_local_to_global (SwfdecMovie *	movie,
+						 SwfdecRect *		rect);
+void		swfdec_movie_rect_global_to_local (SwfdecMovie *	movie,
+						 SwfdecRect *		rect);
+void		swfdec_movie_set_depth		(SwfdecMovie *		movie,
+						 int			depth);
 void		swfdec_movie_get_mouse		(SwfdecMovie *		movie,
 						 double *		x,
 						 double *		y);
 void		swfdec_movie_send_mouse_change	(SwfdecMovie *		movie,
 						 gboolean		release);
+gboolean	swfdec_movie_mouse_in		(SwfdecMovie *		movie,
+						 double			x,
+						 double			y);
 SwfdecMovie *	swfdec_movie_get_movie_at	(SwfdecMovie *		movie,
 						 double			x,
 						 double			y);
-char *		swfdec_movie_get_path		(SwfdecMovie *		movie);
+char *		swfdec_movie_get_path		(SwfdecMovie *		movie,
+						 gboolean		dot);
 void		swfdec_movie_render		(SwfdecMovie *		movie,
 						 cairo_t *		cr, 
 						 const SwfdecColorTransform *trans,
-						 const SwfdecRect *	inval,
-						 gboolean		fill);
-void		swfdec_movie_execute_script	(SwfdecMovie *		movie,
+						 const SwfdecRect *	inval);
+SwfdecMovie *	swfdec_movie_resolve		(SwfdecMovie *		movie);
+guint		swfdec_movie_get_version	(SwfdecMovie *		movie);
+void		swfdec_movie_execute		(SwfdecMovie *		movie,
 						 SwfdecEventType	condition);
 gboolean      	swfdec_movie_queue_script	(SwfdecMovie *		movie,
   						 SwfdecEventType	condition);
-void		swfdec_movie_set_variables	(SwfdecMovie *		movie,
-						 const char *		variables);
-void		swfdec_movie_load		(SwfdecMovie *		movie,
+void		swfdec_movie_load_variables	(SwfdecMovie *		movie,
 						 const char *		url,
-						 const char *		target,
 						 SwfdecLoaderRequest	request,
-						 const char *		data,
-						 gsize			data_len);
+						 SwfdecBuffer *		data);
 
 int		swfdec_movie_compare_depths	(gconstpointer		a,
 						 gconstpointer		b);
 SwfdecDepthClass
 		swfdec_depth_classify		(int			depth);
 
-void		swfdec_movie_run_init		(SwfdecMovie *		movie);
-void		swfdec_movie_run_construct	(SwfdecMovie *		movie);
 /* in swfdec_movie_asprops.c */
 gboolean	swfdec_movie_set_asprop		(SwfdecMovie *		movie,
 						 const char *		name,
@@ -234,5 +266,15 @@ gboolean	swfdec_movie_get_asprop		(SwfdecMovie *		movie,
 						 const char *		name,
 						 SwfdecAsValue *	val);
 
+void		swfdec_movie_add_variable_listener (SwfdecMovie *	movie,
+						 SwfdecAsObject *	object,
+						 const char *		name,
+						 const SwfdecMovieVariableListenerFunction	function);
+void		swfdec_movie_remove_variable_listener (SwfdecMovie *	movie,
+						 SwfdecAsObject *	object,
+						 const char *		name,
+						 const SwfdecMovieVariableListenerFunction	function);
+SwfdecResource *swfdec_movie_get_own_resource	(SwfdecMovie *		movie);
+
 G_END_DECLS
 #endif
