@@ -24,11 +24,13 @@
 #include <math.h>
 #include "swfdec_net_stream.h"
 #include "swfdec_amf.h"
+#include "swfdec_as_frame_internal.h"
 #include "swfdec_as_strings.h"
 #include "swfdec_audio_flv.h"
 #include "swfdec_debug.h"
 #include "swfdec_loader_internal.h"
 #include "swfdec_loadertarget.h"
+#include "swfdec_resource_request.h"
 
 /* NB: code and level must be rooted gc-strings */
 static void
@@ -50,12 +52,44 @@ swfdec_net_stream_onstatus (SwfdecNetStream *stream, const char *code, const cha
   swfdec_as_object_call (SWFDEC_AS_OBJECT (stream), SWFDEC_AS_STR_onStatus, 1, &val, NULL);
 }
 
+static cairo_surface_t *
+swfdec_net_stream_decode_video (SwfdecNetStream *stream, SwfdecBuffer *buffer)
+{
+  SwfdecVideoDecoder *decoder = stream->decoder;
+  cairo_surface_t *surface;
+
+  if (decoder == NULL)
+    return NULL;
+
+  if (decoder->codec == SWFDEC_VIDEO_CODEC_VP6 ||
+      decoder->codec == SWFDEC_VIDEO_CODEC_VP6_ALPHA) {
+    guint wsub, hsub;
+    SwfdecBuffer *tmp;
+    if (buffer->length == 0) {
+      SWFDEC_ERROR ("0-byte VP6 video image buffer?");
+      return NULL;
+    }
+    wsub = *buffer->data;
+    hsub = wsub & 0xF;
+    wsub >>= 4;
+    tmp = swfdec_buffer_new_subbuffer (buffer, 1, buffer->length - 1);
+    surface = swfdec_video_decoder_decode (decoder, tmp);
+    swfdec_buffer_unref (tmp);
+    if (hsub || wsub) {
+      SWFDEC_FIXME ("need to subtract %ux%u pixels", wsub, hsub);
+    }
+  } else {
+    surface = swfdec_video_decoder_decode (decoder, buffer);
+  }
+  return surface;
+}
+
 static void swfdec_net_stream_update_playing (SwfdecNetStream *stream);
 static void
 swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
 {
   SwfdecBuffer *buffer;
-  SwfdecVideoFormat format;
+  guint format;
   cairo_surface_t *old;
   gboolean process_events;
   guint process_events_from;
@@ -83,13 +117,11 @@ swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
       stream->format = format;
       stream->decoder = swfdec_video_decoder_new (format);
     }
-    if (stream->decoder) {
-      stream->surface = swfdec_video_decoder_decode (stream->decoder, buffer);
-    }
+    stream->surface = swfdec_net_stream_decode_video (stream, buffer);
     if (stream->surface) {
       GList *walk;
       for (walk = stream->movies; walk; walk = walk->next) {
-	swfdec_video_movie_new_image (walk->data, stream->surface);
+	swfdec_video_movie_new_image (walk->data);
       }
     }
   }
@@ -228,8 +260,7 @@ swfdec_net_stream_loader_target_parse (SwfdecLoaderTarget *target,
     SwfdecLoader *loader)
 {
   SwfdecNetStream *stream = SWFDEC_NET_STREAM (target);
-  SwfdecDecoderClass *klass;
-  gboolean recheck = FALSE;
+  SwfdecStatus status;
   
   if (loader->state != SWFDEC_LOADER_STATE_EOF && swfdec_buffer_queue_get_depth (loader->queue) == 0) {
     SWFDEC_INFO ("nothing to do");
@@ -239,39 +270,21 @@ swfdec_net_stream_loader_target_parse (SwfdecLoaderTarget *target,
     /* FIXME: add mp3 support */
     stream->flvdecoder = g_object_new (SWFDEC_TYPE_FLV_DECODER, NULL);
     SWFDEC_DECODER (stream->flvdecoder)->player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (stream)->context);
-    SWFDEC_DECODER (stream->flvdecoder)->queue = loader->queue;
     swfdec_net_stream_onstatus (stream, SWFDEC_AS_STR_NetStream_Play_Start,
 	SWFDEC_AS_STR_status);
     swfdec_loader_set_data_type (loader, SWFDEC_LOADER_DATA_FLV);
   }
-  klass = SWFDEC_DECODER_GET_CLASS (stream->flvdecoder);
-  g_return_if_fail (klass->parse);
 
-  while (TRUE) {
-    SwfdecStatus status = klass->parse (SWFDEC_DECODER (stream->flvdecoder));
-    switch (status) {
-      case SWFDEC_STATUS_OK:
-	break;
-      case SWFDEC_STATUS_INIT:
-	/* HACK for native flv playback */
-	swfdec_player_initialize (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (stream)->context), 7,
-	    SWFDEC_DECODER (stream->flvdecoder)->rate, 
-	    SWFDEC_DECODER (stream->flvdecoder)->width, 
-	    SWFDEC_DECODER (stream->flvdecoder)->height);
-      case SWFDEC_STATUS_IMAGE:
-	recheck = TRUE;
-	break;
-      case SWFDEC_STATUS_ERROR:
-      case SWFDEC_STATUS_NEEDBITS:
-      case SWFDEC_STATUS_EOF:
-	goto out;
-      default:
-	g_assert_not_reached ();
-	return;
-    }
-  }
-out:
-  if (recheck)
+  status = SWFDEC_STATUS_OK;
+  do {
+    SwfdecBuffer *buffer = swfdec_buffer_queue_pull_buffer (loader->queue);
+    if (buffer == NULL)
+      break;
+    status &= ~SWFDEC_STATUS_NEEDBITS;
+    status |= swfdec_decoder_parse (SWFDEC_DECODER (stream->flvdecoder), buffer);
+  } while ((status & (SWFDEC_STATUS_ERROR | SWFDEC_STATUS_EOF)) == 0);
+
+  if (status & SWFDEC_STATUS_IMAGE)
     swfdec_net_stream_loader_target_recheck (stream);
 }
 
@@ -282,7 +295,7 @@ swfdec_net_stream_loader_target_eof (SwfdecLoaderTarget *target,
   SwfdecNetStream *stream = SWFDEC_NET_STREAM (target);
   guint first, last;
 
-  swfdec_flv_decoder_eof (stream->flvdecoder);
+  swfdec_decoder_eof (SWFDEC_DECODER (stream->flvdecoder));
   swfdec_net_stream_onstatus (stream, SWFDEC_AS_STR_NetStream_Buffer_Flush,
       SWFDEC_AS_STR_status);
   swfdec_net_stream_video_goto (stream, stream->current_time);
@@ -324,6 +337,14 @@ swfdec_net_stream_input_disconnect (SwfdecVideoMovieInput *input, SwfdecVideoMov
   g_object_unref (stream);
 }
 
+static cairo_surface_t *
+swfdec_net_stream_input_get_image (SwfdecVideoMovieInput *input)
+{
+  SwfdecNetStream *stream = SWFDEC_NET_STREAM ((guchar *) input - G_STRUCT_OFFSET (SwfdecNetStream, input));
+
+  return stream->surface;
+}
+
 /*** SWFDEC_NET_STREAM ***/
 
 G_DEFINE_TYPE_WITH_CODE (SwfdecNetStream, swfdec_net_stream, SWFDEC_TYPE_AS_OBJECT,
@@ -350,12 +371,12 @@ swfdec_net_stream_dispose (GObject *object)
 }
 
 static gboolean
-swfdec_net_stream_get_variable (SwfdecAsObject *object, const char *variable,
-    SwfdecAsValue *val, guint *flags)
+swfdec_net_stream_get_variable (SwfdecAsObject *object, SwfdecAsObject *orig,
+    const char *variable, SwfdecAsValue *val, guint *flags)
 {
   SwfdecNetStream *stream;
 
-  if (SWFDEC_AS_OBJECT_CLASS (swfdec_net_stream_parent_class)->get (object, variable, val, flags))
+  if (SWFDEC_AS_OBJECT_CLASS (swfdec_net_stream_parent_class)->get (object, orig, variable, val, flags))
     return TRUE;
 
   stream = SWFDEC_NET_STREAM (object);
@@ -370,27 +391,27 @@ swfdec_net_stream_get_variable (SwfdecAsObject *object, const char *variable,
 	msecs = 0;
       else 
 	msecs = stream->current_time - msecs;
+      SWFDEC_AS_VALUE_SET_NUMBER (val, msecs / 1000.);
     }
-    SWFDEC_AS_VALUE_SET_NUMBER (val, msecs / 1000.);
     *flags = 0;
     return TRUE;
   } else if (variable == SWFDEC_AS_STR_bytesLoaded) {
     if (stream->loader == NULL)
       SWFDEC_AS_VALUE_SET_INT (val, 0);
     else
-      SWFDEC_AS_VALUE_SET_INT (val, swfdec_loader_get_loaded (stream->loader));
+      SWFDEC_AS_VALUE_SET_NUMBER (val, swfdec_loader_get_loaded (stream->loader));
     *flags = 0;
     return TRUE;
   } else if (variable == SWFDEC_AS_STR_bytesTotal) {
-    guint bytes;
+    glong bytes;
     if (stream->loader == NULL) {
       bytes = 0;
     } else { 
       bytes = swfdec_loader_get_size (stream->loader);
-      if (bytes == 0)
+      if (bytes < 0)
 	bytes = swfdec_loader_get_loaded (stream->loader);
     }
-    SWFDEC_AS_VALUE_SET_INT (val, bytes);
+    SWFDEC_AS_VALUE_SET_NUMBER (val, bytes);
     *flags = 0;
     return TRUE;
   }
@@ -402,7 +423,8 @@ swfdec_net_stream_mark (SwfdecAsObject *object)
 {
   SwfdecNetStream *stream = SWFDEC_NET_STREAM (object);
 
-  swfdec_as_object_mark (SWFDEC_AS_OBJECT (stream->conn));
+  if (stream->conn)
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (stream->conn));
 
   SWFDEC_AS_OBJECT_CLASS (swfdec_net_stream_parent_class)->mark (object);
 }
@@ -424,6 +446,7 @@ swfdec_net_stream_init (SwfdecNetStream *stream)
 {
   stream->input.connect = swfdec_net_stream_input_connect;
   stream->input.disconnect = swfdec_net_stream_input_disconnect;
+  stream->input.get_image = swfdec_net_stream_input_get_image;
 
   stream->buffer_time = 100; /* msecs */
 }
@@ -449,15 +472,21 @@ swfdec_net_stream_new (SwfdecNetConnection *conn)
 void
 swfdec_net_stream_set_url (SwfdecNetStream *stream, const char *url)
 {
+  SwfdecAsContext *cx;
   SwfdecLoader *loader;
 
   g_return_if_fail (SWFDEC_IS_NET_STREAM (stream));
   g_return_if_fail (url != NULL);
 
   /* FIXME: use the connection once connections are implemented */
-  loader = swfdec_player_load (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (stream)->context), url);
-  swfdec_net_stream_set_loader (stream, loader);
-  g_object_unref (loader);
+  cx = SWFDEC_AS_OBJECT (stream)->context;
+  g_assert (cx->frame);
+  loader = swfdec_player_request_resource_now (SWFDEC_PLAYER (cx), cx->frame->security, 
+      url, SWFDEC_LOADER_REQUEST_DEFAULT, NULL);
+  if (loader) {
+    swfdec_net_stream_set_loader (stream, loader);
+    g_object_unref (loader);
+  }
 }
 
 void
@@ -468,6 +497,7 @@ swfdec_net_stream_set_loader (SwfdecNetStream *stream, SwfdecLoader *loader)
 
   if (stream->loader) {
     swfdec_loader_set_target (stream->loader, NULL);
+    swfdec_loader_close (stream->loader);
     g_object_unref (stream->loader);
   }
   if (stream->flvdecoder) {
