@@ -22,6 +22,7 @@
 
 #include <glib-object.h>
 #include <cairo.h>
+#include <libswfdec/swfdec_as_types.h>
 #include <libswfdec/swfdec_loader.h>
 
 G_BEGIN_DECLS
@@ -66,7 +67,7 @@ void		swfdec_init			(void);
 
 GType		swfdec_player_get_type		(void);
 
-SwfdecPlayer *	swfdec_player_new		(void);
+SwfdecPlayer *	swfdec_player_new		(SwfdecAsDebugger *	debugger);
 SwfdecPlayer *	swfdec_player_new_from_file	(const char *	filename);
 void		swfdec_player_set_loader	(SwfdecPlayer *	player,
 						 SwfdecLoader *	loader);
@@ -76,11 +77,11 @@ void		swfdec_player_set_loader_with_variables
 						 const char *	variables);
 
 gboolean	swfdec_player_is_initialized	(SwfdecPlayer *	player);
-guint		swfdec_player_get_next_event  	(SwfdecPlayer *	player);
+glong		swfdec_player_get_next_event  	(SwfdecPlayer *	player);
 double		swfdec_player_get_rate		(SwfdecPlayer *	player);
-void		swfdec_player_get_image_size	(SwfdecPlayer *	player,
-						 int *		width,
-						 int *		height);
+void		swfdec_player_get_default_size	(SwfdecPlayer *	player,
+						 guint *	width,
+						 guint *	height);
 void		swfdec_player_get_size		(SwfdecPlayer *	player,
 						 int *		width,
 						 int *		height);
@@ -98,6 +99,11 @@ void		swfdec_player_set_scale_mode	(SwfdecPlayer *		player,
 SwfdecAlignment	swfdec_player_get_alignment	(SwfdecPlayer *		player);
 void		swfdec_player_set_alignment	(SwfdecPlayer *		player,
 						 SwfdecAlignment	align);
+gulong		swfdec_player_get_maximum_runtime
+						(SwfdecPlayer *	player);
+void		swfdec_player_set_maximum_runtime 
+						(SwfdecPlayer *	player,
+						 gulong		msecs);
 					 
 void		swfdec_player_render		(SwfdecPlayer *	player,
 						 cairo_t *	cr,
@@ -106,11 +112,17 @@ void		swfdec_player_render		(SwfdecPlayer *	player,
 						 double		width,
 						 double		height);
 void		swfdec_player_advance		(SwfdecPlayer *	player,
-						 guint		msecs);
+						 gulong		msecs);
 gboolean	swfdec_player_handle_mouse	(SwfdecPlayer *	player, 
 						 double		x,
 						 double		y,
 						 int		button);
+gboolean	swfdec_player_key_press		(SwfdecPlayer *	player,
+						 guint		keycode,
+						 guint		character);
+gboolean	swfdec_player_key_release	(SwfdecPlayer *	player,
+						 guint		keycode,
+						 guint		character);
 /* audio - see swfdec_audio.c */
 void		swfdec_player_render_audio	(SwfdecPlayer *	player,
 						 gint16 *	dest, 
