@@ -22,14 +22,15 @@
 
 #include <libswfdec/swfdec_player.h>
 #include <libswfdec/swfdec_as_context.h>
+#include <libswfdec/swfdec_audio.h>
+#include <libswfdec/swfdec_event.h>
 #include <libswfdec/swfdec_rect.h>
 #include <libswfdec/swfdec_ringbuffer.h>
+#include <libswfdec/swfdec_security.h>
+#include <libswfdec/swfdec_system.h>
 
 G_BEGIN_DECLS
 
-#define SWFDEC_AS_NATIVE(x, y, func) void func (SwfdecAsContext *cx, \
-    SwfdecAsObject *object, guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret);
-
 typedef enum {
   SWFDEC_ALIGN_FLAG_TOP		= (1 << 0),
   SWFDEC_ALIGN_FLAG_BOTTOM	= (1 << 1),
@@ -38,6 +39,7 @@ typedef enum {
 } SwfdecAlignFlag;
 
 typedef void (* SwfdecActionFunc) (gpointer object, gpointer data);
+typedef gboolean (* SwfdecAudioRemoveFunc) (SwfdecAudio *audio, gpointer data);
 
 typedef struct _SwfdecTimeout SwfdecTimeout;
 struct _SwfdecTimeout {
@@ -46,25 +48,31 @@ struct _SwfdecTimeout {
   void			(* free)		(SwfdecTimeout *advance);
 };
 
+#define SWFDEC_PLAYER_N_ACTION_QUEUES 4
+
 struct _SwfdecPlayer
 {
   SwfdecAsContext	context;
 
   /* global properties */
+  SwfdecSystem *	system;			/* our system properties */
   gboolean		initialized;		/* if width and height are set already */
   guint		  	rate;			/* divide by 256 to get iterations per second */
   guint			width;			/* width of movie */
   guint			height;			/* height of movie */
   GList *		roots;			/* all the root movies */
+  GList *		rooted_objects;		/* all the objects we keep track of */
+  GSList *		resource_requests;	/* all external requested URIs - see swfdec_resource_request.[ch] */
   SwfdecCache *		cache;			/* player cache */
   gboolean		bgcolor_set;		/* TRUE if the background color has been set */
   SwfdecColor		bgcolor;		/* background color */
-  SwfdecLoader *	loader;			/* initial loader */
+  SwfdecResource *	resource;		/* initial resource loaded */
   /* stage properties */
   guint			internal_width;		/* width used by the scripting engine */
   guint			internal_height;	/* height used by the scripting engine */
   gint			stage_width;		/* width set by the user */
   gint			stage_height;		/* height set by the user */
+  SwfdecRectangle     	stage;			/* size of the stage set by user */
   guint			align_flags;		/* SwfdecAlignFlag */
   SwfdecScaleMode	scale_mode;		/* scale mode */
   double		scale_x;		/* cached x scale value */
@@ -81,13 +89,14 @@ struct _SwfdecPlayer
   SwfdecAsObject *	Video;			/* Video object */
 
   /* rendering */
-  SwfdecRect		invalid;      		/* area that needs a rredraw */
+  SwfdecRectangle     	invalid_extents;      	/* extents of area that needs a redraw in global coordinates */
+  GArray *		invalidations;		/* fine-grained areas in need of redraw */
 
   /* mouse */
   gboolean		mouse_visible;	  	/* show the mouse (actionscriptable) */
   SwfdecMouseCursor	mouse_cursor;		/* cursor that should be shown */
-  double      		mouse_x;		/* in twips */
-  double		mouse_y;		/* in twips */
+  double      		mouse_x;		/* in stage coordinates */
+  double		mouse_y;		/* in stage coordinates */
   int			mouse_button; 		/* 0 for not pressed, 1 for pressed */
   SwfdecMovie *		mouse_grab;		/* movie that currently has the mouse */
   SwfdecMovie *		mouse_drag;		/* current movie activated by startDrag */
@@ -96,22 +105,26 @@ struct _SwfdecPlayer
   double		mouse_drag_x;		/* offset of mouse in x direction */
   double		mouse_drag_y;		/* offset of mouse in y direction */
 
+  /* key */
+  guint			last_keycode;		/* last keycode that was pressed/released */
+  guint			last_character;		/* UCS4 of last character pressed/released */
+  guint8		key_pressed[256/8];   	/* boolean array for isDown */
+
   /* audio */
   GList *		audio;		 	/* list of playing SwfdecAudio */
-  guint			audio_skip;		/* number of frames to auto-skip when adding new audio */
 
   /* events and advancing */
   SwfdecTick		time;			/* current time */
   GList *		timeouts;	      	/* list of events, sorted by timestamp */
   guint			tick;			/* next tick */
   SwfdecTimeout		iterate_timeout;      	/* callback for iterating */
-  /* iterating */
-  GList *		movies;			/* list of all moveis that want to be iterated */
-  SwfdecRingBuffer *	actions;		/* all actions we've queued up so far */
+  GTimer *		runtime;		/* for checking how long we've been running */
+  gulong		max_runtime;		/* maximum number of seconds the player may run */
   SwfdecRingBuffer *	external_actions;     	/* external actions we've queued up, like resize or loader stuff */
   SwfdecTimeout		external_timeout;      	/* callback for iterating */
-  GQueue *		init_queue;		/* all movies that require an init event */
-  GQueue *		construct_queue;      	/* all movies that require an construct event */
+  /* iterating */
+  GList *		movies;			/* list of all movies that want to be iterated */
+  SwfdecRingBuffer *	actions[SWFDEC_PLAYER_N_ACTION_QUEUES]; /* all actions we've queued up so far */
 };
 
 struct _SwfdecPlayerClass
@@ -119,8 +132,12 @@ struct _SwfdecPlayerClass
   SwfdecAsContextClass	context_class;
 
   void			(* advance)		(SwfdecPlayer *		player,
-						 guint			msecs,
+						 gulong			msecs,
 						 guint			audio_samples);
+  gboolean		(* handle_key)		(SwfdecPlayer *		player,
+						 guint			key,
+						 guint			character,
+						 gboolean		down);
   gboolean		(* handle_mouse)	(SwfdecPlayer *		player,
 						 double			x,
 						 double			y,
@@ -138,10 +155,16 @@ void		swfdec_player_add_movie		(SwfdecPlayer *		player,
 void		swfdec_player_remove_movie	(SwfdecPlayer *		player,
 						 SwfdecMovie *		movie);
 
-void		swfdec_player_lock		(SwfdecPlayer *		player);
+gboolean	swfdec_player_lock		(SwfdecPlayer *		player);
+void		swfdec_player_lock_soft		(SwfdecPlayer *		player);
 void		swfdec_player_unlock		(SwfdecPlayer *		player);
+void		swfdec_player_unlock_soft	(SwfdecPlayer *		player);
 void		swfdec_player_perform_actions	(SwfdecPlayer *		player);
 
+void		swfdec_player_root_object	(SwfdecPlayer *		player,
+						 GObject *		object);
+void		swfdec_player_unroot_object	(SwfdecPlayer *		player,
+						 GObject *		object);
 SwfdecAsObject *swfdec_player_get_export_class	(SwfdecPlayer *		player,
 						 const char *		name);
 void		swfdec_player_set_export_class	(SwfdecPlayer *		player,
@@ -163,11 +186,15 @@ void		swfdec_player_remove_all_external_actions
 						(SwfdecPlayer *      	player,
 						 gpointer		object);
 void		swfdec_player_add_action	(SwfdecPlayer *		player,
-						 gpointer		object,
-						 SwfdecActionFunc   	action_func,
-						 gpointer		action_data);
+						 SwfdecMovie *		movie,
+						 SwfdecEventType	type,
+						 guint			importance);
+void		swfdec_player_add_action_script	(SwfdecPlayer *		player,
+						 SwfdecMovie *		movie,
+						 SwfdecScript *		script,
+						 guint			importance);
 void		swfdec_player_remove_all_actions (SwfdecPlayer *      	player,
-						 gpointer		object);
+						 SwfdecMovie *		movie);
 
 void		swfdec_player_set_drag_movie	(SwfdecPlayer *		player,
 						 SwfdecMovie *		drag,
@@ -175,19 +202,38 @@ void		swfdec_player_set_drag_movie	(SwfdecPlayer *		player,
 						 SwfdecRect *		rect);
 void		swfdec_player_set_align_flags	(SwfdecPlayer *		player,
 						 guint			flags);
+void		swfdec_player_stop_sounds	(SwfdecPlayer *		player,
+						 SwfdecAudioRemoveFunc	func,
+						 gpointer		data);
 void		swfdec_player_stop_all_sounds	(SwfdecPlayer *		player);
-SwfdecMovie *	swfdec_player_add_level_from_loader 
+gboolean	swfdec_player_get_level		(SwfdecPlayer *		player,
+						 const char *		name);
+SwfdecSpriteMovie *
+		swfdec_player_get_movie_at_level(SwfdecPlayer *		player,
+						 int			level);
+SwfdecSpriteMovie *
+		swfdec_player_create_movie_at_level 
 						(SwfdecPlayer *		player,
-						 guint			depth,
-						 SwfdecLoader *		loader,
-						 const char *		variables);
-void		swfdec_player_remove_level	(SwfdecPlayer *		player,
-						 guint			depth);
-SwfdecLoader *	swfdec_player_load		(SwfdecPlayer *         player,
-						 const char *		url);
+						 SwfdecResource *	resource,
+						 int			level);
 void		swfdec_player_launch		(SwfdecPlayer *         player,
+						 SwfdecLoaderRequest	request,
 						 const char *		url,
-						 const char *		target);
+						 const char *		target,
+						 SwfdecBuffer *		data);
+void		swfdec_player_stage_to_global	(SwfdecPlayer *		player,
+						 double *		x,
+						 double *		y);
+void		swfdec_player_global_to_stage	(SwfdecPlayer *		player,
+						 double *		x,
+						 double *		y);
+/* in swfdec_as_interpret.c */
+SwfdecMovie *	swfdec_player_get_movie_from_value 
+						(SwfdecPlayer *		player,
+						 SwfdecAsValue *	val);
+SwfdecMovie *	swfdec_player_get_movie_from_string
+						(SwfdecPlayer *		player,
+						 const char *		s);
 
 
 G_END_DECLS
