@@ -89,7 +89,7 @@ tag_func_sound_stream_block (SwfdecSwfDecoder * s, guint tag)
   }
 
   n_samples = swfdec_bits_get_u16 (&s->b);
-  if (sound->format == SWFDEC_AUDIO_FORMAT_MP3) {
+  if (sound->codec == SWFDEC_AUDIO_CODEC_MP3) {
     skip = swfdec_bits_get_s16 (&s->b);
   } else {
     skip = 0;
@@ -121,69 +121,57 @@ tag_func_define_sound (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecBits *b = &s->b;
   int id;
-  int format;
-  int rate;
-  int size;
-  int type;
   int n_samples;
   SwfdecSound *sound;
 
   id = swfdec_bits_get_u16 (b);
-  format = swfdec_bits_getbits (b, 4);
-  rate = swfdec_bits_getbits (b, 2);
-  size = swfdec_bits_getbits (b, 1);
-  type = swfdec_bits_getbits (b, 1);
-  n_samples = swfdec_bits_get_u32 (b);
-
   sound = swfdec_swf_decoder_create_character (s, id, SWFDEC_TYPE_SOUND);
   if (!sound)
     return SWFDEC_STATUS_OK;
 
-  sound->width = size;
-  rate = 1 << (3 - rate);
-  sound->original_format = SWFDEC_AUDIO_OUT_GET (type ? 2 : 1, 44100 / rate);
+  sound->codec = swfdec_bits_getbits (b, 4);
+  SWFDEC_LOG ("  codec: %u", sound->codec);
+  sound->format = swfdec_audio_format_parse (b);
+  SWFDEC_LOG ("  format: %s", swfdec_audio_format_to_string (sound->format));
+  n_samples = swfdec_bits_get_u32 (b);
   sound->n_samples = n_samples;
-  SWFDEC_DEBUG ("%u samples, %sLE, %uch, %ukHz", n_samples,
-      size ? "S16" : "U8", SWFDEC_AUDIO_OUT_N_CHANNELS (sound->original_format),
-      SWFDEC_AUDIO_OUT_RATE (sound->original_format));
 
-  switch (format) {
+  switch (sound->codec) {
     case 0:
-      if (size == 1)
+      if (swfdec_audio_format_is_16bit (sound->format))
 	SWFDEC_WARNING ("undefined endianness for s16 sound");
       /* just assume LE and hope it works (FIXME: want a switch for this?) */
+      sound->codec = SWFDEC_AUDIO_CODEC_UNCOMPRESSED;
       /* fall through */
     case 3:
-      sound->format = SWFDEC_AUDIO_FORMAT_UNCOMPRESSED;
       sound->encoded = swfdec_bits_get_buffer (&s->b, -1);
       break;
     case 2:
-      sound->format = SWFDEC_AUDIO_FORMAT_MP3;
       sound->skip = swfdec_bits_get_u16 (b);
       sound->encoded = swfdec_bits_get_buffer (&s->b, -1);
       break;
     case 1:
     case 5:
     case 6:
-      sound->format = format;
       sound->encoded = swfdec_bits_get_buffer (&s->b, -1);
       break;
     default:
-      SWFDEC_WARNING ("unknown format %d", format);
-      sound->format = format;
+      SWFDEC_WARNING ("unknown codec %d", sound->codec);
   }
-  sound->n_samples *= rate;
+  sound->n_samples *= swfdec_audio_format_get_granularity (sound->format);
 
   return SWFDEC_STATUS_OK;
 }
 
-static SwfdecBuffer *
-swfdec_sound_get_decoded (SwfdecSound *sound, SwfdecAudioOut *format)
+SwfdecBuffer *
+swfdec_sound_get_decoded (SwfdecSound *sound, SwfdecAudioFormat *format)
 {
   gpointer decoder;
   SwfdecBuffer *tmp;
   SwfdecBufferQueue *queue;
   guint sample_bytes;
+  guint n_samples;
+  guint depth;
 
   g_return_val_if_fail (SWFDEC_IS_SOUND (sound), NULL);
   g_return_val_if_fail (format != NULL, NULL);
@@ -196,48 +184,50 @@ swfdec_sound_get_decoded (SwfdecSound *sound, SwfdecAudioOut *format)
   if (sound->encoded == NULL)
     return NULL;
 
-  decoder = swfdec_audio_decoder_new (sound->format, sound->width, sound->original_format);
+  decoder = swfdec_audio_decoder_new (sound->codec, sound->format);
   if (decoder == NULL)
     return NULL;
-  sound->decoded_format = swfdec_audio_decoder_get_format (decoder);
-  sample_bytes = 2 * SWFDEC_AUDIO_OUT_N_CHANNELS (sound->decoded_format);
-  /* FIXME: The size is only a guess */
-  swfdec_cached_load (SWFDEC_CACHED (sound), sound->n_samples * sample_bytes);
 
   swfdec_audio_decoder_push (decoder, sound->encoded);
   swfdec_audio_decoder_push (decoder, NULL);
   queue = swfdec_buffer_queue_new ();
-  while ((tmp = swfdec_audio_decoder_pull (decoder)))
+  while ((tmp = swfdec_audio_decoder_pull (decoder))) {
     swfdec_buffer_queue_push (queue, tmp);
+  }
+  sound->decoded_format = swfdec_audio_decoder_get_format (decoder);
   swfdec_audio_decoder_free (decoder);
-  tmp = swfdec_buffer_queue_pull (queue, swfdec_buffer_queue_get_depth (queue));
+  depth = swfdec_buffer_queue_get_depth (queue);
+  if (depth == 0) {
+    SWFDEC_ERROR ("decoding didn't produce any data, bailing");
+    return NULL;
+  }
+  swfdec_cached_load (SWFDEC_CACHED (sound), depth);
+  tmp = swfdec_buffer_queue_pull (queue, depth);
   swfdec_buffer_queue_unref (queue);
 
+  sample_bytes = swfdec_audio_format_get_bytes_per_sample (sound->decoded_format);
+  n_samples = sound->n_samples / swfdec_audio_format_get_granularity (sound->decoded_format);
+
   SWFDEC_LOG ("after decoding, got %u samples, should get %u and skip %u", 
-      tmp->length / sample_bytes, sound->n_samples, sound->skip);
+      tmp->length / sample_bytes, n_samples, sound->skip);
   if (sound->skip) {
     SwfdecBuffer *tmp2 = swfdec_buffer_new_subbuffer (tmp, sound->skip * sample_bytes, 
 	tmp->length - sound->skip * sample_bytes);
     swfdec_buffer_unref (tmp);
     tmp = tmp2;
   }
-  /* sound buffer may be bigger due to mp3 not having sample boundaries */
-  if (tmp->length * SWFDEC_AUDIO_OUT_GRANULARITY (sound->decoded_format) 
-      > sound->n_samples * sample_bytes) {
-    SwfdecBuffer *tmp2 = swfdec_buffer_new_subbuffer (tmp, 0, 
-	sound->n_samples * sample_bytes / SWFDEC_AUDIO_OUT_GRANULARITY (sound->decoded_format));
+  if (tmp->length > n_samples * sample_bytes) {
+    SwfdecBuffer *tmp2 = swfdec_buffer_new_subbuffer (tmp, 0, n_samples * sample_bytes);
+    SWFDEC_DEBUG ("%u samples in %u bytes should be available, but %u bytes are, cutting them off",
+	n_samples, n_samples * sample_bytes, tmp->length);
     swfdec_buffer_unref (tmp);
     tmp = tmp2;
-  }
-  if (tmp->length * SWFDEC_AUDIO_OUT_GRANULARITY (sound->decoded_format) 
-      < sound->n_samples * sample_bytes) {
+  } else if (tmp->length < n_samples * sample_bytes) {
     /* we handle this case in swfdec_sound_render */
     /* FIXME: this message is important when writing new codecs, so I made it a warning.
      * It's probably not worth more than INFO for the usual case though */
     SWFDEC_WARNING ("%u samples in %u bytes should be available, but only %u bytes are",
-	sound->n_samples / SWFDEC_AUDIO_OUT_GRANULARITY (sound->decoded_format), 
-	sound->n_samples * sample_bytes / SWFDEC_AUDIO_OUT_GRANULARITY (sound->decoded_format), 
-	tmp->length);
+	n_samples, n_samples * sample_bytes, tmp->length);
   }
   /* only assign here, the decoding code checks this variable */
   sound->decoded = tmp;
@@ -250,55 +240,50 @@ int
 tag_func_sound_stream_head (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecBits *b = &s->b;
-  int format;
-  int rate;
-  int size;
-  int type;
+  SwfdecAudioFormat playback;
+  guint playback_codec;
   int n_samples;
   int latency;
   SwfdecSound *sound;
 
+  playback_codec = swfdec_bits_getbits (b, 4);
   /* we don't care about playback suggestions */
-  swfdec_bits_getbits (b, 8);
-
-  format = swfdec_bits_getbits (b, 4);
-  rate = swfdec_bits_getbits (b, 2);
-  size = swfdec_bits_getbits (b, 1);
-  type = swfdec_bits_getbits (b, 1);
-  n_samples = swfdec_bits_get_u16 (b);
+  playback = swfdec_audio_format_parse (b);
+  SWFDEC_LOG ("  suggested playback format: %s", swfdec_audio_format_to_string (playback));
 
   sound = g_object_new (SWFDEC_TYPE_SOUND, NULL);
+  sound->codec = swfdec_bits_getbits (b, 4);
+  sound->format = swfdec_audio_format_parse (b);
+  n_samples = swfdec_bits_get_u16 (b);
+  if (playback_codec != 0 && playback_codec != sound->codec) {
+    SWFDEC_FIXME ("playback codec %u doesn't match sound codec %u", 
+	playback_codec, sound->codec);
+  }
 
   if (s->parse_sprite->frames[s->parse_sprite->parse_frame].sound_head)
     g_object_unref (s->parse_sprite->frames[s->parse_sprite->parse_frame].sound_head);
   s->parse_sprite->frames[s->parse_sprite->parse_frame].sound_head = sound;
 
-  sound->width = size;
-  sound->original_format = SWFDEC_AUDIO_OUT_GET (type ? 2 : 1, 44100 / (1 << (3 - rate)));
-
-  switch (format) {
-    case 0:
-      if (size == 1)
+  switch (sound->codec) {
+    case SWFDEC_AUDIO_CODEC_UNDEFINED:
+      if (swfdec_audio_format_is_16bit (sound->format)) {
 	SWFDEC_WARNING ("undefined endianness for s16 sound");
-      /* just assume LE and hope it works (FIXME: want a switch for this?) */
-      /* fall through */
-    case 3:
-      sound->format = SWFDEC_AUDIO_FORMAT_UNCOMPRESSED;
+	/* just assume LE and hope it works (FIXME: want a switch for this?) */
+	sound->codec = SWFDEC_AUDIO_CODEC_UNCOMPRESSED;
+      }
       break;
-    case 1:
-      sound->format = SWFDEC_AUDIO_FORMAT_ADPCM;
-      break;
-    case 2:
-      sound->format = SWFDEC_AUDIO_FORMAT_MP3;
+    case SWFDEC_AUDIO_CODEC_MP3:
       /* latency seek */
       latency = swfdec_bits_get_s16 (b);
       break;
-    case 6:
-      sound->format = SWFDEC_AUDIO_FORMAT_NELLYMOSER;
+    case SWFDEC_AUDIO_CODEC_ADPCM:
+    case SWFDEC_AUDIO_CODEC_UNCOMPRESSED:
+    case SWFDEC_AUDIO_CODEC_NELLYMOSER_8KHZ:
+    case SWFDEC_AUDIO_CODEC_NELLYMOSER:
       break;
     default:
-      SWFDEC_WARNING ("unknown format %d", format);
-      sound->format = format;
+      SWFDEC_WARNING ("unknown codec %d", sound->codec);
+      sound->codec = SWFDEC_AUDIO_CODEC_UNDEFINED;
   }
 
   return SWFDEC_STATUS_OK;
@@ -314,7 +299,7 @@ swfdec_sound_chunk_free (SwfdecSoundChunk *chunk)
 }
 
 SwfdecSoundChunk *
-swfdec_sound_parse_chunk (SwfdecSwfDecoder *s, int id)
+swfdec_sound_parse_chunk (SwfdecSwfDecoder *s, SwfdecBits *b, int id)
 {
   int has_envelope;
   int has_loops;
@@ -323,7 +308,6 @@ swfdec_sound_parse_chunk (SwfdecSwfDecoder *s, int id)
   guint i, j;
   SwfdecSound *sound;
   SwfdecSoundChunk *chunk;
-  SwfdecBits *b = &s->b;
 
   sound = swfdec_swf_decoder_get_character (s, id);
   if (!SWFDEC_IS_SOUND (sound)) {
@@ -344,81 +328,62 @@ swfdec_sound_parse_chunk (SwfdecSwfDecoder *s, int id)
   has_in_point = swfdec_bits_getbits (b, 1);
   if (has_in_point) {
     chunk->start_sample = swfdec_bits_get_u32 (b);
+    SWFDEC_LOG ("  start_sample = %u", chunk->start_sample);
   } else {
     chunk->start_sample = 0;
   }
   if (has_out_point) {
     chunk->stop_sample = swfdec_bits_get_u32 (b);
-    if (chunk->stop_sample > sound->n_samples) {
-      SWFDEC_INFO ("more samples specified (%u) than available (%u)", 
-	  chunk->stop_sample, sound->n_samples);
+    if (chunk->stop_sample == 0) {
+      SWFDEC_FIXME ("stop sample == 0???");
+    }
+    SWFDEC_LOG ("  stop_sample = %u", chunk->stop_sample);
+    if (chunk->stop_sample <= chunk->start_sample) {
+      SWFDEC_ERROR ("stopping before starting? (start sample %u, stop sample %u)",
+	  chunk->start_sample, chunk->stop_sample);
+      chunk->stop_sample = 0;
     }
   } else {
-    chunk->stop_sample = sound->n_samples;
+    chunk->stop_sample = 0;
   }
   if (has_loops) {
     chunk->loop_count = swfdec_bits_get_u16 (b);
+    SWFDEC_LOG ("  loop_count = %u", chunk->loop_count);
   } else {
     chunk->loop_count = 1;
   }
   if (has_envelope) {
     chunk->n_envelopes = swfdec_bits_get_u8 (b);
-    chunk->envelope = g_new (SwfdecSoundEnvelope, chunk->n_envelopes);
+    chunk->envelope = g_new0 (SwfdecSoundEnvelope, chunk->n_envelopes);
+    SWFDEC_LOG ("  n_envelopes = %u", chunk->n_envelopes);
   }
-  SWFDEC_LOG ("  start_sample = %u", chunk->start_sample);
-  SWFDEC_LOG ("  stop_sample = %u", chunk->stop_sample);
-  SWFDEC_LOG ("  loop_count = %u", chunk->loop_count);
-  SWFDEC_LOG ("  n_envelopes = %u", chunk->n_envelopes);
 
-  for (i = 0; i < chunk->n_envelopes; i++) {
+  for (i = 0; i < chunk->n_envelopes && swfdec_bits_left (b); i++) {
     chunk->envelope[i].offset = swfdec_bits_get_u32 (b);
-    if (chunk->envelope[i].offset < chunk->start_sample) {
-      SWFDEC_WARNING ("envelope entry offset too small (%d vs %d)",
-	  chunk->envelope[i].offset, chunk->start_sample);
-      chunk->envelope[i].offset = chunk->start_sample;
-    }
-    if (i > 0 && chunk->envelope[i].offset <=
-	chunk->envelope[i-1].offset) {
-      /* FIXME: figure out how to handle this */
-      SWFDEC_ERROR ("sound evelope offsets not sorted");
+    if (i > 0 && chunk->envelope[i-1].offset > chunk->envelope[i].offset) {
+      SWFDEC_ERROR ("unordered sound envelopes");
+      chunk->envelope[i].offset = chunk->envelope[i-1].offset;
     }
+
     for (j = 0; j < 2; j++) {
       chunk->envelope[i].volume[j] = swfdec_bits_get_u16 (b);
       if (chunk->envelope[i].volume[j] > 32768) {
-	SWFDEC_ERROR ("envelope volume too big: %u > 32768", 
+	SWFDEC_ERROR ("envelope volume too big: %u > 32768",
 	    chunk->envelope[i].volume[j]);
 	chunk->envelope[i].volume[j] = 32768;
       }
     }
+
     SWFDEC_LOG ("    envelope = %u { %u, %u }", chunk->envelope[i].offset,
 	(guint) chunk->envelope[i].volume[0], (guint) chunk->envelope[i].volume[1]);
-    /* FIXME: check that mono sound gets averaged and then do this here? */
   }
 
+  if (i < chunk->n_envelopes)
+    SWFDEC_ERROR ("out of bits when reading sound envelopes");
+
   return chunk;
 }
 
-int
-tag_func_start_sound (SwfdecSwfDecoder * s, guint tag)
-{
-  SwfdecBits *b = &s->b;
-  SwfdecSoundChunk *chunk;
-  int id;
-  SwfdecSpriteFrame *frame = &s->parse_sprite->frames[s->parse_sprite->parse_frame];
-
-  id = swfdec_bits_get_u16 (b);
-
-  chunk = swfdec_sound_parse_chunk (s, id);
-  if (chunk) {
-    /* append to keep order */
-    SWFDEC_DEBUG ("appending StartSound event for sound %u to frame %u", id,
-	s->parse_sprite->parse_frame);
-    frame->sound = g_slist_append (frame->sound, chunk);
-  }
-
-  return SWFDEC_STATUS_OK;
-}
-
 int
 tag_func_define_button_sound (SwfdecSwfDecoder * s, guint tag)
 {
@@ -442,7 +407,7 @@ tag_func_define_button_sound (SwfdecSwfDecoder * s, guint tag)
 	    SWFDEC_CHARACTER (button)->id, i);
 	swfdec_sound_chunk_free (button->sounds[i]);
       }
-      button->sounds[i] = swfdec_sound_parse_chunk (s, id);
+      button->sounds[i] = swfdec_sound_parse_chunk (s, &s->b, id);
     }
   }
 
@@ -460,13 +425,79 @@ tag_func_define_button_sound (SwfdecSwfDecoder * s, guint tag)
  * Returns: Number of samples contained in @buffer when rendered
  **/
 guint
-swfdec_sound_buffer_get_n_samples (const SwfdecBuffer *buffer, SwfdecAudioOut format)
+swfdec_sound_buffer_get_n_samples (const SwfdecBuffer *buffer, SwfdecAudioFormat format)
 {
   g_return_val_if_fail (buffer != NULL, 0);
-  g_return_val_if_fail (buffer->length % (2 * SWFDEC_AUDIO_OUT_N_CHANNELS (format)) == 0, 0);
+  g_return_val_if_fail (buffer->length % (2 * swfdec_audio_format_get_channels (format)) == 0, 0);
 
-  return buffer->length / (2 * SWFDEC_AUDIO_OUT_N_CHANNELS (format)) *
-    SWFDEC_AUDIO_OUT_GRANULARITY (format);
+  return buffer->length / (2 * swfdec_audio_format_get_channels (format)) *
+    swfdec_audio_format_get_granularity (format);
+}
+
+static void
+swfdec_sound_buffer_render_stereo (gint16 *dest, const gint16 *source, guint offset,
+    guint n_samples, guint rate)
+{
+  guint i, j;
+
+  source += 2 * (offset / rate);
+  offset %= rate;
+
+  if (offset) {
+    offset = MIN (rate - offset, n_samples);
+    for (i = 0; i < offset; i++) {
+      *dest++ = source[0];
+      *dest++ = source[1];
+    }
+    source += 2;
+    n_samples -= offset;
+  }
+  for (i = rate; i <= n_samples; i += rate) {
+    for (j = 0; j < rate; j++) {
+      *dest++ = source[0];
+      *dest++ = source[1];
+    }
+    source += 2;
+  }
+  n_samples -= i - rate;
+  g_assert (n_samples < rate);
+  for (i = 0; i < n_samples; i++) {
+    *dest++ = source[0];
+    *dest++ = source[1];
+  }
+}
+
+static void
+swfdec_sound_buffer_render_mono (gint16 *dest, const gint16 *source, guint offset,
+    guint n_samples, guint rate)
+{
+  guint i, j;
+
+  source += (offset / rate);
+  offset %= rate;
+
+  if (offset) {
+    offset = MIN (rate - offset, n_samples);
+    for (i = 0; i < offset; i++) {
+      *dest++ = *source;
+      *dest++ = *source;
+    }
+    source++;
+    n_samples -= offset;
+  }
+  for (i = rate; i <= n_samples; i += rate) {
+    for (j = 0; j < rate; j++) {
+      *dest++ = *source;
+      *dest++ = *source;
+    }
+    source++;
+  }
+  n_samples -= i - rate;
+  g_assert (n_samples < rate);
+  for (i = 0; i < n_samples; i++) {
+    *dest++ = *source;
+    *dest++ = *source;
+  }
 }
 
 /**
@@ -480,134 +511,44 @@ swfdec_sound_buffer_get_n_samples (const SwfdecBuffer *buffer, SwfdecAudioOut fo
  * @n_samples: number of samples to render into @dest. If more data would be
  *	       rendered than is available in @source, 0 samples are used instead.
  *
- * Adds data from @source into @dest using the same upsampling algorithm as
- * Flash player.
+ * Adds data from @source into @dest
  **/
-/* NB: if you improve the upsampling algorithm, tests might start to break */
 void
 swfdec_sound_buffer_render (gint16 *dest, const SwfdecBuffer *source, 
-    SwfdecAudioOut format, const SwfdecBuffer *previous, 
+    SwfdecAudioFormat format, const SwfdecBuffer *previous,
     guint offset, guint n_samples)
 {
-  guint i, j;
-  guint channels = SWFDEC_AUDIO_OUT_N_CHANNELS (format);
-  guint rate = SWFDEC_AUDIO_OUT_GRANULARITY (format);
-  gint16 *src, *end;
+  guint channels = swfdec_audio_format_get_channels (format);
+  guint rate = swfdec_audio_format_get_granularity (format);
+  guint width = swfdec_audio_format_is_16bit (format) ? 2 : 1;
+  guint total_samples;
+  gint16 *fixme = NULL;
 
   g_return_if_fail (dest != NULL);
   g_return_if_fail (source != NULL);
   g_return_if_fail (swfdec_sound_buffer_get_n_samples (source, format) > 0);
-  g_return_if_fail (format != 0);
   g_return_if_fail (previous == NULL || swfdec_sound_buffer_get_n_samples (previous, format) > 0);
 
-  src = (gint16 *) source->data;
-  end = (gint16 *) (source->data + source->length);
-  src += channels * (offset / rate);
-  offset %= rate;
-  if (offset) {
-    offset = rate - offset;
-    /* NB: dest will be pointing to uninitialized memory now */
-    dest -= offset * 2;
-    n_samples += offset;
-  }
-  /* this is almost the same as the channels == 1 case, so check for bugfixes in both branches */
-  if (channels == 1) {
-    int values[rate + 1];
-    if (src >= end)
-      n_samples = 0;
-    else if (src != (gint16 *) source->data)
-      values[0] = src[-1];
-    else if (previous)
-      values[0] = ((gint16 *) previous->data)[previous->length / 2 - 1];
-    else
-      values[0] = *src;
-    while (n_samples > 0) {
-      if (src > end)
-	break;
-      else if (src == end)
-	values[rate] = 0;
-      else
-	values[rate] = *src;
-      src++;
-      for (i = rate / 2; i >= 1; i /= 2) {
-	for (j = i; j < rate; j += 2 * i) {
-	  values[j] = (values[j + i] + values[j - i]) / 2;
-	}
-      }
-      for (i = offset; i < MIN (rate, n_samples); i++) {
-	dest[2 * i] += values[i + 1];
-	dest[2 * i + 1] += values[i + 1];
-      }
-      dest += 2 * rate;
-      values[0] = values[rate];
-      offset = 0;
-      n_samples -= MIN (n_samples, rate);
+  total_samples = (source->length / channels / width) * rate;
+  SWFDEC_LOG ("rendering [%u %u) - total: %u samples", offset, n_samples, total_samples);
+  /* FIXME: warn about this? */
+  n_samples = MIN (n_samples, total_samples - offset);
+  
+  if (width == 1) {
+    guint i;
+    /* FIXME: make this faster */
+    fixme = g_try_malloc (source->length * 2);
+    if (fixme == NULL)
+      return;
+    for (i = 0; i < source->length; i++) {
+      fixme[i] = (((gint16) source->data[i]) << 8) - 32768;
     }
+  }
+  if (channels == 2) {
+    swfdec_sound_buffer_render_stereo (dest, (const gint16 *) source->data, offset, n_samples, rate);
   } else {
-    int values[2][rate + 1];
-    if (src >= end) {
-      n_samples = 0;
-    } else if (src != (gint16 *) source->data) {
-      values[0][0] = src[-2];
-      values[1][0] = src[-1];
-    } else if (previous) {
-      values[0][0] = ((gint16 *) previous->data)[previous->length / 2 - 2];
-      values[1][0] = ((gint16 *) previous->data)[previous->length / 2 - 1];
-    } else {
-      values[0][0] = src[0];
-      values[1][0] = src[1];
-    }
-    while (n_samples > 0) {
-      if (src > end) {
-	break;
-      } else if (src == end) {
-	values[0][rate] = 0;
-	values[1][rate] = 0;
-      } else {
-	values[0][rate] = src[0];
-	values[1][rate] = src[1];
-      }
-      src += 2;
-      for (i = rate / 2; i >= 1; i /= 2) {
-	for (j = i; j < rate; j += 2 * i) {
-	  values[0][j] = (values[0][j + i] + values[0][j - i]) / 2;
-	  values[1][j] = (values[1][j + i] + values[1][j - i]) / 2;
-	}
-      }
-      for (i = offset; i < MIN (rate, n_samples); i++) {
-	dest[2 * i] += values[0][i + 1];
-	dest[2 * i + 1] += values[1][i + 1];
-      }
-      dest += 2 * rate;
-      values[0][0] = values[0][rate];
-      values[1][0] = values[1][rate];
-      offset = 0;
-      n_samples -= MIN (n_samples, rate);
-    }
+    swfdec_sound_buffer_render_mono (dest, (const gint16 *) source->data, offset, n_samples, rate);
   }
+  g_free (fixme);
 }
 
-/**
- * swfdec_sound_render:
- * @sound: a #SwfdecSound
- * @dest: target to add to
- * @offset: offset in samples into the data
- * @n_samples: amount of samples to render
- *
- * Renders the given sound onto the existing data in @dest.
- **/
-void
-swfdec_sound_render (SwfdecSound *sound, gint16 *dest,
-    guint offset, guint n_samples)
-{
-  SwfdecBuffer *buffer;
-  SwfdecAudioOut format;
-  g_return_if_fail (SWFDEC_IS_SOUND (sound));
-  /* FIXME: I need a return_if_fail for !created_by_define_sound */
-
-  buffer = swfdec_sound_get_decoded (sound, &format);
-  if (buffer == NULL)
-    return;
-  swfdec_sound_buffer_render (dest, buffer, format, 
-      NULL, offset, n_samples);
-}
