@@ -29,14 +29,14 @@
 #include "swfdec_audio_event.h"
 #include "swfdec_audio_stream.h"
 #include "swfdec_debug.h"
+#include "swfdec_filter.h"
 #include "swfdec_graphic_movie.h"
 #include "swfdec_player_internal.h"
 #include "swfdec_ringbuffer.h"
-#include "swfdec_script.h"
+#include "swfdec_script_internal.h"
 #include "swfdec_sprite.h"
-#include "swfdec_swf_instance.h"
+#include "swfdec_resource.h"
 #include "swfdec_tag.h"
-#include "swfdec_utils.h"
 
 /*** SWFDEC_SPRITE_MOVIE ***/
 
@@ -52,34 +52,91 @@ swfdec_sprite_movie_remove_child (SwfdecMovie *movie, int depth)
   return TRUE;
 }
 
-static void
-swfdec_sprite_movie_run_script (gpointer movie, gpointer data)
-{
-  swfdec_as_object_run (movie, data);
-}
-
-static cairo_operator_t
-swfdec_sprite_convert_operator (guint operator)
-{
-  return CAIRO_OPERATOR_OVER;
-}
-
 static int
 swfdec_get_clipeventflags (SwfdecMovie *movie, SwfdecBits * bits)
 {
-  if (SWFDEC_SWF_DECODER (movie->swf->decoder)->version <= 5) {
+  if (SWFDEC_SWF_DECODER (movie->resource->decoder)->version <= 5) {
     return swfdec_bits_get_u16 (bits);
   } else {
     return swfdec_bits_get_u32 (bits);
   }
 }
 
+static gboolean
+swfdec_sprite_movie_perform_old_place (SwfdecSpriteMovie *movie,
+    SwfdecBits *bits, guint tag)
+{
+  SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
+  SwfdecMovie *mov = SWFDEC_MOVIE (movie);
+  SwfdecMovie *cur;
+  SwfdecSwfDecoder *dec;
+  int depth;
+  cairo_matrix_t transform;
+  gboolean has_ctrans;
+  SwfdecColorTransform ctrans;
+  guint id;
+  SwfdecGraphic *graphic;
+
+  dec = SWFDEC_SWF_DECODER (mov->resource->decoder);
+
+  SWFDEC_LOG ("performing PlaceObject on movie %s", mov->name);
+
+  id = swfdec_bits_get_u16 (bits);
+  SWFDEC_LOG ("  id = %d", id);
+
+  depth = swfdec_bits_get_u16 (bits);
+  if (depth >= 16384) {
+    SWFDEC_FIXME ("depth of placement too high: %u >= 16384", depth);
+  }
+  SWFDEC_LOG ("  depth = %d (=> %d)", depth, depth - 16384);
+  depth -= 16384;
+
+  swfdec_bits_get_matrix (bits, &transform, NULL);
+  SWFDEC_LOG ("  matrix = { %g %g, %g %g } + { %g %g }",
+      transform.xx, transform.yx,
+      transform.xy, transform.yy,
+      transform.x0, transform.y0);
+
+  if (swfdec_bits_left (bits)) {
+    has_ctrans = TRUE;
+    swfdec_bits_get_color_transform (bits, &ctrans);
+    SWFDEC_LOG ("  color transform = %d %d  %d %d  %d %d  %d %d",
+	ctrans.ra, ctrans.rb,
+	ctrans.ga, ctrans.gb,
+	ctrans.ba, ctrans.bb,
+	ctrans.aa, ctrans.ab);
+  } else {
+    has_ctrans = FALSE;
+  }
+
+  /* 3) perform the actions depending on the set properties */
+  cur = swfdec_movie_find (mov, depth);
+  graphic = swfdec_swf_decoder_get_character (dec, id);
+
+  if (!SWFDEC_IS_GRAPHIC (graphic)) {
+    SWFDEC_FIXME ("character %u is not a graphic (does it even exist?), aborting", id);
+    return FALSE;
+  }
+
+  cur = swfdec_movie_new (player, depth, mov, mov->resource, graphic, NULL);
+  swfdec_movie_set_static_properties (cur, &transform,
+      has_ctrans ? &ctrans : NULL, -1, 0, 0, NULL);
+  swfdec_movie_queue_script (cur, SWFDEC_EVENT_INITIALIZE);
+  swfdec_movie_queue_script (cur, SWFDEC_EVENT_CONSTRUCT);
+  swfdec_movie_queue_script (cur, SWFDEC_EVENT_LOAD);
+  swfdec_movie_initialize (cur);
+
+  return TRUE;
+}
+
+
 static gboolean
 swfdec_sprite_movie_perform_place (SwfdecSpriteMovie *movie, SwfdecBits *bits, guint tag)
 {
   SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
   SwfdecMovie *mov = SWFDEC_MOVIE (movie);
   SwfdecMovie *cur;
+  SwfdecSwfDecoder *dec;
   gboolean has_clip_actions;
   gboolean has_clip_depth;
   gboolean has_name;
@@ -88,7 +145,7 @@ swfdec_sprite_movie_perform_place (SwfdecSpriteMovie *movie, SwfdecBits *bits, g
   gboolean has_transform;
   gboolean has_character;
   gboolean move;
-  gboolean depth;
+  int depth;
   gboolean cache;
   gboolean has_blend_mode = 0;
   gboolean has_filter = 0;
@@ -98,9 +155,11 @@ swfdec_sprite_movie_perform_place (SwfdecSpriteMovie *movie, SwfdecBits *bits, g
   guint ratio, id, version;
   SwfdecEventList *events;
   const char *name;
+  guint blend_mode;
   SwfdecGraphic *graphic;
 
-  version = SWFDEC_SWF_DECODER (mov->swf->decoder)->version;
+  dec = SWFDEC_SWF_DECODER (mov->resource->decoder);
+  version = dec->version;
 
   /* 1) check which stuff is set */
   has_clip_actions = swfdec_bits_getbit (bits);
@@ -185,69 +244,15 @@ swfdec_sprite_movie_perform_place (SwfdecSpriteMovie *movie, SwfdecBits *bits, g
   }
 
   if (has_filter) {
-    guint i, n_filters, filter_id;
-    n_filters = swfdec_bits_get_u8 (bits);
-    SWFDEC_LOG ("  filters: %u", n_filters);
-    for (i = 0; i < n_filters && swfdec_bits_left (bits); i++) {
-      filter_id = swfdec_bits_get_u8 (bits);
-      switch (filter_id) {
-	case 0:
-	  SWFDEC_WARNING ("    drop shadow");
-	  swfdec_bits_skip_bytes (bits, 16);
-	  break;
-	case 1:
-	  SWFDEC_WARNING ("    blur");
-	  swfdec_bits_skip_bytes (bits, 9);
-	  break;
-	case 2:
-	  SWFDEC_WARNING ("    glow");
-	  swfdec_bits_skip_bytes (bits, 15);
-	  break;
-	case 3:
-	  SWFDEC_WARNING ("    bevel");
-	  swfdec_bits_skip_bytes (bits, 27);
-	  break;
-	case 4:
-	  {
-	    guint n;
-	    n = swfdec_bits_get_u8 (bits);
-	    SWFDEC_WARNING ("    gradient glow");
-	    swfdec_bits_skip_bytes (bits, n * 5 + 19);
-	  }
-	  break;
-	case 5:
-	  {
-	    guint x, y;
-	    x = swfdec_bits_get_u8 (bits);
-	    y = swfdec_bits_get_u8 (bits);
-	    SWFDEC_WARNING ("    %u x %u convolution", x, y);
-	    swfdec_bits_skip_bytes (bits, (x + y) * 4 + 13);
-	  }
-	  break;
-	case 6:
-	  SWFDEC_WARNING ("    color matrix");
-	  swfdec_bits_skip_bytes (bits, 20 * 4);
-	  break;
-	case 7:
-	  {
-	    guint n;
-	    n = swfdec_bits_get_u8 (bits);
-	    SWFDEC_WARNING ("    gradient bevel");
-	    swfdec_bits_skip_bytes (bits, n * 5 + 19);
-	  }
-	  break;
-	default:
-	  SWFDEC_ERROR ("unknown filter id %u", filter_id);
-	  break;
-      }
-    }
+    GSList *filters = swfdec_filter_parse (player, bits);
+    g_slist_free (filters);
   }
 
   if (has_blend_mode) {
-    /* FIXME: implement */
-    guint operator = swfdec_bits_get_u8 (bits);
-    swfdec_sprite_convert_operator (operator);
-    SWFDEC_ERROR ("  operator = %u", operator);
+    blend_mode = swfdec_bits_get_u8 (bits);
+    SWFDEC_LOG ("  blend mode = %u", blend_mode);
+  } else {
+    blend_mode = 0;
   }
 
   if (has_clip_actions) {
@@ -269,15 +274,16 @@ swfdec_sprite_movie_perform_place (SwfdecSpriteMovie *movie, SwfdecBits *bits, g
       SwfdecBits action_bits;
 
       swfdec_bits_init_bits (&action_bits, bits, length);
-      if (event_flags & SWFDEC_EVENT_KEY_PRESS)
+      if (event_flags & (1<<SWFDEC_EVENT_KEY_PRESS))
 	key_code = swfdec_bits_get_u8 (&action_bits);
       else
 	key_code = 0;
 
       SWFDEC_INFO ("clip event with flags 0x%X, key code %d", event_flags, key_code);
 #define SWFDEC_IMPLEMENTED_EVENTS \
-  (SWFDEC_EVENT_LOAD | SWFDEC_EVENT_UNLOAD | SWFDEC_EVENT_ENTER | SWFDEC_EVENT_INITIALIZE | SWFDEC_EVENT_CONSTRUCT | \
-   SWFDEC_EVENT_MOUSE_DOWN | SWFDEC_EVENT_MOUSE_MOVE | SWFDEC_EVENT_MOUSE_UP)
+  ((1<< SWFDEC_EVENT_LOAD) | (1<<SWFDEC_EVENT_UNLOAD) | (1<<SWFDEC_EVENT_ENTER) | \
+   (1<< SWFDEC_EVENT_INITIALIZE) | (1<<SWFDEC_EVENT_CONSTRUCT) | \
+   (1<< SWFDEC_EVENT_MOUSE_DOWN) | (1<<SWFDEC_EVENT_MOUSE_MOVE) | (1<<SWFDEC_EVENT_MOUSE_UP))
       if (event_flags & ~SWFDEC_IMPLEMENTED_EVENTS) {
 	SWFDEC_ERROR ("using non-implemented clip events %u", event_flags & ~SWFDEC_IMPLEMENTED_EVENTS);
       }
@@ -295,7 +301,7 @@ swfdec_sprite_movie_perform_place (SwfdecSpriteMovie *movie, SwfdecBits *bits, g
 
   /* 3) perform the actions depending on the set properties */
   cur = swfdec_movie_find (mov, depth);
-  graphic = swfdec_swf_decoder_get_character (SWFDEC_SWF_DECODER (mov->swf->decoder), id);
+  graphic = swfdec_swf_decoder_get_character (dec, id);
   if (move) {
     if (cur == NULL) {
       SWFDEC_INFO ("no movie at depth %d, ignoring move command", depth);
@@ -307,7 +313,7 @@ swfdec_sprite_movie_perform_place (SwfdecSpriteMovie *movie, SwfdecBits *bits, g
 	klass->replace (cur, graphic);
     }
     swfdec_movie_set_static_properties (cur, has_transform ? &transform : NULL, 
-	has_ctrans ? &ctrans : NULL, ratio, clip_depth, events);
+	has_ctrans ? &ctrans : NULL, ratio, clip_depth, blend_mode, events);
   } else {
     if (cur != NULL && version > 5) {
       SWFDEC_INFO ("depth %d is already occupied by movie %s, not placing", depth, cur->name);
@@ -319,14 +325,12 @@ swfdec_sprite_movie_perform_place (SwfdecSpriteMovie *movie, SwfdecBits *bits, g
 	swfdec_event_list_free (events);
       return FALSE;
     }
-    cur = swfdec_movie_new (player, depth, mov, graphic, name);
+    cur = swfdec_movie_new (player, depth, mov, mov->resource, graphic, name);
     swfdec_movie_set_static_properties (cur, has_transform ? &transform : NULL, 
-	has_ctrans ? &ctrans : NULL, ratio, clip_depth, events);
-    if (SWFDEC_IS_SPRITE_MOVIE (cur)) {
-      g_queue_push_tail (player->init_queue, cur);
-      g_queue_push_tail (player->construct_queue, cur);
-      swfdec_movie_queue_script (cur, SWFDEC_EVENT_LOAD);
-    }
+	has_ctrans ? &ctrans : NULL, ratio, clip_depth, blend_mode, events);
+    swfdec_movie_queue_script (cur, SWFDEC_EVENT_INITIALIZE);
+    swfdec_movie_queue_script (cur, SWFDEC_EVENT_CONSTRUCT);
+    swfdec_movie_queue_script (cur, SWFDEC_EVENT_LOAD);
     swfdec_movie_initialize (cur);
   }
 
@@ -336,15 +340,31 @@ out:
   return TRUE;
 }
 
+static void
+swfdec_sprite_movie_start_sound (SwfdecMovie *movie, SwfdecBits *bits)
+{
+  SwfdecSoundChunk *chunk;
+  int id;
+
+  id = swfdec_bits_get_u16 (bits);
+  chunk = swfdec_sound_parse_chunk (SWFDEC_SWF_DECODER (movie->resource->decoder), bits, id);
+  if (chunk) {
+    SwfdecAudio *audio = swfdec_audio_event_new_from_chunk (SWFDEC_PLAYER (
+	  SWFDEC_AS_OBJECT (movie)->context), chunk);
+    if (audio)
+      g_object_unref (audio);
+  }
+}
+
 static gboolean
 swfdec_sprite_movie_perform_one_action (SwfdecSpriteMovie *movie, guint tag, SwfdecBuffer *buffer,
-    gboolean skip_scripts)
+    gboolean skip_scripts, gboolean first_time)
 {
   SwfdecMovie *mov = SWFDEC_MOVIE (movie);
   SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (mov)->context);
   SwfdecBits bits;
 
-  g_assert (mov->swf);
+  g_assert (mov->resource);
   swfdec_bits_init (&bits, buffer);
 
   SWFDEC_LOG ("%p: executing %uth tag %s in frame %u", movie, movie->next_action - 1, 
@@ -354,11 +374,13 @@ swfdec_sprite_movie_perform_one_action (SwfdecSpriteMovie *movie, guint tag, Swf
       SWFDEC_LOG ("SCRIPT action");
       if (!skip_scripts) {
 	SwfdecScript *script = swfdec_swf_decoder_get_script (
-	    SWFDEC_SWF_DECODER (mov->swf->decoder), buffer->data);
+	    SWFDEC_SWF_DECODER (mov->resource->decoder), buffer->data);
 	g_assert (script);
-	swfdec_player_add_action (player, mov, swfdec_sprite_movie_run_script, script);
+	swfdec_player_add_action_script (player, mov, script, 2);
       }
       return TRUE;
+    case SWFDEC_TAG_PLACEOBJECT:
+      return swfdec_sprite_movie_perform_old_place (movie, &bits, tag);
     case SWFDEC_TAG_PLACEOBJECT2:
     case SWFDEC_TAG_PLACEOBJECT3:
       return swfdec_sprite_movie_perform_place (movie, &bits, tag);
@@ -377,6 +399,9 @@ swfdec_sprite_movie_perform_one_action (SwfdecSpriteMovie *movie, guint tag, Swf
 	  SWFDEC_INFO ("could not remove, no child at depth %d", depth);
       }
       return TRUE;
+    case SWFDEC_TAG_STARTSOUND:
+      swfdec_sprite_movie_start_sound (mov, &bits);
+      return TRUE;
     case SWFDEC_TAG_SHOWFRAME:
       if (movie->frame < movie->n_frames) {
 	movie->frame++;
@@ -384,6 +409,66 @@ swfdec_sprite_movie_perform_one_action (SwfdecSpriteMovie *movie, guint tag, Swf
 	SWFDEC_ERROR ("too many ShowFrame tags");
       }
       return FALSE;
+    case SWFDEC_TAG_EXPORTASSETS:
+      {
+	SwfdecResource *resource = swfdec_movie_get_own_resource (mov);
+	guint i, count;
+
+	g_assert (resource); /* must hold, ExportAssets can only be in root movies */
+	if (!first_time)
+	  return TRUE;
+	count = swfdec_bits_get_u16 (&bits);
+	SWFDEC_LOG ("exporting %u assets", count);
+	for (i = 0; i < count && swfdec_bits_left (&bits); i++) {
+	  guint id;
+	  SwfdecCharacter *object;
+	  char *name;
+	  id = swfdec_bits_get_u16 (&bits);
+	  object = swfdec_swf_decoder_get_character (SWFDEC_SWF_DECODER (resource->decoder), id);
+	  name = swfdec_bits_get_string_with_version (&bits, SWFDEC_AS_CONTEXT (player)->version);
+	  if (object == NULL) {
+	    SWFDEC_ERROR ("cannot export id %u as %s, id wasn't found", id, name);
+	  } else if (name == NULL) {
+	    SWFDEC_ERROR ("cannot export id %u, no name was given", id);
+	  } else {
+	    SWFDEC_LOG ("exporting %s %u as %s", G_OBJECT_TYPE_NAME (object), id, name);
+	    swfdec_resource_add_export (resource, object, name); 
+	  }
+	  g_free (name);
+	}
+      }
+      return TRUE;
+    case SWFDEC_TAG_DOINITACTION:
+      if (!first_time)
+	return TRUE;
+      if (!swfdec_movie_get_own_resource (mov)) {
+	SWFDEC_FIXME ("behavior of init actions in DefineSprite untested");
+      }
+      {
+	guint id;
+	SwfdecSprite *sprite;
+	char *name;
+
+	id = swfdec_bits_get_u16 (&bits);
+	SWFDEC_LOG ("InitAction");
+	SWFDEC_LOG ("  id = %u", id);
+	sprite = swfdec_swf_decoder_get_character (SWFDEC_SWF_DECODER (mov->resource->decoder), id);
+	if (!SWFDEC_IS_SPRITE (sprite)) {
+	  SWFDEC_ERROR ("character %u is not a sprite", id);
+	  return TRUE;
+	}
+	if (sprite->init_action != NULL) {
+	  SWFDEC_ERROR ("sprite %u already has an init action", id);
+	  return TRUE;
+	}
+	name = g_strdup_printf ("InitAction %u", id);
+	sprite->init_action = swfdec_script_new_from_bits (&bits, name, SWFDEC_AS_CONTEXT (player)->version);
+	g_free (name);
+	if (sprite->init_action) {
+	  swfdec_player_add_action_script (player, mov, sprite->init_action, 0);
+	}
+      }
+      return TRUE;
     default:
       g_assert_not_reached ();
       return FALSE;
@@ -404,6 +489,22 @@ swfdec_movie_is_compatible (SwfdecMovie *movie, SwfdecMovie *with)
   return TRUE;
 }
 
+static GList *
+my_g_list_split (GList *list, GList *split)
+{
+  GList *prev;
+
+  if (split == NULL)
+    return list;
+
+  prev = split->prev;
+  if (prev == NULL)
+    return NULL;
+  prev->next = NULL;
+  split->prev = NULL;
+  return list;
+}
+
 void
 swfdec_sprite_movie_goto (SwfdecSpriteMovie *movie, guint goto_frame)
 {
@@ -433,9 +534,22 @@ swfdec_sprite_movie_goto (SwfdecSpriteMovie *movie, guint goto_frame)
   SWFDEC_DEBUG ("performing goto %u -> %u for character %u", 
       movie->frame, goto_frame, SWFDEC_CHARACTER (movie->sprite)->id);
   if (goto_frame < movie->frame) {
+    GList *walk;
     movie->frame = 0;
-    old = mov->list;
-    mov->list = NULL;
+    for (walk = mov->list; walk && 
+	swfdec_depth_classify (SWFDEC_MOVIE (walk->data)->depth) != SWFDEC_DEPTH_CLASS_TIMELINE;
+	walk = walk->next) {
+      /* do nothing */
+    }
+    old = walk;
+    mov->list = my_g_list_split (mov->list, old);
+    for (walk = old; walk && 
+	swfdec_depth_classify (SWFDEC_MOVIE (walk->data)->depth) == SWFDEC_DEPTH_CLASS_TIMELINE;
+	walk = walk->next) {
+      /* do nothing */
+    }
+    old = my_g_list_split (old, walk);
+    mov->list = g_list_concat (mov->list, walk);
     n = goto_frame;
     movie->next_action = 0;
   } else {
@@ -445,15 +559,18 @@ swfdec_sprite_movie_goto (SwfdecSpriteMovie *movie, guint goto_frame)
   }
   while (n) {
     guint tag;
+    gboolean first_time;
     SwfdecBuffer *buffer;
-    /* FIXME: These actions should probably just be added to the action queue */
-    if (movie == mov->swf->movie &&
-	mov->swf->parse_frame <= movie->frame)
-      swfdec_swf_instance_advance (mov->swf);
     if (!swfdec_sprite_get_action (movie->sprite, movie->next_action, &tag, &buffer))
       break;
     movie->next_action++;
-    if (!swfdec_sprite_movie_perform_one_action (movie, tag, buffer, n > 1))
+    if (movie->next_action > movie->max_action) {
+      first_time = TRUE;
+      movie->max_action = movie->next_action;
+    } else {
+      first_time = FALSE;
+    }
+    if (!swfdec_sprite_movie_perform_one_action (movie, tag, buffer, n > 1, first_time))
       n--;
   }
   /* now try to copy eventual movies */
@@ -481,7 +598,8 @@ swfdec_sprite_movie_goto (SwfdecSpriteMovie *movie, guint goto_frame)
 	if (klass->replace)
 	  klass->replace (prev, cur->graphic);
 	swfdec_movie_set_static_properties (prev, &cur->original_transform,
-	    &cur->original_ctrans, cur->original_ratio, cur->clip_depth, cur->events);
+	    &cur->original_ctrans, cur->original_ratio, cur->clip_depth, 
+	    cur->blend_mode, cur->events);
 	swfdec_movie_destroy (cur);
 	cur = prev;
 	continue;
@@ -511,11 +629,24 @@ swfdec_sprite_movie_dispose (GObject *object)
 }
 
 static void
-swfdec_sprite_movie_do_enter_frame (gpointer movie, gpointer unused)
+swfdec_sprite_movie_init_movie (SwfdecMovie *mov)
 {
-  if (SWFDEC_MOVIE (movie)->will_be_removed)
-    return;
-  swfdec_movie_execute_script (movie, SWFDEC_EVENT_ENTER);
+  SwfdecSpriteMovie *movie = SWFDEC_SPRITE_MOVIE (mov);
+
+  g_assert (movie->frame == (guint) -1);
+  movie->frame = 0;
+  swfdec_sprite_movie_goto (movie, 1);
+}
+
+static void
+swfdec_sprite_movie_add (SwfdecAsObject *object)
+{
+  SwfdecPlayer *player = SWFDEC_PLAYER (object->context);
+
+  if (player->MovieClip)
+    swfdec_as_object_set_constructor (object, player->MovieClip);
+
+  SWFDEC_AS_OBJECT_CLASS (swfdec_sprite_movie_parent_class)->add (object);
 }
 
 static void
@@ -528,7 +659,10 @@ swfdec_sprite_movie_iterate (SwfdecMovie *mov)
   if (mov->will_be_removed)
     return;
 
-  swfdec_player_add_action (player, movie, swfdec_sprite_movie_do_enter_frame, NULL);
+  if (movie->sprite && movie->frame == (guint) -1)
+    movie->frame = 0;
+
+  swfdec_player_add_action (player, mov, SWFDEC_EVENT_ENTER, 2);
   if (movie->playing && movie->sprite != NULL) {
     if (movie->frame == movie->n_frames)
       goto_frame = 1;
@@ -547,7 +681,6 @@ swfdec_sprite_movie_iterate_end (SwfdecMovie *mov)
   SwfdecSpriteMovie *movie = SWFDEC_SPRITE_MOVIE (mov);
   SwfdecSpriteFrame *last;
   SwfdecSpriteFrame *current;
-  GSList *walk;
   SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (mov)->context);
 
   if (!SWFDEC_MOVIE_CLASS (swfdec_sprite_movie_parent_class)->iterate_end (mov)) {
@@ -558,20 +691,9 @@ swfdec_sprite_movie_iterate_end (SwfdecMovie *mov)
   if (movie->sprite == NULL)
     return TRUE;
   g_assert (movie->frame <= movie->n_frames);
-  if (movie->frame == 0) {
-    SWFDEC_WARNING ("not at first frame yet");
+  if (movie->frame == 0)
     return TRUE;
-  }
   current = &movie->sprite->frames[movie->frame - 1];
-  /* first start all event sounds */
-  /* FIXME: is this correct? */
-  if (movie->sound_frame != movie->frame) {
-    for (walk = current->sound; walk; walk = walk->next) {
-      SwfdecAudio *audio = swfdec_audio_event_new (player, walk->data);
-      if (audio)
-	g_object_unref (audio);
-    }
-  }
 
   /* then do the streaming thing */
   if (current->sound_head == NULL ||
@@ -608,44 +730,12 @@ new_decoder:
 
   if (current->sound_block) {
     movie->sound_stream = swfdec_audio_stream_new (player, 
-	movie->sprite, movie->frame);
+	movie->sprite, movie->frame - 1);
     movie->sound_frame = movie->frame;
   }
   return TRUE;
 }
 
-static void
-swfdec_sprite_movie_init_movie (SwfdecMovie *mov)
-{
-  SwfdecSpriteMovie *movie = SWFDEC_SPRITE_MOVIE (mov);
-  SwfdecAsContext *context = SWFDEC_AS_OBJECT (movie)->context;
-  SwfdecAsObject *constructor = NULL;
-
-  g_assert (mov->swf != NULL);
-
-  if (movie->sprite) {
-    const char *name;
-
-    g_assert (movie->sprite->parse_frame > 0);
-    movie->n_frames = movie->sprite->n_frames;
-    name = swfdec_swf_instance_get_export_name (mov->swf,
-	SWFDEC_CHARACTER (movie->sprite));
-    if (name != NULL) {
-      name = swfdec_as_context_get_string (context, name);
-      constructor = swfdec_player_get_export_class (SWFDEC_PLAYER (context),
-	  name);
-    }
-  }
-  if (constructor == NULL)
-    constructor = SWFDEC_PLAYER (context)->MovieClip;
-
-  swfdec_as_object_set_constructor (SWFDEC_AS_OBJECT (movie), constructor, FALSE);
-  swfdec_sprite_movie_goto (movie, 1);
-  if (!swfdec_sprite_movie_iterate_end (mov)) {
-    g_assert_not_reached ();
-  }
-}
-
 static void
 swfdec_sprite_movie_finish_movie (SwfdecMovie *mov)
 {
@@ -660,41 +750,6 @@ swfdec_sprite_movie_finish_movie (SwfdecMovie *mov)
   }
 }
 
-static SwfdecMovie *
-swfdec_sprite_movie_get_by_name (SwfdecMovie *movie, const char *name)
-{
-  GList *walk;
-  guint version = SWFDEC_AS_OBJECT (movie)->context->version;
-
-  for (walk = movie->list; walk; walk = walk->next) {
-    SwfdecMovie *cur = walk->data;
-    if (cur->original_name == SWFDEC_AS_STR_EMPTY)
-      continue;
-    if ((version >= 7 && cur->name == name) ||
-	swfdec_str_case_equal (cur->name, name))
-      return cur;
-  }
-  return NULL;
-}
-
-static gboolean
-swfdec_sprite_movie_get_variable (SwfdecAsObject *object, const char *variable,
-    SwfdecAsValue *val, guint *flags)
-{
-  SwfdecMovie *movie;
-
-  if (SWFDEC_AS_OBJECT_CLASS (swfdec_sprite_movie_parent_class)->get (object, variable, val, flags))
-    return TRUE;
-  
-  movie = swfdec_sprite_movie_get_by_name (SWFDEC_MOVIE (object), variable);
-  if (movie == NULL)
-    return FALSE;
-
-  SWFDEC_AS_VALUE_SET_OBJECT (val, SWFDEC_AS_OBJECT (movie));
-  *flags = 0;
-  return TRUE;
-}
-
 static void
 swfdec_sprite_movie_mark (SwfdecAsObject *object)
 {
@@ -718,7 +773,7 @@ swfdec_sprite_movie_class_init (SwfdecSpriteMovieClass * g_class)
 
   object_class->dispose = swfdec_sprite_movie_dispose;
 
-  asobject_class->get = swfdec_sprite_movie_get_variable;
+  asobject_class->add = swfdec_sprite_movie_add;
   asobject_class->mark = swfdec_sprite_movie_mark;
 
   movie_class->init_movie = swfdec_sprite_movie_init_movie;
@@ -731,5 +786,87 @@ static void
 swfdec_sprite_movie_init (SwfdecSpriteMovie * movie)
 {
   movie->playing = TRUE;
+  movie->frame = (guint) -1;
 }
 
+/* cute little hack */
+extern void
+swfdec_sprite_movie_clear (SwfdecAsContext *cx, SwfdecAsObject *object,
+    guint argc, SwfdecAsValue *argv, SwfdecAsValue *rval);
+/**
+ * swfdec_sprite_movie_unload:
+ * @movie: a #SwfdecMovie
+ *
+ * Clears all contents from the given movie. This means deleting all
+ * variables and removing all children movie clips.
+ **/
+void
+swfdec_sprite_movie_unload (SwfdecSpriteMovie *movie)
+{
+  SwfdecMovie *mov;
+  SwfdecAsValue hack;
+
+  g_return_if_fail (SWFDEC_IS_SPRITE_MOVIE (movie));
+
+  mov = SWFDEC_MOVIE (movie);
+  swfdec_sprite_movie_clear (SWFDEC_AS_OBJECT (movie)->context, 
+      SWFDEC_AS_OBJECT (movie), 0, NULL, &hack);
+  /* FIXME: destroy or unload? */
+  while (mov->list)
+    swfdec_movie_remove (mov->list->data);
+  swfdec_as_object_delete_all_variables (SWFDEC_AS_OBJECT (movie));
+  movie->frame = (guint) -1;
+  movie->n_frames = 0;
+  movie->next_action = 0;
+  movie->max_action = 0;
+  movie->sprite = NULL;
+  swfdec_movie_queue_update (SWFDEC_MOVIE (movie), SWFDEC_MOVIE_INVALID_EXTENTS);
+}
+
+/**
+ * swfdec_sprite_movie_get_frames_loaded:
+ * @movie: a #SwfdecSpriteMovie
+ *
+ * Computes the number of loaded frames as used by the _framesloaded property
+ * or the WaitForFrame actions. If the @movie is fully loaded, this is the 
+ * amount of total frames of the sprite it displays, or 0 if it has no sprite.
+ * If the movie is not fully loaded, it is the amount of frames that are 
+ * completely loaded minus one. Welcome to the world of Flash.
+ *
+ * Returns: The number of loaded frames as reported by ActionScript.
+ **/
+int
+swfdec_sprite_movie_get_frames_loaded (SwfdecSpriteMovie *movie)
+{
+  SwfdecResource *resource;
+  SwfdecDecoder *dec;
+
+  g_return_val_if_fail (SWFDEC_IS_SPRITE_MOVIE (movie), 0);
+
+  resource = swfdec_movie_get_own_resource (SWFDEC_MOVIE (movie));
+  if (resource == NULL)
+    return 1;
+  dec = resource->decoder;
+  if (dec == NULL)
+    return -1;
+  if (dec->frames_loaded < dec->frames_total)
+    return dec->frames_loaded - 1;
+  return dec->frames_total;
+}
+
+int
+swfdec_sprite_movie_get_frames_total (SwfdecSpriteMovie *movie)
+{
+  SwfdecResource *resource;
+  SwfdecDecoder *dec;
+
+  g_return_val_if_fail (SWFDEC_IS_SPRITE_MOVIE (movie), 0);
+
+  resource = swfdec_movie_get_own_resource (SWFDEC_MOVIE (movie));
+  if (resource == NULL)
+    return 1;
+  dec = resource->decoder;
+  if (dec == NULL)
+    return 0;
+  return dec->frames_total;
+}
