@@ -36,12 +36,15 @@
 #include "swfdec_debug.h"
 #include "swfdec_player_internal.h"
 #include "swfdec_script.h"
+#include "swfdec_script_internal.h"
 #include "swfdec_sprite.h"
+#include "swfdec_tag.h"
 
 enum {
   SWFDEC_STATE_INIT1 = 0,
   SWFDEC_STATE_INIT2,
-  SWFDEC_STATE_PARSETAG,
+  SWFDEC_STATE_PARSE_FIRST_TAG,
+  SWFDEC_STATE_PARSE_TAG,
   SWFDEC_STATE_EOF,
 };
 
@@ -51,49 +54,17 @@ static void
 swfdec_swf_decoder_dispose (GObject *object)
 {
   SwfdecSwfDecoder *s = SWFDEC_SWF_DECODER (object);
-  guint i,j;
-
-  if (s->root_actions) {
-    for (i = 0; i < s->main_sprite->n_frames; i++) {
-      GArray *array = s->root_actions[i];
-      if (array) {
-	for (j = 0; j < array->len; j++) {
-	  SwfdecRootAction *action = &g_array_index (array, SwfdecRootAction, j);
-
-	  switch (action->type) {
-	    case SWFDEC_ROOT_ACTION_EXPORT:
-	      {
-		SwfdecRootExportData *data = action->data;
-		g_free (data->name);
-		g_object_unref (data->character);
-		g_free (data);
-	      }
-	      break;
-	    case SWFDEC_ROOT_ACTION_INIT_SCRIPT:
-	      swfdec_script_unref (action->data);
-	      break;
-	    default:
-	      g_assert_not_reached ();
-	      break;
-	  }
-	}
-	g_array_free (array, TRUE);
-      }
-    }
-    g_free (s->root_actions);
-    s->root_actions = NULL;
-  }
 
   g_hash_table_destroy (s->characters);
   g_object_unref (s->main_sprite);
   g_hash_table_destroy (s->scripts);
 
-  if (s->uncompressed_buffer) {
+  if (s->compressed)
     inflateEnd (&s->z);
-    swfdec_buffer_unref (s->uncompressed_buffer);
-    s->uncompressed_buffer = NULL;
+  if (s->buffer) {
+    swfdec_buffer_unref (s->buffer);
+    s->buffer = NULL;
   }
-  swfdec_buffer_queue_unref (s->input_queue);
 
   if (s->jpegtables) {
     swfdec_buffer_unref (s->jpegtables);
@@ -116,32 +87,60 @@ zfree (void *opaque, void *addr)
   g_free (addr);
 }
 
+static SwfdecBuffer *
+swfdec_buffer_merge (const SwfdecBuffer *front, const SwfdecBuffer *end)
+{
+  SwfdecBuffer *new;
+
+  g_return_val_if_fail (front != NULL, NULL);
+  g_return_val_if_fail (end != NULL, NULL);
+
+  new = swfdec_buffer_new_and_alloc (front->length + end->length);
+  if (front->length)
+    memcpy (new->data, front->data, front->length);
+  if (end->length)
+    memcpy (new->data + front->length, end->data, end->length);
+  return new;
+}
+
 static gboolean
-swfdec_swf_decoder_deflate_all (SwfdecSwfDecoder * s)
+swfdec_swf_decoder_deflate (SwfdecSwfDecoder * s, SwfdecBuffer *buffer)
 {
   int ret;
-  SwfdecBuffer *buffer;
   SwfdecDecoder *dec = SWFDEC_DECODER (s);
   
-  while ((buffer = swfdec_buffer_queue_pull_buffer (SWFDEC_DECODER (s)->queue))) {
-    if (s->compressed) {
-      int offset = s->z.total_out;
+  if (s->buffer == NULL) {
+    /* never written to */
+    g_assert (s->state == SWFDEC_STATE_INIT1);
+    s->buffer = buffer;
+  } else if (s->state == SWFDEC_STATE_INIT1) {
+    /* not initialized yet */
+    SwfdecBuffer *merge = swfdec_buffer_merge (s->buffer, buffer);
+    swfdec_buffer_unref (s->buffer);
+    swfdec_buffer_unref (buffer);
+    s->buffer = merge;
+  } else if (s->compressed) {
+    s->z.next_in = buffer->data;
+    s->z.avail_in = buffer->length;
+    ret = inflate (&s->z, Z_SYNC_FLUSH);
+    if (ret < Z_OK) {
+      SWFDEC_ERROR ("error uncompressing data: %s", s->z.msg);
+      return FALSE;
+    }
 
-      s->z.next_in = buffer->data;
-      s->z.avail_in = buffer->length;
-      ret = inflate (&s->z, Z_SYNC_FLUSH);
-      if (ret < Z_OK) {
-	SWFDEC_ERROR ("error uncompressing data: %s", s->z.msg);
-	return FALSE;
-      }
+    dec->bytes_loaded = s->z.total_out + 8;
+    swfdec_buffer_unref (buffer);
+  } else {
+    guint max = buffer->length;
 
-      swfdec_buffer_unref (buffer);
-
-      buffer = swfdec_buffer_new_subbuffer (s->uncompressed_buffer, offset,
-	  s->z.total_out - offset);
-    } 
-    dec->bytes_loaded += buffer->length;
-    swfdec_buffer_queue_push (s->input_queue, buffer);
+    if (dec->bytes_loaded + max > s->buffer->length) {
+      SWFDEC_WARNING ("%u bytes more than declared filesize", 
+	  dec->bytes_loaded + max - s->buffer->length);
+      max = s->buffer->length - dec->bytes_loaded;
+    }
+    memcpy (s->buffer->data + dec->bytes_loaded, buffer->data, max);
+    dec->bytes_loaded += max;
+    swfdec_buffer_unref (buffer);
   }
 
   return TRUE;
@@ -150,59 +149,79 @@ swfdec_swf_decoder_deflate_all (SwfdecSwfDecoder * s)
 static gboolean
 swf_inflate_init (SwfdecSwfDecoder * s)
 {
-  SwfdecDecoder *dec = SWFDEC_DECODER (s);
   z_stream *z;
   int ret;
-  guint8 *data;
-
-  data = g_try_malloc (dec->bytes_total - 8);
-  if (data == NULL)
-    return FALSE;
-  s->uncompressed_buffer = swfdec_buffer_new_for_data (data, dec->bytes_total - 8);
   z = &s->z;
   z->zalloc = zalloc;
   z->zfree = zfree;
   ret = inflateInit (z);
   SWFDEC_DEBUG ("inflateInit returned %d", ret);
 
-  z->next_out = s->uncompressed_buffer->data;
-  z->avail_out = s->uncompressed_buffer->length;
+  z->next_out = s->buffer->data + 8;
+  z->avail_out = s->buffer->length - 8;
   z->opaque = NULL;
   return TRUE;
 }
 
+static void
+swfdec_swf_decoder_init_bits (SwfdecSwfDecoder *dec, SwfdecBits *bits)
+{
+  swfdec_bits_init (bits, dec->buffer);
+  bits->end = bits->ptr + SWFDEC_DECODER (dec)->bytes_loaded;
+  bits->ptr += dec->bytes_parsed;
+  g_assert (bits->ptr <= bits->end);
+}
+
+static void
+swfdec_swf_decoder_flush_bits (SwfdecSwfDecoder *dec, SwfdecBits *bits)
+{
+  g_assert (bits->idx == 0);
+  g_assert (bits->buffer == dec->buffer);
+
+  dec->bytes_parsed = bits->ptr - dec->buffer->data;
+  g_assert (dec->bytes_parsed <= SWFDEC_DECODER (dec)->bytes_loaded);
+}
+
 static int
 swf_parse_header1 (SwfdecSwfDecoder * s)
 {
   SwfdecDecoder *dec = SWFDEC_DECODER (s);
   int sig1, sig2, sig3;
-  SwfdecBuffer *buffer;
+  SwfdecBuffer *buffer, *rest;
+  SwfdecBits bits;
+  guint8 *data;
 
-  /* NB: we're still reading from the original queue, since deflating is not initialized yet */
-  buffer = swfdec_buffer_queue_pull (dec->queue, 8);
-  if (buffer == NULL) {
+  g_assert (s->buffer != NULL);
+  if (s->buffer->length <= 8) {
     return SWFDEC_STATUS_NEEDBITS;
   }
 
-  s->b.buffer = buffer;
-  s->b.ptr = buffer->data;
-  s->b.idx = 0;
-  s->b.end = buffer->data + buffer->length;
-
-  sig1 = swfdec_bits_get_u8 (&s->b);
-  sig2 = swfdec_bits_get_u8 (&s->b);
-  sig3 = swfdec_bits_get_u8 (&s->b);
-
-  s->version = swfdec_bits_get_u8 (&s->b);
-  dec->bytes_total = swfdec_bits_get_u32 (&s->b);
-
-  swfdec_buffer_unref (buffer);
-  SWFDEC_DECODER (s)->bytes_loaded = 8;
+  swfdec_bits_init (&bits, s->buffer);
 
+  sig1 = swfdec_bits_get_u8 (&bits);
+  sig2 = swfdec_bits_get_u8 (&bits);
+  sig3 = swfdec_bits_get_u8 (&bits);
   if ((sig1 != 'F' && sig1 != 'C') || sig2 != 'W' || sig3 != 'S') {
     return SWFDEC_STATUS_ERROR;
   }
 
+  s->version = swfdec_bits_get_u8 (&bits);
+  dec->bytes_total = swfdec_bits_get_u32 (&bits);
+  if (dec->bytes_total <= 8) {
+    SWFDEC_ERROR ("Joke? Flash files need to be bigger than %u bytes", dec->bytes_total);
+    dec->bytes_total = 0;
+    return SWFDEC_STATUS_ERROR;
+  }
+  rest = swfdec_bits_get_buffer (&bits, -1);
+
+  data = g_try_malloc (dec->bytes_total);
+  if (data == NULL)
+    return SWFDEC_STATUS_ERROR;
+  buffer = swfdec_buffer_new_for_data (data, dec->bytes_total);
+  memcpy (buffer->data, s->buffer->data, 8);
+  swfdec_buffer_unref (s->buffer);
+  s->buffer = buffer;
+
   s->compressed = (sig1 == 'C');
   if (s->compressed) {
     SWFDEC_DEBUG ("compressed");
@@ -211,7 +230,11 @@ swf_parse_header1 (SwfdecSwfDecoder * s)
   } else {
     SWFDEC_DEBUG ("not compressed");
   }
+  SWFDEC_DECODER (s)->bytes_loaded = 8;
+  s->bytes_parsed = 8;
   s->state = SWFDEC_STATE_INIT2;
+  swfdec_swf_decoder_deflate (s, rest);
+  dec->data_type = SWFDEC_LOADER_DATA_SWF;
 
   return SWFDEC_STATUS_OK;
 }
@@ -219,17 +242,16 @@ swf_parse_header1 (SwfdecSwfDecoder * s)
 static int
 swf_parse_header2 (SwfdecSwfDecoder * s)
 {
-  int n;
-  SwfdecBuffer *buffer;
+  guint n;
   SwfdecRect rect;
   SwfdecDecoder *dec = SWFDEC_DECODER (s);
 
-  buffer = swfdec_buffer_queue_peek (s->input_queue, 32);
-  if (buffer == NULL) {
+  swfdec_swf_decoder_init_bits (s, &s->b);
+  n = swfdec_bits_peekbits (&s->b, 5);
+  /*  rect      rate + total_frames */
+  n = ((5 + 4 * n + 7) / 8 + (2 + 2)) * 8;
+  if (swfdec_bits_left (&s->b) < n)
     return SWFDEC_STATUS_NEEDBITS;
-  }
-
-  swfdec_bits_init (&s->b, buffer);
 
   swfdec_bits_get_rect (&s->b, &rect);
   if (rect.x0 != 0.0 || rect.y0 != 0.0)
@@ -241,102 +263,80 @@ swf_parse_header2 (SwfdecSwfDecoder * s)
   swfdec_bits_syncbits (&s->b);
   dec->rate = swfdec_bits_get_u16 (&s->b);
   if (dec->rate == 0) {
-    SWFDEC_INFO ("rate is 0, setting to 1");
-    dec->rate = 1;
+    SWFDEC_INFO ("rate is 0, setting to 65536");
+    dec->rate = 65536;
   }
   SWFDEC_LOG ("rate = %g", dec->rate / 256.0);
   dec->frames_total = swfdec_bits_get_u16 (&s->b);
   SWFDEC_LOG ("n_frames = %d", dec->frames_total);
-
-  n = s->b.ptr - s->b.buffer->data;
-  swfdec_buffer_unref (buffer);
-
-  buffer = swfdec_buffer_queue_pull (s->input_queue, n);
-  swfdec_buffer_unref (buffer);
-
   swfdec_sprite_set_n_frames (s->main_sprite, dec->frames_total, dec->rate);
 
-  s->state = SWFDEC_STATE_PARSETAG;
+  swfdec_swf_decoder_flush_bits (s, &s->b);
+
+  s->state = SWFDEC_STATE_PARSE_FIRST_TAG;
   return SWFDEC_STATUS_INIT;
 }
 
 static SwfdecStatus
-swfdec_swf_decoder_parse (SwfdecDecoder *dec)
+swfdec_swf_decoder_parse_one (SwfdecSwfDecoder *s)
 {
-  SwfdecSwfDecoder *s = SWFDEC_SWF_DECODER (dec);
   int ret = SWFDEC_STATUS_OK;
-  SwfdecBuffer *buffer;
 
   s->b = s->parse;
-  g_assert (dec->player);
 
   switch (s->state) {
     case SWFDEC_STATE_INIT1:
       ret = swf_parse_header1 (s);
       break;
     case SWFDEC_STATE_INIT2:
-      if (!swfdec_swf_decoder_deflate_all (s))
-	return SWFDEC_STATUS_ERROR;
       ret = swf_parse_header2 (s);
       break;
-    case SWFDEC_STATE_PARSETAG:
+    case SWFDEC_STATE_PARSE_FIRST_TAG:
+    case SWFDEC_STATE_PARSE_TAG:
     {
       guint header_length;
       guint x;
       SwfdecTagFunc func;
       guint tag;
       guint tag_len;
-
-      if (!swfdec_swf_decoder_deflate_all (s))
-	return SWFDEC_STATUS_ERROR;
+      SwfdecBits bits;
 
       /* we're parsing tags */
-      buffer = swfdec_buffer_queue_peek (s->input_queue, 2);
-      if (buffer == NULL) {
+      swfdec_swf_decoder_init_bits (s, &bits);
+      if (swfdec_bits_left (&bits) < 2 * 8)
 	return SWFDEC_STATUS_NEEDBITS;
-      }
-      swfdec_bits_init (&s->b, buffer);
 
-      x = swfdec_bits_get_u16 (&s->b);
+      x = swfdec_bits_get_u16 (&bits);
       tag = (x >> 6) & 0x3ff;
       SWFDEC_DEBUG ("tag %d %s", tag, swfdec_swf_decoder_get_tag_name (tag));
       tag_len = x & 0x3f;
       if (tag_len == 0x3f) {
-	swfdec_buffer_unref (buffer);
-	buffer = swfdec_buffer_queue_peek (s->input_queue, 6);
-	if (buffer == NULL) {
+	if (swfdec_bits_left (&bits) < 4 * 8)
 	  return SWFDEC_STATUS_NEEDBITS;
-	}
-	swfdec_bits_init (&s->b, buffer);
 
-	swfdec_bits_get_u16 (&s->b);
-	tag_len = swfdec_bits_get_u32 (&s->b);
+	tag_len = swfdec_bits_get_u32 (&bits);
 	header_length = 6;
       } else {
 	header_length = 2;
       }
-      swfdec_buffer_unref (buffer);
 
       SWFDEC_INFO ("parsing at %d, tag %d %s, length %d",
-	  swfdec_buffer_queue_get_offset (s->input_queue), tag,
+	  s->bytes_parsed, tag,
 	  swfdec_swf_decoder_get_tag_name (tag), tag_len);
 
-      if (swfdec_buffer_queue_get_depth (s->input_queue) < tag_len +
-	  header_length) {
+      if (swfdec_bits_left (&bits) / 8 < tag_len)
 	return SWFDEC_STATUS_NEEDBITS;
-      }
 
-      buffer = swfdec_buffer_queue_pull (s->input_queue, header_length);
-      swfdec_buffer_unref (buffer);
+      swfdec_bits_init_bits (&s->b, &bits, tag_len);
+      swfdec_swf_decoder_flush_bits (s, &bits);
 
-      if (tag_len > 0)
-	buffer = swfdec_buffer_queue_pull (s->input_queue, tag_len);
-      else
-	buffer = NULL;
-      swfdec_bits_init (&s->b, buffer);
       func = swfdec_swf_decoder_get_tag_func (tag);
       if (tag == 0) {
 	s->state = SWFDEC_STATE_EOF;
+      } else if ((swfdec_swf_decoder_get_tag_flag (tag) & SWFDEC_TAG_FIRST_ONLY) 
+	  && s->state == SWFDEC_STATE_PARSE_TAG) {
+	SWFDEC_WARNING ("tag %d %s must be first tag in file, ignoring",
+	    tag, swfdec_swf_decoder_get_tag_name (tag));
       } else if (func == NULL) {
 	SWFDEC_WARNING ("tag function not implemented for %d %s",
 	    tag, swfdec_swf_decoder_get_tag_name (tag));
@@ -345,39 +345,60 @@ swfdec_swf_decoder_parse (SwfdecDecoder *dec)
 	ret = func (s, tag);
 	s->parse_sprite = NULL;
 
-	swfdec_bits_syncbits (&s->b);
 	if (swfdec_bits_left (&s->b)) {
 	  SWFDEC_WARNING
 	      ("early finish (%d bytes) at %d, tag %d %s, length %d",
 	      swfdec_bits_left (&s->b) / 8,
-	      swfdec_buffer_queue_get_offset (s->input_queue), tag,
+	      s->bytes_parsed, tag,
 	      swfdec_swf_decoder_get_tag_name (tag), tag_len);
 	}
       } else {
 	ret = SWFDEC_STATE_EOF;
 	SWFDEC_ERROR ("data after last frame");
       }
+      s->state = SWFDEC_STATE_PARSE_TAG;
 
-
-      if (buffer)
-	swfdec_buffer_unref (buffer);
-
-    }
       break;
+    }
     case SWFDEC_STATE_EOF:
-      if (swfdec_buffer_queue_get_depth (s->input_queue) > 0) {
-	SWFDEC_WARNING ("%u bytes after EOF", swfdec_buffer_queue_get_depth (s->input_queue));
-	swfdec_buffer_queue_clear (s->input_queue);
+      if (s->bytes_parsed < SWFDEC_DECODER (s)->bytes_loaded) {
+	SWFDEC_WARNING ("%u bytes after EOF", SWFDEC_DECODER (s)->bytes_loaded - s->bytes_parsed);
       }
       return SWFDEC_STATUS_EOF;
+    default:
+      g_assert_not_reached ();
   }
 
   /* copy state */
-  dec->frames_loaded = s->main_sprite->parse_frame;
+  SWFDEC_DECODER (s)->frames_loaded = s->main_sprite->parse_frame;
 
   return ret;
 }
 
+static SwfdecStatus
+swfdec_swf_decoder_parse (SwfdecDecoder *dec, SwfdecBuffer *buffer)
+{
+  SwfdecSwfDecoder *s = SWFDEC_SWF_DECODER (dec);
+  SwfdecStatus status = 0;
+
+  swfdec_swf_decoder_deflate (s, buffer);
+  do {
+    status |= swfdec_swf_decoder_parse_one (s);
+  } while ((status & (SWFDEC_STATUS_EOF | SWFDEC_STATUS_NEEDBITS | SWFDEC_STATUS_ERROR)) == 0);
+  return status;
+}
+
+static SwfdecStatus
+swfdec_swf_decoder_eof (SwfdecDecoder *dec)
+{
+  if (dec->bytes_loaded < dec->bytes_total) {
+    SWFDEC_ERROR ("only %u of %u bytes provided, broken transmission?",
+	dec->bytes_loaded, dec->bytes_total);
+  }
+
+  return 0;
+}
+
 static void
 swfdec_swf_decoder_class_init (SwfdecSwfDecoderClass *class)
 {
@@ -387,6 +408,7 @@ swfdec_swf_decoder_class_init (SwfdecSwfDecoderClass *class)
   object_class->dispose = swfdec_swf_decoder_dispose;
 
   decoder_class->parse = swfdec_swf_decoder_parse;
+  decoder_class->eof = swfdec_swf_decoder_eof;
 }
 
 static void
@@ -396,7 +418,6 @@ swfdec_swf_decoder_init (SwfdecSwfDecoder *s)
 
   s->characters = g_hash_table_new_full (g_direct_hash, g_direct_equal, 
       NULL, g_object_unref);
-  s->input_queue = swfdec_buffer_queue_new ();
   s->scripts = g_hash_table_new_full (g_direct_hash, g_direct_equal,
       NULL, (GDestroyNotify) swfdec_script_unref);
 }
@@ -445,32 +466,6 @@ swfdec_swf_decoder_create_character (SwfdecSwfDecoder * s, guint id, GType type)
   return result;
 }
 
-void
-swfdec_swf_decoder_add_root_action (SwfdecSwfDecoder *s,
-    SwfdecRootActionType type, gpointer data)
-{
-  SwfdecSprite *sprite;
-  GArray *array;
-  SwfdecRootAction action;
-
-  g_return_if_fail (SWFDEC_IS_SWF_DECODER (s));
-  sprite = s->main_sprite;
-  g_return_if_fail (sprite->parse_frame < sprite->n_frames);
-
-  if (s->root_actions == NULL)
-    s->root_actions = g_new0 (GArray *, sprite->n_frames);
-
-  array = s->root_actions[sprite->parse_frame];
-  if (array == NULL) {
-    s->root_actions[sprite->parse_frame] = 
-      g_array_new (FALSE, FALSE, sizeof (SwfdecRootAction));
-    array = s->root_actions[sprite->parse_frame];
-  }
-  action.type = type;
-  action.data = data;
-  g_array_append_val (array, action);
-}
-
 void
 swfdec_swf_decoder_add_script (SwfdecSwfDecoder *s, SwfdecScript *script)
 {
@@ -478,7 +473,7 @@ swfdec_swf_decoder_add_script (SwfdecSwfDecoder *s, SwfdecScript *script)
   g_return_if_fail (script != NULL);
   g_return_if_fail (script->buffer != NULL);
 
-  g_hash_table_insert (s->scripts, script->buffer->data, script);
+  g_hash_table_insert (s->scripts, (gpointer) script->main, script);
 }
 
 SwfdecScript *
