@@ -35,23 +35,6 @@ G_BEGIN_DECLS
 //typedef struct _SwfdecSwfDecoder SwfdecSwfDecoder;
 typedef struct _SwfdecSwfDecoderClass SwfdecSwfDecoderClass;
 typedef int (* SwfdecTagFunc) (SwfdecSwfDecoder *, guint);
-typedef struct _SwfdecRootExportData SwfdecRootExportData;
-
-typedef enum {
-  SWFDEC_ROOT_ACTION_EXPORT,		/* contains a SwfdecExportData */
-  SWFDEC_ROOT_ACTION_INIT_SCRIPT,	/* contains a SwfdecScript */
-} SwfdecRootActionType;
-
-typedef struct _SwfdecRootAction SwfdecRootAction;
-struct _SwfdecRootAction {
-  guint type;
-  gpointer data;
-};
-
-struct _SwfdecRootExportData {
-  char *		name;
-  SwfdecCharacter *	character;
-};
 
 #define SWFDEC_TYPE_SWF_DECODER                    (swfdec_swf_decoder_get_type())
 #define SWFDEC_IS_SWF_DECODER(obj)                 (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SWFDEC_TYPE_SWF_DECODER))
@@ -62,35 +45,35 @@ struct _SwfdecRootExportData {
 
 struct _SwfdecSwfDecoder
 {
-  SwfdecDecoder decoder;
+  SwfdecDecoder		decoder;
 
-  int version;
+  int			version;
 
-  int compressed;
-  z_stream z;
-  SwfdecBuffer *uncompressed_buffer;
-  SwfdecBufferQueue *input_queue;
+  gboolean    		compressed;	/* TRUE if this is a compressed flash file */
+  z_stream		z;		/* decompressor in use or uninitialized memory */
+  SwfdecBuffer *	buffer;		/* buffer containing uncompressed data */
+  guint			bytes_parsed;	/* number of bytes that have been processed by the parser */
 
-  int state;				/* where we are in the top-level state engine */
-  SwfdecBits parse;			/* where we are in global parsing */
-  SwfdecBits b;				/* temporary state while parsing */
+  int			state;		/* where we are in the top-level state engine */
+  SwfdecBits		parse;		/* where we are in global parsing */
+  SwfdecBits		b;		/* temporary state while parsing */
 
   /* defined objects */
-  GHashTable *characters;		/* list of all objects with an id (called characters) */
-  SwfdecSprite *main_sprite;
-  SwfdecSprite *parse_sprite;
-  GArray **root_actions;		/* actions to be executed by the root sprite */
-  GHashTable *scripts;			/* buffer -> script mapping for all scripts */
+  GHashTable *		characters;   	/* list of all objects with an id (called characters) */
+  SwfdecSprite *	main_sprite;	/* the root sprite */
+  SwfdecSprite *	parse_sprite;	/* the sprite that parsed at the moment */
+  GHashTable *		scripts;      	/* buffer -> script mapping for all scripts */
 
-  gboolean protection;			/* TRUE is this file is protected and may not be edited */
-  char *password;			/* MD5'd password to open for editing or NULL if may not be opened */
+  gboolean		use_network;	/* allow network or local access */
+  gboolean		has_metadata;	/* TRUE if this file contains metadata */
+  gboolean		protection;   	/* TRUE is this file is protected and may not be edited */
+  char *		password;     	/* MD5'd password to open for editing or NULL if may not be opened */
 
-  SwfdecBuffer *jpegtables;
-  char *url;
+  SwfdecBuffer *	jpegtables;	/* jpeg tables for DefineJPEG compressed jpeg files */
 };
 
 struct _SwfdecSwfDecoderClass {
-  SwfdecDecoderClass decoder_class;
+  SwfdecDecoderClass	decoder_class;
 };
 
 GType		swfdec_swf_decoder_get_type		(void);
@@ -101,9 +84,6 @@ gpointer	swfdec_swf_decoder_create_character	(SwfdecSwfDecoder *	s,
 							 guint	        	id,
 							 GType			type);
 
-void		swfdec_swf_decoder_add_root_action	(SwfdecSwfDecoder *	s,
-							 SwfdecRootActionType	type,
-							 gpointer		data);
 void		swfdec_swf_decoder_add_script		(SwfdecSwfDecoder *	s,
 							 SwfdecScript *		script);
 SwfdecScript *	swfdec_swf_decoder_get_script		(SwfdecSwfDecoder *	s,
