@@ -32,14 +32,15 @@
 #include "swfdec_bits.h"
 #include "swfdec_button.h"
 #include "swfdec_debug.h"
-#include "swfdec_edittext.h"
+#include "swfdec_text_field.h"
+#include "swfdec_filter.h"
 #include "swfdec_font.h"
 #include "swfdec_image.h"
 #include "swfdec_morphshape.h"
 #include "swfdec_movie.h" /* for SwfdecContent */
 #include "swfdec_pattern.h"
 #include "swfdec_player_internal.h"
-#include "swfdec_script.h"
+#include "swfdec_script_internal.h"
 #include "swfdec_shape.h"
 #include "swfdec_sound.h"
 #include "swfdec_sprite.h"
@@ -326,43 +327,56 @@ swfdec_button_append_content (SwfdecButton *button, guint states, SwfdecContent
 static int
 tag_func_define_button_2 (SwfdecSwfDecoder * s, guint tag)
 {
-  SwfdecBits *bits = &s->b;
-  int id;
-  int flags;
-  int offset;
+  SwfdecBits bits;
+  int id, reserved;
+  guint length;
   SwfdecButton *button;
   char *script_name;
 
-  id = swfdec_bits_get_u16 (bits);
+  id = swfdec_bits_get_u16 (&s->b);
   button = swfdec_swf_decoder_create_character (s, id, SWFDEC_TYPE_BUTTON);
   if (!button)
     return SWFDEC_STATUS_OK;
 
   SWFDEC_LOG ("  ID: %d", id);
 
-  flags = swfdec_bits_get_u8 (bits);
-  if (flags & 0x01)
-    button->menubutton = TRUE;
-  offset = swfdec_bits_get_u16 (bits);
+  reserved = swfdec_bits_getbits (&s->b, 7);
+  button->menubutton = swfdec_bits_getbit (&s->b) ? TRUE : FALSE;
+  length = swfdec_bits_get_u16 (&s->b);
 
-  SWFDEC_LOG ("  flags = %d", flags);
-  SWFDEC_LOG ("  offset = %d", offset);
+  SWFDEC_LOG ("  reserved = %d", reserved);
+  SWFDEC_LOG ("  menu = %d", button->menubutton);
+  SWFDEC_LOG ("  length of region = %d", length);
 
-  while (swfdec_bits_peek_u8 (bits)) {
-    int reserved;
+  if (length)
+    swfdec_bits_init_bits (&bits, &s->b, length > 2 ? length - 2 : 0);
+  else
+    swfdec_bits_init_bits (&bits, &s->b, swfdec_bits_left (&s->b) / 8);
+  while (swfdec_bits_peek_u8 (&bits)) {
     guint character;
     guint depth;
     guint states;
+    gboolean has_blend_mode, has_filters;
     SwfdecContent *content;
 
-    swfdec_bits_syncbits (bits);
-    reserved = swfdec_bits_getbits (bits, 4);
-    states = swfdec_bits_getbits (bits, 4);
-    character = swfdec_bits_get_u16 (bits);
-    depth = swfdec_bits_get_u16 (bits);
+    if (s->version >= 8) {
+      reserved = swfdec_bits_getbits (&bits, 2);
+      has_blend_mode = swfdec_bits_getbit (&bits);
+      has_filters = swfdec_bits_getbit (&bits);
+      SWFDEC_LOG ("  reserved = %d", reserved);
+      SWFDEC_LOG ("  has_blend_mode = %d", has_blend_mode);
+      SWFDEC_LOG ("  has_filters = %d", has_filters);
+    } else {
+      reserved = swfdec_bits_getbits (&bits, 4);
+      has_blend_mode = 0;
+      has_filters = 0;
+      SWFDEC_LOG ("  reserved = %d", reserved);
+    }
+    states = swfdec_bits_getbits (&bits, 4);
+    character = swfdec_bits_get_u16 (&bits);
+    depth = swfdec_bits_get_u16 (&bits);
 
-    SWFDEC_LOG ("  reserved = %d", reserved);
-    SWFDEC_LOG ("states: %s%s%s%scharacter=%u layer=%u",
+    SWFDEC_LOG ("  states: %s%s%s%scharacter=%u layer=%u",
         states & (1 << SWFDEC_BUTTON_HIT) ? "HIT " : "", 
 	states & (1 << SWFDEC_BUTTON_DOWN) ? "DOWN " : "", 
         states & (1 << SWFDEC_BUTTON_OVER) ? "OVER " : "",
@@ -370,16 +384,24 @@ tag_func_define_button_2 (SwfdecSwfDecoder * s, guint tag)
 	character, depth);
     content = swfdec_content_new (depth);
 
-    swfdec_bits_get_matrix (bits, &content->transform, NULL);
+    swfdec_bits_get_matrix (&bits, &content->transform, NULL);
     content->has_transform = TRUE;
     SWFDEC_LOG ("matrix: %g %g  %g %g   %g %g",
 	content->transform.xx, content->transform.yy, 
 	content->transform.xy, content->transform.yx,
 	content->transform.x0, content->transform.y0);
-    swfdec_bits_get_color_transform (bits, &content->color_transform);
+    swfdec_bits_get_color_transform (&bits, &content->color_transform);
     content->has_color_transform = TRUE;
 
     content->graphic = swfdec_swf_decoder_get_character (s, character);
+    if (has_blend_mode) {
+      content->blend_mode = swfdec_bits_get_u8 (&bits);
+      SWFDEC_LOG ("  blend mode = %u", content->blend_mode);
+    }
+    if (has_filters) {
+      GSList *list = swfdec_filter_parse (SWFDEC_DECODER (s)->player, &bits);
+      g_slist_free (list);
+    }
     if (!SWFDEC_IS_GRAPHIC (content->graphic)) {
       SWFDEC_ERROR ("id %u does not reference a graphic, ignoring", character);
       swfdec_content_free (content);
@@ -387,24 +409,34 @@ tag_func_define_button_2 (SwfdecSwfDecoder * s, guint tag)
       swfdec_button_append_content (button, states, content);
     }
   }
-  swfdec_bits_get_u8 (bits);
+  swfdec_bits_get_u8 (&bits);
+  if (swfdec_bits_left (&bits)) {
+    SWFDEC_WARNING ("%u bytes left when parsing button records", swfdec_bits_left (&bits) / 8);
+  }
 
   script_name = g_strdup_printf ("Button%u", SWFDEC_CHARACTER (button)->id);
-  while (offset != 0) {
+  while (length != 0) {
     guint condition, key;
 
-    offset = swfdec_bits_get_u16 (bits);
-    condition = swfdec_bits_get_u16 (bits);
+    length = swfdec_bits_get_u16 (&s->b);
+    if (length)
+      swfdec_bits_init_bits (&bits, &s->b, length > 2 ? length - 2 : 0);
+    else
+      swfdec_bits_init_bits (&bits, &s->b, swfdec_bits_left (&s->b) / 8);
+    condition = swfdec_bits_get_u16 (&bits);
     key = condition >> 9;
     condition &= 0x1FF;
 
-    SWFDEC_LOG ("  offset = %d", offset);
+    SWFDEC_LOG (" length = %d", length);
 
     if (button->events == NULL)
       button->events = swfdec_event_list_new (SWFDEC_DECODER (s)->player);
-    SWFDEC_LOG ("new event for condition %u (key %u)", condition, key);
-    swfdec_event_list_parse (button->events, &s->b, s->version, condition, key,
+    SWFDEC_LOG ("  new event for condition %u (key %u)", condition, key);
+    swfdec_event_list_parse (button->events, &bits, s->version, condition, key,
 	script_name);
+    if (swfdec_bits_left (&bits)) {
+      SWFDEC_WARNING ("%u bytes left after parsing script", swfdec_bits_left (&bits) / 8);
+    }
   }
   g_free (script_name);
 
@@ -477,81 +509,17 @@ tag_func_define_button (SwfdecSwfDecoder * s, guint tag)
 static int
 tag_func_file_attributes (SwfdecSwfDecoder *s, guint tag)
 {
-  int has_metadata, use_network;
-
   if (swfdec_bits_getbits (&s->b, 3))
     SWFDEC_INFO ("reserved bits (1) aren't 0");
-  has_metadata = swfdec_bits_getbit (&s->b);
+  s->has_metadata = swfdec_bits_getbit (&s->b);
+  SWFDEC_LOG ("  has metadata: %d", s->has_metadata);
   if (swfdec_bits_getbits (&s->b, 3))
     SWFDEC_INFO ("reserved bits (2) aren't 0");
-  use_network = swfdec_bits_getbit (&s->b);
-  /* FIXME: do something useful with this data */
+  s->use_network = swfdec_bits_getbit (&s->b);
+  SWFDEC_LOG ("  use network: %d", s->use_network);
   if (swfdec_bits_getbits (&s->b, 24))
     SWFDEC_INFO ("reserved bits (3) aren't 0");
-
-  return SWFDEC_STATUS_OK;
-}
-
-static int
-tag_func_export_assets (SwfdecSwfDecoder * s, guint tag)
-{
-  SwfdecBits *bits = &s->b;
-  guint count, i;
-
-  count = swfdec_bits_get_u16 (bits);
-  SWFDEC_LOG ("exporting %u assets", count);
-  for (i = 0; i < count && swfdec_bits_left (bits); i++) {
-    guint id;
-    SwfdecCharacter *object;
-    char *name;
-    id = swfdec_bits_get_u16 (bits);
-    object = swfdec_swf_decoder_get_character (s, id);
-    name = swfdec_bits_get_string_with_version (bits, s->version);
-    if (object == NULL) {
-      SWFDEC_ERROR ("cannot export id %u as %s, id wasn't found", id, name);
-      g_free (name);
-    } else if (name == NULL) {
-      SWFDEC_ERROR ("cannot export id %u, no name was given", id);
-    } else {
-      SwfdecRootExportData *data = g_new (SwfdecRootExportData, 1);
-      data->name = name;
-      data->character = object;
-      SWFDEC_LOG ("exporting %s %u as %s", G_OBJECT_TYPE_NAME (object), id, name);
-      g_object_ref (object);
-      swfdec_swf_decoder_add_root_action (s, SWFDEC_ROOT_ACTION_EXPORT, data);
-    }
-  }
-
-  return SWFDEC_STATUS_OK;
-}
-
-static int
-tag_func_do_init_action (SwfdecSwfDecoder * s, guint tag)
-{
-  SwfdecBits *bits = &s->b;
-  guint id;
-  SwfdecSprite *sprite;
-  char *name;
-
-  id = swfdec_bits_get_u16 (bits);
-  SWFDEC_LOG ("  id = %u", id);
-  sprite = swfdec_swf_decoder_get_character (s, id);
-  if (!SWFDEC_IS_SPRITE (sprite)) {
-    SWFDEC_ERROR ("character %u is not a sprite", id);
-    return SWFDEC_STATUS_OK;
-  }
-  if (sprite->init_action != NULL) {
-    SWFDEC_ERROR ("sprite %u already has an init action", id);
-    return SWFDEC_STATUS_OK;
-  }
-  name = g_strdup_printf ("InitAction %u", id);
-  sprite->init_action = swfdec_script_new_for_context (SWFDEC_AS_CONTEXT (SWFDEC_DECODER (s)->player),
-      bits, name, s->version);
-  g_free (name);
-  if (sprite->init_action) {
-    swfdec_script_ref (sprite->init_action);
-    swfdec_swf_decoder_add_root_action (s, SWFDEC_ROOT_ACTION_INIT_SCRIPT, sprite->init_action);
-  }
+  /* initialize default security if it wasn't initialized yet */
 
   return SWFDEC_STATUS_OK;
 }
@@ -591,22 +559,22 @@ static int
 tag_func_do_action (SwfdecSwfDecoder * s, guint tag)
 {
   SwfdecScript *script;
+  SwfdecBits bits;
   char *name;
 
   name = g_strdup_printf ("Sprite%u_Frame%u", SWFDEC_CHARACTER (s->parse_sprite)->id,
       s->parse_sprite->parse_frame);
-  script = swfdec_script_new_for_context (SWFDEC_AS_CONTEXT (SWFDEC_DECODER (s)->player), &s->b, name, s->version);
+  bits = s->b;
+  script = swfdec_script_new_from_bits (&bits, name, s->version);
   g_free (name);
   if (script) {
     swfdec_swf_decoder_add_script (s, script);
-    swfdec_sprite_add_action (s->parse_sprite, tag, swfdec_buffer_ref (script->buffer));
+    tag_func_enqueue (s, tag);
   }
 
   return SWFDEC_STATUS_OK;
 }
 
-/* may appear inside DefineSprite */
-#define SPRITE 1
 struct tag_func_struct
 {
   const char *name;
@@ -614,35 +582,35 @@ struct tag_func_struct
   int flag;
 };
 static struct tag_func_struct tag_funcs[] = {
-  [SWFDEC_TAG_END] = {"End", tag_func_end, SPRITE},
-  [SWFDEC_TAG_SHOWFRAME] = {"ShowFrame", tag_func_show_frame, SPRITE},
+  [SWFDEC_TAG_END] = {"End", tag_func_end, SWFDEC_TAG_DEFINE_SPRITE },
+  [SWFDEC_TAG_SHOWFRAME] = {"ShowFrame", tag_func_show_frame, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINESHAPE] = {"DefineShape", tag_define_shape, 0},
   [SWFDEC_TAG_FREECHARACTER] = {"FreeCharacter", NULL, 0},
-  [SWFDEC_TAG_PLACEOBJECT] = {"PlaceObject", NULL, SPRITE},
-  [SWFDEC_TAG_REMOVEOBJECT] = {"RemoveObject", tag_func_enqueue, SPRITE},
+  [SWFDEC_TAG_PLACEOBJECT] = {"PlaceObject", tag_func_enqueue, SWFDEC_TAG_DEFINE_SPRITE },
+  [SWFDEC_TAG_REMOVEOBJECT] = {"RemoveObject", tag_func_enqueue, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINEBITSJPEG] = {"DefineBitsJPEG", tag_func_define_bits_jpeg, 0},
   [SWFDEC_TAG_DEFINEBUTTON] = {"DefineButton", tag_func_define_button, 0},
   [SWFDEC_TAG_JPEGTABLES] = {"JPEGTables", swfdec_image_jpegtables, 0},
   [SWFDEC_TAG_SETBACKGROUNDCOLOR] =
-      {"SetBackgroundColor", tag_func_set_background_color, 0},
+      {"SetBackgroundColor", tag_func_set_background_color, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINEFONT] = {"DefineFont", tag_func_define_font, 0},
   [SWFDEC_TAG_DEFINETEXT] = {"DefineText", tag_func_define_text, 0},
-  [SWFDEC_TAG_DOACTION] = {"DoAction", tag_func_do_action, SPRITE},
+  [SWFDEC_TAG_DOACTION] = {"DoAction", tag_func_do_action, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINEFONTINFO] = {"DefineFontInfo", tag_func_define_font_info, 0},
   [SWFDEC_TAG_DEFINESOUND] = {"DefineSound", tag_func_define_sound, 0},
-  [SWFDEC_TAG_STARTSOUND] = {"StartSound", tag_func_start_sound, SPRITE},
+  [SWFDEC_TAG_STARTSOUND] = {"StartSound", tag_func_enqueue, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINEBUTTONSOUND] =
       {"DefineButtonSound", tag_func_define_button_sound, 0},
-  [SWFDEC_TAG_SOUNDSTREAMHEAD] = {"SoundStreamHead", tag_func_sound_stream_head, SPRITE},
-  [SWFDEC_TAG_SOUNDSTREAMBLOCK] = {"SoundStreamBlock", tag_func_sound_stream_block, SPRITE},
+  [SWFDEC_TAG_SOUNDSTREAMHEAD] = {"SoundStreamHead", tag_func_sound_stream_head, SWFDEC_TAG_DEFINE_SPRITE },
+  [SWFDEC_TAG_SOUNDSTREAMBLOCK] = {"SoundStreamBlock", tag_func_sound_stream_block, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINEBITSLOSSLESS] =
       {"DefineBitsLossless", tag_func_define_bits_lossless, 0},
   [SWFDEC_TAG_DEFINEBITSJPEG2] = {"DefineBitsJPEG2", tag_func_define_bits_jpeg_2, 0},
   [SWFDEC_TAG_DEFINESHAPE2] = {"DefineShape2", tag_define_shape, 0},
   [SWFDEC_TAG_DEFINEBUTTONCXFORM] = {"DefineButtonCXForm", NULL, 0},
   [SWFDEC_TAG_PROTECT] = {"Protect", tag_func_protect, 0},
-  [SWFDEC_TAG_PLACEOBJECT2] = {"PlaceObject2", tag_func_enqueue, SPRITE},
-  [SWFDEC_TAG_REMOVEOBJECT2] = {"RemoveObject2", tag_func_enqueue, SPRITE},
+  [SWFDEC_TAG_PLACEOBJECT2] = {"PlaceObject2", tag_func_enqueue, SWFDEC_TAG_DEFINE_SPRITE },
+  [SWFDEC_TAG_REMOVEOBJECT2] = {"RemoveObject2", tag_func_enqueue, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINESHAPE3] = {"DefineShape3", tag_define_shape_3, 0},
   [SWFDEC_TAG_DEFINETEXT2] = {"DefineText2", tag_func_define_text, 0},
   [SWFDEC_TAG_DEFINEBUTTON2] = {"DefineButton2", tag_func_define_button_2, 0},
@@ -655,37 +623,42 @@ static struct tag_func_struct tag_funcs[] = {
   [SWFDEC_TAG_NAMECHARACTER] = {"NameCharacter", NULL, 0},
   [SWFDEC_TAG_SERIALNUMBER] = {"SerialNumber", NULL, 0},
   [SWFDEC_TAG_GENERATORTEXT] = {"GeneratorText", NULL, 0},
-  [SWFDEC_TAG_FRAMELABEL] = {"FrameLabel", tag_func_frame_label, SPRITE},
-  [SWFDEC_TAG_SOUNDSTREAMHEAD2] = {"SoundStreamHead2", tag_func_sound_stream_head, SPRITE},
+  [SWFDEC_TAG_FRAMELABEL] = {"FrameLabel", tag_func_frame_label, SWFDEC_TAG_DEFINE_SPRITE },
+  [SWFDEC_TAG_SOUNDSTREAMHEAD2] = {"SoundStreamHead2", tag_func_sound_stream_head, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINEMORPHSHAPE] =
       {"DefineMorphShape", tag_define_morph_shape, 0},
   [SWFDEC_TAG_DEFINEFONT2] = {"DefineFont2", tag_func_define_font_2, 0},
   [SWFDEC_TAG_TEMPLATECOMMAND] = {"TemplateCommand", NULL, 0},
   [SWFDEC_TAG_GENERATOR3] = {"Generator3", NULL, 0},
   [SWFDEC_TAG_EXTERNALFONT] = {"ExternalFont", NULL, 0},
-  [SWFDEC_TAG_EXPORTASSETS] = {"ExportAssets", tag_func_export_assets, 0},
+  [SWFDEC_TAG_EXPORTASSETS] = {"ExportAssets", tag_func_enqueue, 0},
   [SWFDEC_TAG_IMPORTASSETS] = {"ImportAssets", NULL, 0},
   [SWFDEC_TAG_ENABLEDEBUGGER] = {"EnableDebugger", NULL, 0},
-  [SWFDEC_TAG_DOINITACTION] = {"DoInitAction", tag_func_do_init_action, SPRITE},
+  [SWFDEC_TAG_DOINITACTION] = {"DoInitAction", tag_func_enqueue, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINEVIDEOSTREAM] = {"DefineVideoStream", tag_func_define_video, 0},
-  [SWFDEC_TAG_VIDEOFRAME] = {"VideoFrame", tag_func_video_frame, SPRITE},
+  [SWFDEC_TAG_VIDEOFRAME] = {"VideoFrame", tag_func_video_frame, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINEFONTINFO2] = {"DefineFontInfo2", tag_func_define_font_info, 0},
-  [SWFDEC_TAG_MX4] = {"MX4", NULL, 0},
+  [SWFDEC_TAG_DEBUGID] = {"DebugID", NULL, 0},
   [SWFDEC_TAG_ENABLEDEBUGGER2] = {"EnableDebugger2", NULL, 0},
   [SWFDEC_TAG_SCRIPTLIMITS] = {"ScriptLimits", NULL, 0},
-  [SWFDEC_TAG_SETTABINDEX] = {"SetTabIndex", NULL, 0},
-  [SWFDEC_TAG_FILEATTRIBUTES] = {"FileAttributes", tag_func_file_attributes, 0},
-  [SWFDEC_TAG_PLACEOBJECT3] = {"PlaceObject3", tag_func_enqueue, SPRITE},
+  [SWFDEC_TAG_SETTABINDEX] = {"SetTabIndex", NULL, SWFDEC_TAG_DEFINE_SPRITE },
+  [SWFDEC_TAG_FILEATTRIBUTES] = {"FileAttributes", tag_func_file_attributes, SWFDEC_TAG_FIRST_ONLY },
+  [SWFDEC_TAG_PLACEOBJECT3] = {"PlaceObject3", tag_func_enqueue, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_IMPORTASSETS2] = {"ImportAssets2", NULL, 0},
   [SWFDEC_TAG_DEFINEFONTALIGNZONES] = {"DefineFontAlignZones", NULL, 0},
   [SWFDEC_TAG_CSMTEXTSETTINGS] = {"CSMTextSettings", NULL, 0},
   [SWFDEC_TAG_DEFINEFONT3] = {"DefineFont3", tag_func_define_font_3, 0},
-  [SWFDEC_TAG_AVM2DECL] = {"AVM2Decl", NULL, 0},
+  [SWFDEC_TAG_AVM2DECL] = {"AVM2Decl", NULL, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_METADATA] = {"Metadata", NULL, 0},
   [SWFDEC_TAG_DEFINESCALINGGRID] = {"DefineScalingGrid", NULL, 0},
-  [SWFDEC_TAG_AVM2ACTION] = {"AVM2Action", NULL, 0},
+  [SWFDEC_TAG_AVM2ACTION] = {"AVM2Action", NULL, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINESHAPE4] = {"DefineShape4", tag_define_shape_4, 0},
   [SWFDEC_TAG_DEFINEMORPHSHAPE2] = {"DefineMorphShape2", NULL, 0},
+  [SWFDEC_TAG_PRIVATE_IMAGE] = { "PrivateImage", NULL, 0},
+  [SWFDEC_TAG_DEFINESCENEDATA] = { "DefineSceneData", NULL, 0},
+  [SWFDEC_TAG_DEFINEBINARYDATA] = { "DefineBinaryData", NULL, 0},
+  [SWFDEC_TAG_DEFINEFONTNAME] = { "DefineFontName", NULL, 0},
+  [SWFDEC_TAG_STARTSOUND2] = {"StartSound2", NULL, SWFDEC_TAG_DEFINE_SPRITE }
 };
 
 static const int n_tag_funcs = sizeof (tag_funcs) / sizeof (tag_funcs[0]);
