@@ -74,22 +74,48 @@ typedef enum {
   SWFDEC_TAG_DEFINEVIDEOSTREAM	  = 60,
   SWFDEC_TAG_VIDEOFRAME		  = 61,
   SWFDEC_TAG_DEFINEFONTINFO2	  = 62,
-  SWFDEC_TAG_MX4		  = 63,     /*(?) */
+  SWFDEC_TAG_DEBUGID		  = 63,
   SWFDEC_TAG_ENABLEDEBUGGER2	  = 64,
   SWFDEC_TAG_SCRIPTLIMITS	  = 65,
   SWFDEC_TAG_SETTABINDEX	  = 66,
+#if 0
+  /* magic tags that seem to be similar to FILEATTRIBUTES */
+  SWFDEC_TAG_			  = 67,
+  SWFDEC_TAG_			  = 68,
+#endif
   SWFDEC_TAG_FILEATTRIBUTES	  = 69,
   SWFDEC_TAG_PLACEOBJECT3	  = 70,
   SWFDEC_TAG_IMPORTASSETS2	  = 71,
+#if 0
+  /* seems similar to SWFDEC_TAG_AVM2DECL */
+  SWFDEC_TAG_			  = 72, /* allowed with DefineSprite */
+#endif
   SWFDEC_TAG_DEFINEFONTALIGNZONES = 73,
   SWFDEC_TAG_CSMTEXTSETTINGS	  = 74,
   SWFDEC_TAG_DEFINEFONT3	  = 75,
   SWFDEC_TAG_AVM2DECL		  = 76,
   SWFDEC_TAG_METADATA		  = 77,
   SWFDEC_TAG_DEFINESCALINGGRID	  = 78,
+#if 0
+  /* more magic tags that seem to be similar to FILEATTRIBUTES */
+  SWFDEC_TAG_			  = 80,
+  SWFDEC_TAG_			  = 81,
+#endif
   SWFDEC_TAG_AVM2ACTION		  = 82,
   SWFDEC_TAG_DEFINESHAPE4	  = 83,
-  SWFDEC_TAG_DEFINEMORPHSHAPE2    = 84
+  SWFDEC_TAG_DEFINEMORPHSHAPE2    = 84,
+  SWFDEC_TAG_PRIVATE_IMAGE    	  = 85,
+  SWFDEC_TAG_DEFINESCENEDATA  	  = 86,
+  SWFDEC_TAG_DEFINEBINARYDATA 	  = 87,
+  SWFDEC_TAG_DEFINEFONTNAME	  = 88,
+  SWFDEC_TAG_STARTSOUND2	  = 89
 } SwfdecTag;
 
+typedef enum {
+  /* tag is allowe inside DefineSprite */
+  SWFDEC_TAG_DEFINE_SPRITE = (1 << 0),
+  /* tag must be first tag */
+  SWFDEC_TAG_FIRST_ONLY = (1 << 1)
+} SwfdecTagFlag;
+
 #endif
