@@ -78,6 +78,12 @@ fscommand_cb (SwfdecPlayer *player, const char *command, const char *parameter,
   }
 }
 
+static void
+initialized_cb (SwfdecPlayer *player, GParamSpec *pspec, gulong *time_left)
+{
+  *time_left = ceil (10000 / swfdec_player_get_rate (player));
+}
+
 static void
 run_test (gpointer testp, gpointer unused)
 {
@@ -85,12 +91,12 @@ run_test (gpointer testp, gpointer unused)
   SwfdecLoader *loader;
   SwfdecPlayer *player;
   SwfdecBuffer *buffer;
-  guint time_left;
+  gulong time_left = G_MAXULONG;
   char *str;
   GString *string, *output;
   GError *error = NULL;
   gboolean quit = FALSE;
-  SwfdecInteraction *inter;
+  SwfdecInteraction *inter = NULL;
 
   output = g_string_new ("");
   g_string_append_printf (output, "Testing %s:\n", test->filename);
@@ -100,15 +106,11 @@ run_test (gpointer testp, gpointer unused)
     goto fail;
   }
   string = g_string_new ("");
-  player = swfdec_player_new ();
+  player = swfdec_player_new (NULL);
+  g_object_set (player, "memory-until-gc", (gulong) 0, NULL);
   g_signal_connect (player, "trace", G_CALLBACK (trace_cb), string);
   g_signal_connect (player, "fscommand", G_CALLBACK (fscommand_cb), &quit);
   swfdec_player_set_loader (player, loader);
-  if (!swfdec_player_is_initialized (player)) {
-    g_string_append_printf (output, "  ERROR: player is not initialized\n");
-    g_object_unref (player);
-    goto fail;
-  }
   str = g_strdup_printf ("%s.act", test->filename);
   if (g_file_test (str, G_FILE_TEST_EXISTS)) {
     inter = swfdec_interaction_new_from_file (str, &error);
@@ -121,7 +123,7 @@ run_test (gpointer testp, gpointer unused)
     }
     time_left = swfdec_interaction_get_duration (inter);
   } else {
-    time_left = ceil (10000 / swfdec_player_get_rate (player));
+    g_signal_connect (player, "notify::initialized", G_CALLBACK (initialized_cb), &time_left);
     inter = NULL;
   }
   g_free (str);
@@ -129,19 +131,25 @@ run_test (gpointer testp, gpointer unused)
   /* FIXME: Make the number of iterations configurable? */
   while (quit == FALSE) {
     /* FIXME: will not do 10 iterations if there's other stuff loaded */
-    guint advance = swfdec_player_get_next_event (player);
+    glong advance = swfdec_player_get_next_event (player);
 
     if (inter) {
       int t = swfdec_interaction_get_next_event (inter);
       g_assert (t >= 0);
-      advance = MIN (advance, (guint) t);
+      if (advance < 0)
+	advance = t;
+      else
+	advance = MIN (advance, t);
     }
-    if (advance > time_left)
+    if (advance < 0 || (guint) advance > time_left)
       break;
-    swfdec_player_advance (player, advance);
-    if (inter)
-      swfdec_interaction_advance (inter, player, advance);
     time_left -= advance;
+    swfdec_player_advance (player, advance);
+    if (inter) {
+      swfdec_interaction_advance (inter, player, advance);
+      if (time_left == 0)
+	break;
+    }
   }
   g_signal_handlers_disconnect_by_func (player, trace_cb, string);
   g_object_unref (player);
@@ -169,6 +177,7 @@ run_test (gpointer testp, gpointer unused)
 	g_error_free (error);
       } else {
 	g_string_append (output, result);
+	g_free (result);
       }
     }
     g_string_free (string, TRUE);
@@ -182,6 +191,8 @@ run_test (gpointer testp, gpointer unused)
   g_string_append (output, "  OK\n");
   test->success = TRUE;
 fail:
+  if (inter)
+    swfdec_interaction_free (inter);
   if (test->mutex)
     g_mutex_lock (test->mutex);
   test->output = g_string_free (output, FALSE);
@@ -201,6 +212,8 @@ main (int argc, char **argv)
   GError *error = NULL;
 
   g_thread_init (NULL);
+  /* by default get rid of the loads of warnings the tests produce */
+  g_setenv ("SWFDEC_DEBUG", "2", FALSE);
   swfdec_init ();
   failed_tests = g_string_new ("");
 
@@ -248,7 +261,7 @@ main (int argc, char **argv)
       Test *test = walk->data;
       
       run_test (test, NULL);
-      g_print (test->output);
+      g_print ("%s", test->output);
       if (!test->success) {
 	failures++;
 	g_string_append_printf (failed_tests, 
@@ -276,7 +289,7 @@ main (int argc, char **argv)
       Test *test = walk->data;
       while (test->output == NULL)
 	g_cond_wait (cond, mutex);
-      g_print (test->output);
+      g_print ("%s", test->output);
       if (!test->success) {
 	failures++;
 	g_string_append_printf (failed_tests, 
